//  Extinction Suite (v4.1.5)
//           by LUKAS PAYNE
//	version date 08/09/2021
//
// This macro uses "the DCRAW reader plugin," by David Coffin, to allow ImageJ to open RAW
// files.
//
// A portion is based on the BatchConvert.txt macro at http://rsb.info.nih.gov/ij/plugins/batch-converter.html,
// with significant alterations.
//
// It may be used to open RAW (or otherwise) files in ImageJ, split the color channels if desired,
// and then save opened files as TIFF, 8-bit TIFF,
// JPEG, GIF, PNG, PGM, BMP, FITS, Text Image, ZIP or "Raw"

// Following the conversion/splitting, the program may be used to average and, if desired or applicable, re-merge the
// the averaged channels.
//
// The resulting averaged images can then be put through the extinction analysis process to create
// the same number of extinction images as there are channels/polarisation cases.
//
// Finally, the extinction images may be further analysed to produce single and ensemble nanoparticle extinction data.


showMessage("Any open images should be closed prior to running the Extinction Suite");

Dialog.create("Extinction Suite Mode Selection");
Modechoices=newArray("Unpolarized","Polarized");
operation=newArray("Run Folder Mapping Module","Run Extinction Suite");
Dialog.addChoice("Choose Image Processing and Analysis Mode",Modechoices,Modechoices[0]);
Dialog.addChoice("Choose Operation",operation,operation[1]);
Dialog.show();

Extmode=Dialog.getChoice();
operationchoice=Dialog.getChoice();

if(Extmode=="Unpolarized")
	polarization=false;
if(Extmode=="Polarized")
	polarization=true;
if(operationchoice==operation[0]){
	mapping=true;
	extinctionsuite=false;
}
if(operationchoice==operation[1]){
	mapping=false;
	extinctionsuite=true;
}

if(mapping==true){
	showMessage("Choose Location in which to keep New Base folder for imaging/analysis");
	
	compdirec=getDirectory("Choose Computer Folder");
	expmode=newArray("Extinction","Extinction + Darkfield","Darkfield");
	Dialog.create("Folder Options");
	Dialog.setInsets(0,0,0);
	Dialog.addString("Input Title of New Base Folder", "e.g.2015_01_01_goldNP");
	Dialog.setInsets(10,0,0);
	Dialog.addChoice("Experiment type:",expmode,expmode[0]);
	//Dialog.addCheckbox("Will you acquire Focus brightfield images?",true);
	//Dialog.setInsets(10,0,0);
	//Dialog.addCheckbox("Will you acquire Darkfield images?",true);
	Dialog.setInsets(0,0,0);
	Dialog.addCheckbox("Will you acquire BG Images with darkfield exposure times?",true);
	color=newArray("M-Channel Grayscale Images","RAW","RGB","M-Channel Color Images");
	Dialog.setInsets(15,0,0);
	Dialog.addChoice("Choose Image Type",color,color[0]);
	Dialog.show();

	basetitle=Dialog.getString();
	exproute=Dialog.getChoice();
	darkBGopt=Dialog.getCheckbox();;
	colorchoice=Dialog.getChoice();;;
	File.makeDirectory(compdirec+basetitle);
	if(colorchoice==color[0]){
		Dialog.create("M-Channel Options");
		//Dialog.setInsets(0,30,0);
		Dialog.addNumber("Number of colour sets?", 3);
		//Dialog.setInsets(0,30,0);
		Dialog.addMessage("In the next entry, please input the desired names for\neach color set, SEPARATED by COMMAS with NO SPACES.");
		//Dialog.setInsets(0,30,0);
		Dialog.addString("Color set names","450,550,650",40);
		Dialog.show();

		numchan=Dialog.getNumber();
		colnames=Dialog.getString;
		colortitles=CSV(colnames);
	}
	if(polarization==false){
		if(exproute!=expmode[2]){
			setstrt=0;
			setend=3;
			if(exproute==expmode[1]){
				setend=4;
				if(darkBGopt==true)
					setend=5;
			}		
		}
		if(exproute==expmode[2]){
			setstrt=1;
			setend=4;
			if(darkBGopt==true)
				setend=5;
		}
		count=setend-setstrt;
		foldertitles=newArray("Focus","Reference","BG","Dark","DarkBG");
		BASEdirec=compdirec+basetitle+File.separator;
		for(i=setstrt;i<setend;i++)
			File.makeDirectory(BASEdirec+foldertitles[i]);
		if(colorchoice==color[0]){
			for(i=setstrt;i<setend;i++){
				for(j=0;j<numchan;j++)
					File.makeDirectory(BASEdirec+foldertitles[i]+File.separator+colortitles[j]);
			}
		}
	}

	if(polarization==true){
		Dialog.create("Polarization Options");
		Dialog.addNumber("Initial Angle",0);
		Dialog.addNumber("Final Focus/Defocus Angle",180);
		Dialog.addNumber("Angle Step Size",10);
		Dialog.show();

		polstart=Dialog.getNumber();
		polend=Dialog.getNumber();;
		polstep=Dialog.getNumber();;;

		polarnum=((polend-polstart)/polstep)+1;
		polnums=newArray(polarnum);
		
		for(poloop=0;poloop<polarnum;poloop++)
			polnums[poloop]=polstart+poloop*polstep; ////numeric array of polarizer angles
		polnames=polbuffer(polstart,polend,polstep); /////string array of polarizer angles
		polnames=Array.concat(polnames,"BG");
		polnames=Array.concat(polnames,"DarkBG");
		foldertitles=newArray("Focus","Reference","Dark");
		bgcount=1;
		if(exproute!=expmode[0]){
			if(darkBGopt==true)
				bgcount=2;
		}
		if(exproute!=expmode[2]){
			setstrt=0;
			if(exproute==expmode[0])
				setend=1;
			else
				setend=2;
		}
		if(exproute==expmode[2]){
			setstrt=1;
			setend=2;
		}
		for(poloop=0;poloop<polarnum+bgcount;poloop++){
			BASEdirec=compdirec+basetitle+File.separator+polnames[poloop]+File.separator;
			File.makeDirectory(BASEdirec);
			if(poloop<polarnum){
				if(exproute!=expmode[2])
					File.makeDirectory(BASEdirec+"Focus");
				File.makeDirectory(BASEdirec+"Reference");
				if(exproute!=expmode[0])
					File.makeDirectory(BASEdirec+"Dark");
			}
			if(poloop>=polarnum){
				setstrt=0;
				setend=1;
			}
			if(colorchoice==color[0]){
				for(i=setstrt;i<setend;i++){
					for(j=0;j<numchan;j++){
						if(poloop<polarnum)
							File.makeDirectory(BASEdirec+foldertitles[i]+File.separator+colortitles[j]);
						else
							File.makeDirectory(BASEdirec+File.separator+colortitles[j]);
					}
				}
			}
		}
	}
}
////////////////////////////////////////////////////end new addition
//////////////////////////////////////////////////////////////////// Array functions
function findMin(array){  //added in v4.1.4
	loc=0;
	length=array.length;
	i=0;
	while (length>0 && i<array.length){
		if(array[loc]>array[i]){
			loc=i;
			length=length-i;
		}
		i++;
	}
	return loc;
}

function CSV(str){
	num=0;
	char=",";
	len=lengthOf(str);
	indx1=indexOf(str,char);
	indx2=indx1;
	if(indx1==-1){
		num=0;
		chans=newArray(1);
		chans[0]=str;
	}
	while(indx2>-1){
		num++;
		indx2=indexOf(str,char,indx2+1);
	}
	if(num>0){
		chans=newArray(num+1);
		for(i=0;i<num;i++){
			CSI=indexOf(str,char);
			chans[i]=substring(str,0,CSI);
			str=substring(str,CSI+1);
			if(i==num-1)
				chans[i+1]=str;
		}
	}
	return chans;
}

function CSVf(str){
	num=0;
	char=",";
	len=lengthOf(str);
	indx1=indexOf(str,char);
	indx2=indx1;
	if(indx1==-1){
		num=0;
		chans=newArray(1);
		chans[0]=parseFloat(str);
	}
	while(indx2>-1){
		num++;
		indx2=indexOf(str,char,indx2+1);
	}
	if(num>0){
		chans=newArray(num+1);
		for(i=0;i<num;i++){
			CSI=indexOf(str,char);
			chans[i]=parseFloat(substring(str,0,CSI));
			str=substring(str,CSI+1);
			if(i==num-1)
				chans[i+1]=parseFloat(str);
		}
	}
	return chans;
}

function convertTo8Bit() {
	if (bitDepth==24)
		run("8-bit Color", "number=256");
	else
		run("8-bit");
}

function listrebuilder(array,value){
	numoccur=0;
	for (i=0;i<array.length;i++){
		if(endsWith(array[i],value))
			numoccur++;
	}
	if (numoccur==0){
		valueposition=newArray(1);
		valueposition[0]=0;
		return valueposition;
	}
	if (numoccur>0){
		valueposition=newArray(numoccur);
		newlist=newArray(numoccur);
		numoccur=0;
		for(i=0;i<array.length;i++){
			if(endsWith(array[i],value)){
				valueposition[numoccur]=i;
				numoccur++;
			}
		}
		for(i=0;i<numoccur;i++)
			newlist[i]=array[valueposition[i]];
		return newlist;
	}
}

function listRebwExt(array,value,char,exclude,inverse,extension){
	numoccur=0;
	for (i=0;i<array.length;i++){
		if(exclude==false){
			if(endsWith(array[i],extension)==true) 
				numoccur++;
		}
		if(char!=""){
			if(exclude==true && inverse==false){
				if((endsWith(array[i],extension)==true) && ((indexOf(array[i],value+char)==-1) && (indexOf(array[i],char+value)==-1)))
					numoccur++;
			}
			if(exclude==true && inverse==true){
				if((endsWith(array[i],extension)==true) && ((indexOf(array[i],value+char)>-1) || (indexOf(array[i],char+value)>-1)))
					numoccur++;
			}
		}
		else{
			title=getNCleanTitle(array[i]);
			if(exclude==true && inverse==false){
				if((endsWith(array[i],extension)==true) && (indexOf(array[i],value)==-1) || (lengthOf(value)!=lengthOf(title)))
					numoccur++;
			}
			if(exclude==true && inverse==true){
				if((endsWith(array[i],extension)==true) && (indexOf(array[i],value)>-1) && (lengthOf(value)==lengthOf(title)))
					numoccur++;
			}
		}
	}
	if (numoccur==0){
		valueposition=newArray(1);
		valueposition[0]=0;
		return valueposition;
	}
	if (numoccur>0){
		valueposition=newArray(numoccur);
		newlist=newArray(numoccur);
		numoccur=0;
		for(i=0;i<array.length;i++){
			if(exclude==false){
				if(endsWith(array[i],extension)==true){
					valueposition[numoccur]=i;
					numoccur++;
				}
			}
			if(char!=""){
				if(exclude==true && inverse==false){
					if((endsWith(array[i],extension)==true) && (indexOf(array[i],value+char)==-1) && (indexOf(array[i],char+value)==-1)){
						valueposition[numoccur]=i;
						numoccur++;
					}
				}
				if(exclude==true && inverse==true){
					if((endsWith(array[i],extension)==true) && ((indexOf(array[i],value+char)>-1) || (indexOf(array[i],char+value)>-1))){
						valueposition[numoccur]=i;
						numoccur++;
					}
				}
			}
			else{
				title=getNCleanTitle(array[i]);
				if(exclude==true && inverse==false){
					if((endsWith(array[i],extension)==true) && (indexOf(array[i],value)==-1) || lengthOf(value)!=lengthOf(title)){
						valueposition[numoccur]=i;
						numoccur++;
					}
				}
				if(exclude==true && inverse==true){
					if((endsWith(array[i],extension)==true) && (indexOf(array[i],value)>-1) && (lengthOf(value)==lengthOf(title))){
						valueposition[numoccur]=i;
						numoccur++;
					}
				}
			}
		}
		for(i=0;i<numoccur;i++)
			newlist[i]=array[valueposition[i]];
		return newlist;
	}
}

function subind(outer,inner,reps){
	sumrep=0;
	ind1=newArray(outer*inner);
	ind2=newArray(outer*inner);
	for(ii=0;ii<outer;ii++){
		for(jj=0;jj<inner;jj++){
			nn=jj+1;
			ind1[jj+ii*inner]=inner*sumrep+(nn-1)*reps[ii]+1;
			ind2[jj+ii*inner]=inner*sumrep+nn*reps[ii];
		}
		sumrep=sumrep+reps[ii];
	}
	return Array.concat(ind1,ind2);
}

function avgbgst(stack,name){
	selectWindow(stack);
	run("Z Project...", "projection=[Average Intensity]");
	rename(name);
	close(stack);
	return name;
}

function avgsubs(stack,ind1,ind2,outname){
	len=ind1.length;
	for(ii=0;ii<len;ii++){
		selectWindow(stack);
		run("Make Substack...", "channels=1 slices="+ind1[ii]+"-"+ind2[ii]);
		rename(ii);
		avgbgst(ii,outname+"_"+ii);
	}
	close(stack);
	return 0;
}

function arrindstr(value,len){
	newarr=newArray(len);
	for(i=0;i<len;i++){
		newarr[i]=value+"_"+i;
	}
	return newarr;
}

function arrfill(length,value){
	array=newArray(length);
	for(l=0;l<length;l++)
		array[l]=value;
	return array;
}

function listparse(stringarr){
	len=stringarr.length;
	newarr=newArray(len);
	for(ii=0;ii<len;ii++){
		newarr[ii]=parseInt(stringarr[ii]);
	}
	return newarr;
}

function syncext(temp,len,list,cname,char,pname,savepath,timepoints,timepath){
	newtitle="Extinction_"+cname+char+pname;
	for(i=0;i<len;i++){
		for(j=0;j<2;j++){
			open(temp+list[4*i+j*2]);
			open(temp+list[4*i+1+j*2]);
			if(j==0){
				imageCalculator("Divide create 32-bit", list[4*i+j*2], list[4*i+1+j*2]);
			}
			if(j==1){
				imageCalculator("Divide create 32-bit", list[4*i+1+j*2], list[4*i+j*2]);
			}
			run("Multiply...", "value=-1.000000");
			run("Add...", "value=1");
			sindx=buffer(4*len,2*i+j+1);
			rename(newtitle+"_"+sindx);
			close(list[4*i+j*2]);
			close(list[4*i+1+j*2]);
			selectWindow(newtitle+"_"+sindx);
			saveAs("Tiff", timepath+newtitle+"_"+sindx);
			close();
		}
	}
	run("Image Sequence...", "dir=&timepath sort");
	rename("Extinction");
	run("Z Project...", "projection=[Average Intensity]");
	close("Extinction");
	rename(newtitle);
	saveAs("Tiff",savepath+newtitle);
	close();
	if(timepoints==false){
		nlist=getFileList(timepath);
		for(i=0;i<(4*len)/2;i++){
			File.delete(timepath+nlist[i]);
		}
		File.delete(timepath);
	}
	return 0;
}

function syncref(temp,len,list,cname,char,pname,savepath){
	for(i=0;i<len/4;i++){ 
		for(j=0;j<2;j++){
			if(j==0){
				open(temp+list[4*i+1+j*2]);
				rename("one");
			}
			if(j==1){
				open(temp+list[4*i+j*2]);
				rename("two");
			}
		}
		imageCalculator("Add create 32-bit", "one", "two");
		rename("pair"+i);
		close("one");
		close("two");
		if(i==1){
			imageCalculator("Add create 32-bit","pair"+i-1,"pair"+i);
			rename("sum");
		}
		if(i>1){
			imageCalculator(operator, "sum", "pair"+i);
			close("sum");
			close("pair"+i);
			rename("sum");
		}
	}
	newval=len/2;
	run("Divide...", "value=newval");
	title="Reference_"+cname+char+pname;
	rename(title);
	saveAs("Tiff",savepath+title);
	close();
	return 0;
}

function stdark(temp,list,cname,char,pname,refpath,savepath,timepoints,timepath){
	path=temp+list[0];
	run("Image Sequence...", "dir=&path sort");
	rename("dark");
	open(refpath+"Reference_"+cname+char+pname+".tif");
	rename("ref");
	imageCalculator("Divide create 32-bit stack","dark","ref");
	close("dark");
	close("ref");
	newtitle="Darkfield_"+cname+char+pname;
	rename(newtitle);
	if(timepoints==true){
		newtitle=newtitle+"_";
		run("Image Sequence... ", "dir=&timepath format=TIFF name="+newtitle+" use");
	}
	run("Z Project...", "projection=[Average Intensity]");
	close(newtitle);
	rename(newtitle);
	saveAs("Tiff",savepath+newtitle);
	close();
	return 0;
}

function stref(temp,list,cname,char,pname,refpath){
	path=temp+list[0];
	run("Image Sequence...", "dir=&path sort");
	rename("ref");
	run("Z Project...", "projection=[Average Intensity]");
	close("ref");
	newtitle="Reference_"+cname+char+pname;
	rename(newtitle);
	saveAs("Tiff",refpath+newtitle);
	close();
	return 0;
}

function rebuildmatching(array,refarr){
	numoccur=0;
	for(i=0;i<array.length;i++){
		if(refarr[i]==true)
			numoccur++;
	}
	if(numoccur==0){
		newlist=newArray(1);
		return newlist;
	}
	if(numoccur>0){
		valueposition=newArray(numoccur);
		newlist=newArray(numoccur);
		numoccur=0;
		for(i=0;i<array.length;i++){
			if(refarr[i]==true){
				valueposition[numoccur]=i;
				numoccur++;
			}
		}
		for(i=0;i<numoccur;i++)
			newlist[i]=array[valueposition[i]];
		return newlist;
	}
}

function listfill(array,value){
	for(l=0;l<array.length;l++)
		array[l]=value;
	return array;
}

function stripchar(array,char){
	new=newArray(array.length);
	for(i=0;i<array.length;i++)
		new[i]=replace(array[i],char,"");
	return new;
}

function getExtension(){
	t1=getTitle();
	indexten=lastIndexOf(t1,".");
	extension=substring(t1,indexten);
	return extension;
}

function getCleanTitle(){
	t1=getTitle();
	indexten=lastIndexOf(t1,".");
	cleantitle=substring(t1,0,indexten);
	return cleantitle;
}

function getNCleanTitle(name){
	indexten=lastIndexOf(name,".");
	cleantitle=substring(name,0,indexten);
	return cleantitle;
}

function getCleanSliceTitle(){
	t1=getInfo("slice.label");
	indexten=lastIndexOf(t1,".");
	cleantitle=substring(t1,0,indexten);
	return cleantitle;
}

function getNewStr(str,char1,char2){
	char1in=indexOf(str,char1)+1;
	char2in=indexOf(str,char2);
	newstr=substring(str,char1in,char2in);
	return newstr;
}

function findmatchindex(str,refarr){
	for(z=0;z<refarr.length;z++){
		if(indexOf(str,refarr[z])!=-1)
			refindex=z;
	}
	return refindex;
}

function charcount(str,char){
	num=0;
	len=lengthOf(str);
	indx1=indexOf(str, char);
	while(indx1>-1){
		num++;
		//str=substring(str,indx+1,len);
		//len=lengthOf(str);
		indx1=indexOf(str,char,indx1+1);
	}
	return num+1;
}

function duallistrebuilder(array,valuearray){
	numoccur=0;
	for (i=0;i<array.length;i++){
		for(j=0;j<valuearray.length;j++){
			//if(endsWith(array[i],valuearray[j]))
			if(indexOf(array[i],valuearray[j])>-1)
				numoccur++;
		}
	}
	if (numoccur==0){
		valueposition=newArray(1);
		valueposition[0]=0;
		return valueposition;
	}
	if (numoccur>0){
		valueposition=newArray(numoccur);
		newlist=newArray(numoccur);
		numoccur=0;
		for(i=0;i<array.length;i++){
			for(j=0;j<valuearray.length;j++){
				//if(endsWith(array[i],valuearray[j])){
				if(indexOf(array[i],valuearray[j])>-1){
					valueposition[numoccur]=i;
					numoccur++;
				}
			}
		}
		for(i=0;i<numoccur;i++)
			newlist[i]=array[valueposition[i]];
		return newlist;
	}
}
/////////////////////////////////////////function to check rank order of particle measurements
function checkRanks(rankarr,valuearr){
	arlen=rankarr.length;
	sumg=0;
	suml=0;
	rankg=newArray(arlen);
	rankl=newArray(arlen);
	for(i=0;i<arlen;i++){
		numg=0;
		for(j=0;j<arlen;j++)
			if(rankarr[i]<rankarr[j])
				numg++;
		rankg[i]=numg;
		sumg+=numg;
		numl=0;
		for(j=0;j<arlen;j++)
			if(rankarr[i]>rankarr[j])
				numl++;
		rankl[i]=numl;
		suml+=numl;
	}
	indxg=newArray(sumg);
	indxl=newArray(suml);
	sumg=0;
	suml=0;
	for(i=0;i<arlen;i++){
		for(j=0;j<arlen;j++){
			if(rankarr[i]<rankarr[j]){
				indxg[sumg]=j;
				sumg++;
			}
			if(rankarr[i]>rankarr[j]){
				indxl[suml]=j;
				suml++;
			}
		}
	}
	valid=newArray(arlen);
	sumg=0;
	suml=0;
	for(i=0;i<arlen;i++){
		numg=0;
		numl=0;
		for(j=0;j<rankg[i];j++){
			if(valuearr[i]>valuearr[indxg[j+sumg]]){
				numg++;
			}
		}
		sumg+=rankg[i];
		for(j=0;j<rankl[i];j++){
			if(valuearr[i]<valuearr[indxl[j+suml]]){
				numl++;
			}
		}
		suml+=rankl[i];
		if(numg==rankg[i] && numl==rankl[i])
			valid[i]=true;
		else 
			valid[i]=false;
	}
	num=0;
	for(i=0;i<arlen;i++)
		if(valid[i]==true)
			num++;
	if(num==arlen)
		return true;
	else
		return false;
}
//////////////////////////////////function to obtain index of highest rank
function rank1(rankarr){
	arlen=rankarr.length;
	keep=true;
	holdnum=rankarr;
	count=0;
	while(keep==true){
		for(i=0;i<arlen;i++){
			holdnum[i]=holdnum[i]-1;
			if(holdnum[i]==0){
				count++;
				keep=false;
			}
		}
	}
	prime=newArray(count);
	count=0;
	for(i=0;i<arlen;i++){
		if(holdnum[i]==0){
			prime[count]=i;
			count++;
		}
	}
	return prime;
}
/////////////////////////////////////////function to determine how many times a value occurs in array
function arrayvalue(array,value){
	numoccur=0;
	for (i=0;i<array.length;i++){
		if(array[i]==value){
			numoccur++;
		}
	}
	return numoccur;
}

function arrayvalueindices(array,value){////////////////function providing an new array of the indices where a value occurs in the target array
	numoccur=0;
	for (i=0;i<array.length;i++){
		if(array[i]==value){
			numoccur++;
		}
	}
	if (numoccur==0){
		valueposition=newArray(1);
		valueposition[0]=0;
		return valueposition;
	}
	if (numoccur>0){
		valueposition=newArray(numoccur);
		numoccur=0;
		for(i=0;i<array.length;i++){
			if(array[i]==value){
				valueposition[numoccur]=i;
				numoccur++;
			}
		}
		return valueposition;
	}
}

function arraytrimmer(array,value){
	numoccur=0;
	for (i=0;i<array.length;i++){
		if(array[i]==value){
			numoccur++;
		}
	}
	if (numoccur==0){
		return array;
	}
	if (numoccur>0){
		valueantiposition=newArray(array.length-numoccur);
		numoccur1=0;
		for(i=0;i<array.length;i++){
			if(array[i]!=value){
				valueantiposition[numoccur1]=i;
				numoccur1++;
			}
		}
		trimarray=newArray(array.length-numoccur);
		for(i=0;i<array.length-numoccur;i++){
			trimarray[i]=array[valueantiposition[i]];
		}
		return trimarray;
	}
}

function countskipper(array,value){
	numoccur=0;
	for(i=0;i<array.length;i++){
		if(array[i]==value){
			numoccur++;
		}
	}
	count=newArray(array.length);
	for(i=0;i<array.length;i++){
		count[i]=i;
	}
	if(numoccur==0){
		return count;
	}
	if(numoccur>0){
		valueposition=newArray(numoccur);
		numoccur=0;
		for(i=0;i<array.length;i++){
			if(array[i]==value){
				valueposition[numoccur]=i;
				numoccur++;
			}
		}
		for(i=0;i<numoccur;i++){
			count=arraytrimmer(count,valueposition[i]);
		}
		return count;
	}
}

function fronttrim (array,number){
	R=Array.reverse(array);
	S=Array.trim(R,R.length-number);
	T=Array.reverse(S);
	return T;
}

function trimposition(array,position){
	trimarray=newArray(array.length-1);
	frontarray=newArray(position);
	backarray=newArray(array.length-position-1);
	for (i=0;i<position;i++){
		frontarray[i]=array[i];
	}
	for (i=0;i<array.length-position-1;i++){
		backarray[i]=array[i+position+1];
	}
	trimarray=Array.concat(frontarray,backarray);
	return trimarray;
}

function arraySum (array){
	sum=array[0];
	for(i=0;i<array.length-1;i++){
		sum=array[i+1]+sum;
	}
	return sum;
}

function arrayrebuild(datarray,refarray,refvalue){
	numoccur=0;
	for (i=0;i<refarray.length;i++){
		if(refarray[i]==refvalue){
			numoccur++;
		}
	}
	if (numoccur==0){
		rebuilt=newArray(1);
		rebuilt[0]=0;
		return rebuilt;
	}
	if (numoccur>0){
		rebuilt=newArray(numoccur);
		numoccur=0;
		for(i=0;i<refarray.length;i++){
			if(refarray[i]==refvalue){
				rebuilt[numoccur]=datarray[i];
				numoccur++;
			}
		}
		return rebuilt;
	}
}

function arrsectmean(start,stop,arr){
	sum=0;
	for(i=start;i<stop+1;i++)
		sum+=arr[i];
	print(sum);
	mean=sum/(stop-start+1);
	return mean;
}
//////////////////////////////////////////////////////////////////Directory specific array/list functions
function foldReject(direc,reject){
	folders=getFileList(direc);
	len=folders.length;
	tot=0;
	for(f=0;f<len;f++){
		if((folders[f]!=reject) && (folders[f]!=reject+"/")){
			tot++;
		}
	}
	cleanlist=newArray(tot);
	tot=0;
	for(f=0;f<len;f++){
		if((folders[f]!=reject) && (folders[f]!=reject+"/")){
			cleanlist[tot]=folders[f];
			tot++;
		}
	}
	return cleanlist;
}

function dirListInfo(direc){
	names=getFileList(direc);
	len=names.length;
	fold=0;
	file=0;
	for(n=0;n<len;n++){
		if(File.isDirectory(direc+names[n])==true) fold+=1;
	}
	file=len-fold;
	if(fold>file){
		list=newArray(fold);
		count=0;
		for(n=0;n<len;n++){
			if(File.isDirectory(direc+names[n])==true){
				list[count]=names[n];
				count++;
			}
		}
	}
	if(file>=fold){
		print("true");
		list=newArray(file);
		count=0;
		for(n=0;n<len;n++){
			if(File.isDirectory(direc+names[n])==false){
				list[count]=names[n];
				count++;
			}
		}
	}
	return list;
}

//////////////////////////////////////// Numeric functions
function isOdd(value){
	return value&1;
}
function isInt(value){
	if(value%1==0)
		return true;
	else
		return false;
}
function buffer(max,step){
	if(isNaN(step))
		newstr=step;
	else{
		maxdig=floor(Math.log10(abs(max*10)))-1;
		if(step==0)
			dig=0;
		else
			dig=floor(Math.log10(abs(step*10)))-1;
		buflen=maxdig-dig;
		bufstr="";
		for(num=0;num<buflen;num++)
			bufstr=bufstr+"0";
		newstr=bufstr+d2s(step,0);
	}
	return newstr;
}
function ellipseRad(xsemiax,ysemiax,theta){
	return (xsemiax*ysemiax)/sqrt(pow(cos(theta)*ysemiax,2)+pow(sin(theta)*xsemiax,2));
}
///////////////////////////////////////////////////////// Gaussian Coordinate Fitting
function gaussCoord(initialx,initialy,rad,imagename){
	radint=isInt(rad);
	if(radint==true){
		num=rad*2+1;
	}
	if(radint==false){
		radint=isInt(2*rad);
		if(radint==true){
			radodd=isOdd(2*rad);
			if(radodd==true){
				num=rad*2;
			}
			else{
				num=rad*2+1;
			}
		}
		if(radint==false){
			radodd=isOdd(round(2*rad));
			if(radodd==true){
				num=round(rad*2);
			}
			if(radodd==false){
				num=round(rad*2)+1;
			}
		}
	}
	centerR=sqrt(pow(initialx,2)+pow(initialy,2));
	x=newArray(num);
	y=newArray(num);
	xI=newArray(num);
	yI=newArray(num);
	
	selectWindow(imagename);
			
	for(i=0;i<num;i++){
		x[i]=initialx-rad+i;
		y[i]=initialy-rad+i;
		xI[i]=getPixel(initialx-rad+i,initialy);
		yI[i]=getPixel(initialx,initialy-rad+i);
	}
			
	Fit.doFit("Gaussian",x,xI);
	xoffset=Fit.p(0);
	xamp=Fit.p(1);
	xcent=Fit.p(2);
	xwidth=Fit.p(3);
	rxfit=Fit.rSquared;
					
	Fit.doFit("Gaussian",y,yI);
	yoffset=Fit.p(0);
	yamp=Fit.p(1);
	ycent=Fit.p(2);
	ywidth=Fit.p(3);
	ryfit=Fit.rSquared;
	
	selectWindow(imagename);
	
	cent=newArray(2);
	wids=newArray(2);
	centx=newArray(2);
	centy=newArray(2);
	for(j=0;j<2;j++){
		if(j==0){a=1;b=1;}
		if(j==1){a=-1;b=-1;}
		pixcoord=newArray(num);
		pixI=newArray(num);
		for(i=0;i<num;i++){
			x[i]=initialx-rad+i;
			y[i]=initialy-a*rad+b*i;
			pixcoord[i]=sqrt(pow(initialx-rad+i,2)+pow(initialy-rad+i,2));
			pixI[i]=getPixel(x[i],y[i]);
		}
		Fit.doFit("Gaussian",pixcoord,pixI);
		cent[j]=Fit.p(2);
		wids[j]=Fit.p(3);
		centx[j]=initialx-((cent[j]-centerR)/sqrt(2));
		centy[j]=initialy-((cent[j]-centerR)/sqrt(2));
	}
	centerx=(xcent+centx[0]+centx[1])/3;
	centery=(ycent+centy[0]+centy[1])/3;
	return newArray(centerx, centery, xwidth, ywidth, wids[0], wids[1]);
}
//////////////////////////////////////////////// ROI Coordinate Function
function ROIcoord(inrad,rad){
	radint=isInt(rad);
	if(radint==true){
		length=rad*2+1;
	}
	if(radint==false){
		radint=isInt(2*rad);
		if(radint==true){
			radodd=isOdd(2*rad);
			if(radodd==true){
				length=rad*2;
			}
			else{
				length=rad*2+1;
			}
		}
		if(radint==false){
			radodd=isOdd(round(2*rad));
			if(radodd==true){
				length=round(rad*2);
			}
			if(radodd==false){
				length=round(rad*2)+1;
			}
		}
	}
	r=newArray(pow(length,2));
	s=newArray(pow(length,2));
	goodpoints=0;
	for(l=0;l<length;l++){
		sum=l*(length);
		for(k=0;k<length;k++){
			r[k+sum]=round(k-rad);
			s[k+sum]=round(l-rad);
			if(inrad==0){
				if(sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))>=inrad && sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))<=rad){
					goodpoints++;
				}
			}
			else{
				if(sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))>inrad && sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))<=rad){
					goodpoints++;
				}
			}
		}
	}
	
	newarrx=newArray(goodpoints);
	newarry=newArray(goodpoints);
	goodpoints=0;
	for(l=0;l<pow(length,2);l++){
		if(inrad==0){
			if(sqrt(pow(r[l],2)+pow(s[l],2))>=inrad && sqrt(pow(r[l],2)+pow(s[l],2))<=rad){
				newarrx[goodpoints]=r[l];
				newarry[goodpoints]=s[l];
				goodpoints++;
			}
		}
		else{
			if(sqrt(pow(r[l],2)+pow(s[l],2))>inrad && sqrt(pow(r[l],2)+pow(s[l],2))<=rad){
				newarrx[goodpoints]=r[l];
				newarry[goodpoints]=s[l];
				goodpoints++;
			}
		}
	}
	r=newarrx;
	s=newarry;
	fullpix=Array.concat(r,s);
	return fullpix;
}
//////////////////////////////////////////////// ROI Extinction Measurement Function
function ROIext(x0,y0,posarray,name){
	selectWindow(name);
	ROIx=Array.slice(posarray,0,posarray.length/2);
	ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
	pixval=newArray(ROIx.length);
	for(l=0;l<ROIx.length;l++){
		pixval[l]=getPixel(x0+ROIx[l],y0+ROIy[l]);
	}
	Array.getStatistics(pixval,min,max,mean,stdDev);
	ROIdata=newArray(mean,ROIx.length);
	return ROIdata;
}
//////////////////////////////////////////////// ROI Dark Extinction Measurement Function
function dROIext(x0,y0,posarray,name){
	selectWindow(name);
	ROIx=Array.slice(posarray,0,posarray.length/2);
	ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
	pixval=newArray(ROIx.length);
	for(l=0;l<ROIx.length;l++){
		pixval[l]=1-1/(1-getPixel(x0+ROIx[l],y0+ROIy[l]));
	}
	Array.getStatistics(pixval,min,max,mean,stdDev);
	ROIdata=newArray(mean,ROIx.length);
	return ROIdata;
}
//////////////////////////////////////////////// Sigma-loop Extinction Measurement Function
function sigmaloopext(x0,y0,posarray,name,Swidth){
	selectWindow(name);
	ROIx=Array.slice(posarray,0,posarray.length/2);
	ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
	pixelvalues=newArray(ROIx.length);
	for(l=0;l<ROIx.length;l++){
		pixelvalues[l]=getPixel(x0+ROIx[l],y0+ROIy[l]);
	}
	Array.getStatistics(pixelvalues,min,max,mean,sigma);
	numpix=pixelvalues.length;
	goodcounts=0;
	for(l=0;l<numpix;l++){
		if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
			goodcounts++;
		}
	}
	pixpos=newArray(numpix);
	sigma1=1;
	sigma2=2;
	iteration=0;
	while(sigma2!=sigma1){
		pixvalues=newArray(goodcounts);
		goodcounts=0;
		for(l=0;l<numpix;l++){
			if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
				pixvalues[goodcounts]=pixelvalues[l];
				pixpos[l]=l;
				goodcounts++;
			}
			else{
				pixpos[l]=-1;
			}
		}
		sigma1=sigma;
		Array.getStatistics(pixvalues,min,max,mean,sigma);
		goodcounts=0;
		for(l=0;l<numpix;l++){
			if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
				goodcounts++;
			}
		}
		sigma2=sigma;
		iteration++;
	}
	finalmean=mean;
	quality=100*goodcounts/numpix;
	datarray=newArray(finalmean,quality);
	return datarray;
}
//////////////////////////////////////////////// Sigma-loop Dark Extinction Measurement Function
function dsigmaloopext(x0,y0,posarray,name,Swidth){
	selectWindow(name);
	ROIx=Array.slice(posarray,0,posarray.length/2);
	ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
	pixelvalues=newArray(ROIx.length);
	for(l=0;l<ROIx.length;l++){
		pixelvalues[l]=1-1/(1-getPixel(x0+ROIx[l],y0+ROIy[l]));
	}
	Array.getStatistics(pixelvalues,min,max,mean,sigma);
	numpix=pixelvalues.length;
	goodcounts=0;
	for(l=0;l<numpix;l++){
		if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
			goodcounts++;
		}
	}
	pixpos=newArray(numpix);
	sigma1=1;
	sigma2=2;
	iteration=0;
	while(sigma2!=sigma1){
		pixvalues=newArray(goodcounts);
		goodcounts=0;
		for(l=0;l<numpix;l++){
			if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
				pixvalues[goodcounts]=pixelvalues[l];
				pixpos[l]=l;
				goodcounts++;
			}
			else{
				pixpos[l]=-1;
			}
		}
		sigma1=sigma;
		Array.getStatistics(pixvalues,min,max,mean,sigma);
		goodcounts=0;
		for(l=0;l<numpix;l++){
			if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
				goodcounts++;
			}
		}
		sigma2=sigma;
		iteration++;
	}
	finalmean=mean;
	quality=100*goodcounts/numpix;
	datarray=newArray(finalmean,quality,goodcounts);
	return datarray;
}
//////////////////////////////////////////////// Double Radius with sigma-loop Extinction Measurement Function
function DRext(X,Y,posarray1,posarray2,IMAGE,areacalib,Nsigma){
	DATA1=ROIext(X,Y,posarray1,IMAGE);
	M1=DATA1[0];
	A1=DATA1[1];
		
	DATA2=sigmaloopext(X,Y,posarray2,IMAGE,Nsigma);
	M2=DATA2[0];
	
	extinction=areacalib*A1*(M1-M2);
	return extinction;
}
//////////////////////////////////////////////// Double Radius with sigma-loop Extinction Measurement Function
function dDRext(X,Y,posarray1,posarray2,IMAGE,areacalib,Nsigma){
	DATA1=dROIext(X,Y,posarray1,IMAGE);
	M1=DATA1[0];
	A1=DATA1[1];
		
	DATA2=dsigmaloopext(X,Y,posarray2,IMAGE,Nsigma);
	M2=DATA2[0];
	
	extinction=areacalib*A1*(M1-M2);
	return extinction;
}
//////////////////////////////////////////////// Sigma Loop Outer Radius Quality Check Function
function sigcheck(X,Y,posarray,IMAGE,Nsigma){	
	DATA2=sigmaloopext(X,Y,posarray,IMAGE,Nsigma);
	if(DATA2[1]>=50){
		qfac=true;
	}
	if(DATA2[1]<50){
		qfac=false;
	}
	quality=qfac;
	return quality;
}
//////////////////////////////////////////////// Single Radius (calibrated) Extinction Measurement Function
function sinext(X,Y,posarray1,IMAGE,areacalib){
	DATA1=ROIext(X,Y,posarray1,IMAGE);
	M1=DATA1[0];
	A1=DATA1[1];
	ext=areacalib*M1*A1;
	return ext;
}
//////////////////////////////////////////////// Dark Single Radius (calibrated) Extinction Measurement Function
function dsinext(X,Y,posarray1,IMAGE,areacalib){
	DATA1=dROIext(X,Y,posarray1,IMAGE);
	M1=DATA1[0];
	A1=DATA1[1];
	ext=areacalib*M1*A1;
	return ext;
}
////////////////////////////////////////////////average the two single and dark-single extinction measurements
function avgsinext(X,Y,shiftx,shifty,posarray1,IMAGE,areacalib){
	return (sinext(X,Y,posarray1,IMAGE,areacalib)/2)+(dsinext(X-shiftx,Y-shifty,posarray1,IMAGE,areacalib)/2);
}
/////////////////////////////////////////////Registration Function
function imageReg(image1, image2, noisetol1,noisetol2, shiftol, BGtype1, BGtype2, numpts){
	width=getWidth();
	height=getHeight();
	if(width>=height){
		gdim=width;
		ldim=height;
	}
	else{
		gdim=height;
		ldim=width;
	}
	fract=0.2;
	numres1=0;
	selectWindow(image1);
	while(numres1<numpts && fract*gdim<ldim){
		makeOval(0.5*gdim-0.5*fract*gdim,0.5*ldim-0.5*fract*gdim,fract*gdim,fract*gdim);
		run("Find Maxima...", "prominence="+noisetol1+" strict exclude "+BGtype1+" output=List");
		numres1=nResults;
		fract=fract+0.01;
	}
	x1=newArray(numres1);
	y1=newArray(numres1);
	for (i=0;i<numres1;i++){
		x1[i]=getResult("X",i);
		y1[i]=getResult("Y",i);
	}
	fract=0.2;
	numres2=0;
	selectWindow(image2);
	while(numres2<numpts && fract*gdim<ldim){
		makeOval(0.5*gdim-0.5*fract*gdim,0.5*ldim-0.5*fract*gdim,fract*gdim,fract*gdim);
		run("Find Maxima...", "prominence="+noisetol2+" strict exclude "+BGtype2+" output=List");
		numres2=nResults;
		fract=fract+0.01;
	}
	x2=newArray(numres2);
	y2=newArray(numres2);
	for (i=0;i<numres2;i++){
		x2[i]=getResult("X",i);
		y2[i]=getResult("Y",i);
	}
	run("Clear Results");
	//////////////Differences
	xdif1=newArray(numres1*(numres1-1)/2);
	ydif1=newArray(numres1*(numres1-1)/2);
	xdif2=newArray(numres2*(numres2-1)/2);
	ydif2=newArray(numres2*(numres2-1)/2);
	
	for(i=0;i<2;i++){
		if(i==0){xs=x1; ys=y1; xdifs=xdif1; ydifs=ydif1; N=numres1;}
		if(i==1){xs=x2; ys=y2; xdifs=xdif2; ydifs=ydif2; N=numres2;}
		for(n=0;n<N-1;n++){
			for(j=0;j<N-1-n;j++){
				xdifs[j+n*(N-1)-n*n+(n*(n+1)/2)]=xs[n]-xs[n+1+j];
				ydifs[j+n*(N-1)-n*n+(n*(n+1)/2)]=ys[n]-ys[n+1+j];
			}
		}
	}
	////////////////////////check difference matching
	if(numres1<=numres2){
		NR1=numres1;
		NR2=numres2;
		xD1=xdif1;
		yD1=ydif1;
		xD2=xdif2;
		yD2=ydif2;
	}
	else{
		NR1=numres2;
		NR2=numres1;
		xD1=xdif2;
		yD1=ydif2;
		xD2=xdif1;
		yD2=ydif1;
	}
	if(NR1>=2 && NR2>=2){ //added in v4.1.4 on 17/05/2021
		indocr=newArray(NR1*(NR1-1)/2);
		for(i=0;i<NR1*(NR1-1)/2;i++){
			num=0;
			occur=0;
			while(occur==0 &&  num<(NR2*(NR2-1)/2)){
				if((xD2[num]-2*shiftol<=xD1[i] && xD2[num]+2*shiftol>=xD1[i]) && (yD2[num]-2*shiftol<=yD1[i] && yD2[num]+2*shiftol>=yD1[i]))
					occur++;
				num++;
			}
			if(i==0){
				checker=newArray(1);
				if(occur==0)
					checker[0]=-1;
				else
					checker[0]=num-1;
			}
			else{
				inter=newArray(1);
				if(occur==0)
					inter[0]=-1;
				else
					inter[0]=num-1;
				checker=Array.concat(checker,inter);
			}
			indocr[i]=occur;
		}
		//if(occur>0){//added in v4.1.4 on 17/05/2021
			matchindex1=arrayvalueindices(indocr,1);
			matchindex2=arraytrimmer(checker,-1);
			////////////////////create reference array indicating which initial (x,y) point from which to reference using differences.
			ref1=newArray(numres1*(numres1-1)/2);
			ref2=newArray(numres2*(numres2-1)/2);
			for(i=0;i<2;i++){
				if(i==0){N=numres1; refs=ref1;}
				if(i==1){N=numres2; refs=ref2;}
				for(n=0;n<N-1;n++)
					for(j=0;j<N-1-n;j++)
						refs[j+n*(N-1)-n*n+(n*(n+1)/2)]=n;
			}
			////////////////////////find midpoints of matching differences in both sets
			xmid1=newArray(matchindex1.length);
			ymid1=newArray(matchindex1.length);
			xmid2=newArray(matchindex1.length);
			ymid2=newArray(matchindex1.length);
			if (matchindex1.length==0 || matchindex2.length==0){//added in v4.1.4 on 23/05/2021
				if(numres1>=numres2){
					NRc=numres1;
					xc1=x1;
					yc1=y1;
					xc2=x2;
					yc2=y2;
				}
				else{
					NRc=numres2;
					xc1=x2;
					yc1=y2;
					xc2=x1;
					yc2=y1;
				}
				rdist=newArray(NRc);
				for(ii=0;ii<NRc;ii++){
					rdist[ii]=sqrt(pow(xc1[ii]-xc2[0],2)+pow(yc1[ii]-yc2[0],2));
				}
				rmin=findMin(rdist);
				xcdif=xc1[rmin]-xc2[0];
				ycdif=yc1[rmin]-yc2[0];
				if(numres1<numres2){
					xcdif=-xcdif;
					ycdif=-ycdif;
				}
				shiftarr=newArray(xcdif,0,ycdif,0,0.5);
				return shiftarr;
			}
			else{//
				for(i=0;i<2;i++){
					if(i==0){
						xs=x1; ys=y1; xdifs=xdif1; ydifs=ydif1; xmids=xmid1; ymids=ymid1; refs=ref1;
						if(numres1<=numres2)
							match=matchindex1;
						else
							match=matchindex2;
					}
					if(i==1){
						xs=x2; ys=y2; xdifs=xdif2; ydifs=ydif2; xmids=xmid2; ymids=ymid2; refs=ref2;
						if(numres1<=numres2)
							match=matchindex2;
						else
							match=matchindex1;
					}
					for(n=0;n<matchindex1.length;n++){
						xmids[n]=xs[refs[match[n]]]-xdifs[match[n]]/2;
						ymids[n]=ys[refs[match[n]]]-ydifs[match[n]]/2;
					}
				}
				///////////////find shifts from matching midpoints
				xshifts=newArray(matchindex1.length);
				yshifts=newArray(matchindex1.length);
				for(i=0;i<matchindex1.length;i++){
					xshifts[i]=xmid1[i]-xmid2[i];
					yshifts[i]=ymid1[i]-ymid2[i];
				}
				if(matchindex1.length>1){//added in v4.1.4 on 17/05/2021
					Array.getStatistics(xshifts,min,max,mshiftx,stdx);
					Array.getStatistics(yshifts,min,max,mshifty,stdy);
					/////////////determine if there are any abberations in the shift data after first pass
					if(stdx<=sqrt(3)*shiftol/sqrt(xshifts.length) || stdy<=sqrt(3)*shiftol/sqrt(xshifts.length)){
						shiftarr=newArray(mshiftx,stdx,mshifty,stdy,matchindex1.length); //added final (5th) argument to returned array in v4.1.4 on 17/05/2021
						return shiftarr;
					}
					/////////////////////////////////////if there were abberations in the shift data, run second pass pattern recognition by looking at the relative midpoints
					if(stdx>sqrt(3)*shiftol/sqrt(xshifts.length) || stdy>sqrt(3)*shiftol/sqrt(xshifts.length)){
						repeat=true;
						numb=0;
						while(repeat==true && numb<xmid1.length-1){
							numg=0;
							midcheck=newArray(xmid1.length-numb);
							for(n=0;n<xmid1.length-numb-1;n++){
								xmdif1=xmid1[numb]-xmid1[numb+n+1];
								ymdif1=ymid1[numb]-ymid1[numb+n+1];
								xmdif2=xmid2[numb]-xmid2[numb+n+1];
								ymdif2=ymid2[numb]-ymid2[numb+n+1];
								if((xmdif2-sqrt(6)*shiftol<=xmdif1 && xmdif2+sqrt(6)*shiftol>=xmdif1) && (ymdif2-sqrt(6)*shiftol<=ymdif1 && ymdif2+sqrt(6)*shiftol>=ymdif1)){
									midcheck[n+1]=1;
									numg++;
								}
								else
									midcheck[n+1]=0;
							}
							midcheck[0]=1;
							numg=numg+1;
							nxref=arrayvalueindices(midcheck,1);
							nyref=arrayvalueindices(midcheck,1);
							nshiftx=newArray(nxref.length);
							nshifty=newArray(nyref.length);
							for(n=0;n<nxref.length;n++){
								nshiftx[n]=xmid1[nxref[n]]-xmid2[nxref[n]];
								nshifty[n]=ymid1[nyref[n]]-ymid2[nyref[n]];	
							}
							Array.getStatistics(nshiftx,min,max,mshiftx2,stdx2);
							Array.getStatistics(nshifty,min,max,mshifty2,stdy2);
							if(numb==xmid1.length-2){
								//print("Low number of values used to register Image:"+image2+".  Consider adjusting shift tolerance.");
								shiftarr=newArray(mshiftx2,stdx2,mshifty2,stdy2,nxref.length); //added final (5th) argument to returned array in v4.1.4 on 17/05/2021
								return shiftarr;
							}
							if(stdx2>sqrt(3)*shiftol/sqrt(nxref.length) || stdy2>sqrt(3)*shiftol/sqrt(nxref.length) || numg<round((0.5*(xmid1.length-numb-1)))){
								numb++;
							}
							else{
								repeat=false;
								shiftarr=newArray(mshiftx2,stdx2,mshifty2,stdy2,nxref.length); //added final (5th) argument to returned array in v4.1.4 on 17/05/2021
								return shiftarr;
							}
						}
					}
				}
				else{//added in v4.1.4 on 17/05/2021
					shiftarr=newArray(xshifts[0],0,yshifts[0],0,1);
					return shiftarr;
				}
			//}
			}
	}
	else if(NR1==1 || NR2==1 && NR1!=0 && NR2!=0){//added in v4.1.4 on 17/05/2021
		if(numres1>=numres2){
			NRc=numres1;
			xc1=x1;
			yc1=y1;
			xc2=x2;
			yc2=y2;
		}
		else{
			NRc=numres2;
			xc1=x2;
			yc1=y2;
			xc2=x1;
			yc2=y1;
		}
		rdist=newArray(NRc);
		for(ii=0;ii<NRc;ii++){
			rdist[ii]=sqrt(pow(xc1[ii]-xc2[0],2)+pow(yc1[ii]-yc2[0],2));
		}
		rmin=findMin(rdist);
		xcdif=xc1[rmin]-xc2[0];
		ycdif=yc1[rmin]-yc2[0];
		if(numres1<numres2){
			xcdif=-xcdif;
			ycdif=-ycdif;
		}
		shiftarr=newArray(xcdif,0,ycdif,0,0.5);
		return shiftarr;
	}
	else if(NR1==0 || NR2==0){//added in v4.1.4 on 17/05/2021
		shiftarr=newArray(0,0,0,0,0);
		return shiftarr;
	}
}
//////////////////////////////////////////////// ROI Extinction Measurement Function (for radius calibration)
function ROIextCal(x0,y0,xshift,yshift,exradius,posarray,name){
	selectWindow(name);
	ROIx=Array.slice(posarray,0,posarray.length/2);
	ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
	pixval=newArray(ROIx.length);
	for(l=0;l<ROIx.length;l++){
		if(sqrt(pow(ROIx[l]-xshift,2)+pow(ROIy[l]-yshift,2))>exradius)
			pixval[l]=getPixel(x0+ROIx[l],y0+ROIy[l]);
	}
	Array.getStatistics(pixval,min,max,mean,stdDev);
	ROIdata=newArray(mean,ROIx.length);
	return ROIdata;
}
/////////////////////////////////////////////// ROI Dark Extinction Measurement Function (for radius calibration)
function dROIextCal(x0,y0,xshift,yshift,exradius,posarray,name){
	selectWindow(name);
	ROIx=Array.slice(posarray,0,posarray.length/2);
	ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
	pixval=newArray(ROIx.length);
	for(l=0;l<ROIx.length;l++){
		if(sqrt(pow(ROIx[l]+xshift,2)+pow(ROIy[l]+yshift,2))>exradius)
			pixval[l]=1-1/(1-getPixel(x0+ROIx[l],y0+ROIy[l]));
	}
	Array.getStatistics(pixval,min,max,mean,stdDev);
	ROIdata=newArray(mean,ROIx.length);
	return ROIdata;
}
//////////////////////////////////////////////// Single Radius (calibrated) Extinction Measurement Function (radius calibration)
function sinextCal(X,Y,xshift,yshift,exradius,posarray1,IMAGE,areacalib){
	DATA1=ROIextCal(X,Y,xshift,yshift,exradius,posarray1,IMAGE);
	M1=DATA1[0];
	A1=DATA1[1];
	ext=areacalib*M1*A1;
	return ext;
}
////////////////////////////////////////////////Single Radius (calibrated) Dark extinction Measurement Function (radius calibration)
function dsinextCal(X,Y,xshift,yshift,exradius,posarray1,IMAGE,areacalib){
	DATA1=dROIextCal(X,Y,xshift,yshift,exradius,posarray1,IMAGE);
	M1=DATA1[0];
	A1=DATA1[1];
	ext=areacalib*M1*A1;
	//ext=M1;
	return ext;
}
//////////////////////////////////////////////// Radius calibration function
function radcal(path,image,x0,y0,radext,radius1,radius2,mag,pixsiz,shiftx,shifty,Nsig){
	scalecal=1000*pixsiz/mag;
	areacal=pow(scalecal,2);
	if(shiftx==0 && shifty==0)
		shift=false;
	else
		shift=true;
	//setBatchMode(true);
	open(path+image);
	ntitle=getTitle();
	run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel");
	gaussxy=gaussCoord(x0,y0,radius1,image);
	x0=gaussxy[0];
	y0=gaussxy[1];
	opt=newArray(radext);
	opt2=newArray(radext);
	extopt=newArray(radext);
	optradius=newArray(radext);
	selectWindow(ntitle);
	run("Select None");
	run("Clear Results");
	radext=round(radext);
	for(i=0;i<radext;i++){
		optrad=i/2;
		ROI1=ROIcoord(0,optrad);
		if(shift==true){
			ext1=sinextCal(x0,y0,shiftx,shifty,radius1,ROI1,ntitle,areacal);
			ext2=dsinextCal(x0+shiftx,y0+shifty,shiftx,shifty,radius1,ROI1,ntitle,areacal);
			extopt[i]=(ext1+ext2)/2;
		}
		else{
			ROI2=ROIcoord(optrad,2*optrad);
			extopt[i]=DRext(x0,y0,ROI1,ROI2,ntitle,areacal,Nsig);
		}
		optradius[i]=optrad;
	}
	selectWindow(ntitle);
	close();
	//setBatchMode(false);
	Array.getStatistics(extopt,min,max,mean,stdDev);
	maxext=max;
	Plot.create("Ext Vs. Measurement Radius", "Radius (pixels)", "Extinction (unscaled)");
	Plot.setLimits(0, radext/2, 0,maxext+0.1*maxext);
	Plot.setColor("red");
	Plot.add("circles",optradius,extopt);
	Plot.setColor("black");
	setJustification("center");
	Plot.addText("Extinction Cross-Section vs. Measurement Radius",0.5, 0.0);
	Plot.show();
	return 0;
}
//////////////////////invert extinction image
function invertext(){
	run("Multiply...", "value=-1.000000");
	run("Add...", "value=1");
	run("Reciprocal");
	run("Multiply...", "value=-1.000000");
	run("Add...", "value=1");
	return 0;
}
///////////////////determine the reference shift for different colors
function shiftRef(finpath,colinsert,chans,polinsert,polname,noisetol1,noisetol2,shiftol,numpts,useindex){////added final argument useindex on 19/04/2021
	//image1="Extinction"+colinsert+chans[0]+polinsert+polname+".tif";
	//open(finpath+image1);
	//t1=getTitle();
	if(useindex!=1000000)////added on 19/04/2021 v.4.1.3
		shiftlen=1;////
	else////
		shiftlen=chans.length;////
	refshiftx=newArray(shiftlen);//argument updated from chans.length on 19/04/2021 v.4.1.3
	refshifty=newArray(shiftlen);//
	for(i=0;i<shiftlen;i++){//
		if(useindex!=1000000)////
			runidx=useindex;////
		else////
			runidx=i;////
		image2="Extinction"+colinsert+chans[runidx]+polinsert+polname+".tif";//argument of chans[] updated from i to runidx on 19/04/2021 in v.4.1.3
		open(finpath+image2);
		t2=getTitle();
		shifts=imageReg(t2,t2,noisetol1[runidx],noisetol2[runidx],shiftol,"","light",numpts);//argument of chans[] updated from i to runidx on 19/04/2021 in v.4.1.3
		refshiftx[i]=shifts[0];
		refshifty[i]=shifts[2];
		close(t2);
	}
	//close(t1);
	return Array.concat(refshiftx,refshifty);
}
////////////////////In-band filter image for better particle selection
function inband(impath,NA,pixs,mag,lam,s){
	pixs=pixs*1000;
	scale=1000*mag/pixs;
	open(impath);
	t1=getTitle();
	run("Set Scale...", "distance=scale known=1000 unit=nm");
	run("FFT");
	t2=getTitle();
	close(t1);
	height=getHeight();
	center=height/2;
	for(i=0;i<height;i++){
		for(j=0;j<height;j++){
			pixel=sqrt(pow(i-center,2)+pow(j-center,2));
			kv=(pixel*mag)/(pixs*2048);
			kvmin=(1-s/2)*(2*NA)/lam;
			kvmax=(1+s/2)*(2*NA)/lam;
			rawpix=getPixel(i,j);
			if(kvmin<kv &&kv<kvmax){
				taper=pow(cos((kv-kvmin)*PI/(2*(kvmax-kvmin))),2)*rawpix;
				setPixel(i,j,taper);
			}
			if(kv>=kvmax) setPixel(i,j,0);
		}
	}
	run("Inverse FFT");
	rename("In-band");
	close(t2);
	return 0;
}
///////////////////Array learns from difference between old and new value of first entry
function difAdd(old,array,strt,end){
	dif=array[0]-old;
	if(dif>0)
		sym=1;
	else if(dif<0)
		sym=-1;
	else
		sym=0;
	for(i=strt;i<end;i++)
		array[i]=array[i]+sym*abs(dif);
	return array;
}
////////////////function creates x and y arrays from result list
function coordRes(numresults,coord1,coord2){
	xp=newArray(numresults);
	yp=newArray(numresults);
	for(i=0;i<numresults;i++){
		xp[i]=getResult(coord1,i);
		yp[i]=getResult(coord2,i);
	}
	return Array.concat(xp,yp);
}
//////////////function duplicates coordinate arrays and shifts them each by a given fixed value
function coordShift(numresults,x0,y0,shiftx,shifty){
	xn=newArray(numresults);
	yn=newArray(numresults);
	for(i=0;i<numresults;i++){
		xn[i]=x0[i]-shiftx;
		yn[i]=y0[i]-shifty;
	}
	return Array.concat(xn,yn);
}
////////////////////////////////////////////////function writes two equal length arrays to image with first column array1 and 2nd column array 2
function arrtoim(array1,array2,path,imagename){
	newImage(imagename,"32-bit",2,array1.length,1);
	for(i=0;i<array1.length;i++){
		setPixel(0,i,array1[i]);
		setPixel(1,i,array2[i]);
	}
	saveAs("Tiff",path+imagename);
	close();
	return 0;
}
////////////////////////////////////////////////function to create table titles following nomenclature of phenomenon name(lambda,angle)
function tablab(setstrt,setend,phenom,chans,polnames,tau,zeta,eta,aroute,amode,usescale){
	setnum=setend-setstrt;//moved here from the line above for-loop below in v4.1.2 on 26/03/2021
	cnum=chans.length;
	if(polnames.length>1)//added v4.0.7
		pnum=polnames.length-2; //added v4.0.7
	else //added v4.0.7
		pnum=polnames.length;
	titles=newArray(setnum*cnum*pnum);
	if(pnum>1)
		tpinsert=",";
	else
		tpinsert="";
	for(i=setstrt;i<setend;i++){
		if(i==1 || (aroute==amode[2] && i==0)){
			if((tau==1 && zeta==1 && eta==1) || usescale==false)
				units=" (unscaled)";
			else
				units=" (nm^2)";
		}
		else if (usescale==false)
			units=" (unscaled)";
		else
			units=" (nm^2)";
		if(aroute==amode[2])
			in=i-1;
		else
			in=i;
		for(j=0;j<cnum;j++){
			for(k=0;k<pnum;k++)
				titles[k+j*pnum+in*pnum*cnum]=phenom[i]+" ("+chans[j]+"nm"+tpinsert+polnames[k]+")"+units;
		}
	}
	return titles;
}
////////////////////////////////////////////////function to create table titles following nomenclature of fit parameter 1 (lambda), fit parameter 2(lambda), fit parameter 3(lambda), and for fits to raw data, fit parameter 4 (r^2) and fit parameter 5 (RMSE)
function ptablab(setstrt,setend,phenom,chans,polfits,polarnum,tau,zeta,eta,aroute,amode,noise,usescale){
	cnum=chans.length;
	setnum=setend-setstrt;
	titles=newArray(setnum*polfits*3);
	subscript=newArray("ext","scat");
	if(noise==true)
		params=newArray(3);
	else
		params=newArray(5);
	for(i=setstrt;i<setend;i++){
		if(i==1 || (aroute==amode[2] && i==0)){
			if((tau==1 && zeta==1 && eta==1) || usescale==false)
				units=" (unscaled)";
			else
				units=" (nm^2)";
		}
		else if (usescale==false)
			units=" (unscaled)";
		else
			units=" (nm^2)";
		if(aroute==amode[2])
			in=i-1;
		else
			in=i;
		if(noise==true){
			for(k=0;k<3;k++){
				for(l=0;l<polfits;l++){
					pname=buffer(polfits,l+1);
					params[0]="alpha_"+pname;
					params[1]="theta0_"+pname+" (degrees)";
					params[2]="mean sigma_"+subscript[i]+"_"+pname+units;
					titles[l+k*polfits+in*3*polfits]=params[k];
				}
			}
		}
		else{
			for(cs=0;cs<cnum;cs++){
				for(k=0;k<5;k++){ //changed from 3 to 5 in v4.1.4 on 20/05/2021
					params[0]="alpha at "+chans[cs]+"nm";
					params[1]="theta0 at "+chans[cs]+"nm (degrees)";
					params[2]="mean sigma_"+subscript[i]+" at "+chans[cs]+"nm"+units;
					params[3]="R^2 at "+chans[cs]+"nm";
					params[4]="RMSE at "+chans[cs]+"nm";
					titles[k+cs*5+in*5*cnum]=params[k]; // changed from 3 to 5 in v4.1.4 on 20/05/2021
				}
			}
		}
	}
	return titles;
}
///////////////////////////////////////////////function to create "0" buffered array of polarizer angles
function polbuffer(polstart,polend,polstep){
	polarnum=(polend-polstart)/polstep+1;
	polnames=newArray(polarnum);
	for(ps=0;ps<polarnum;ps++){
		polinter=buffer(polend,polstart+ps*polstep);
		polnames[ps]=polinter;
	}
	return polnames;
}
//////////////////////////////////////////////function to measure all particles in all polarizations/colors/datatypes
function dataGrab(impath,procdir,signam,coordim,newimtitle,setstrt,setend,chans,pnum,polnames,polinsert,aroute,amode,drift,refshift,areacal,Nsig,r1,r2){
	driftx=Array.slice(drift,0,drift.length/2);
	drifty=Array.slice(drift,drift.length/2,drift.length);
	refshiftx=Array.slice(refshift,0,refshift.length/2);
	refshifty=Array.slice(refshift,refshift.length/2,refshift.length);
	//pnum=polnames.length; obsolete from version 4.0.5.1 onwards
	cnum=chans.length;
	open(procdir+coordim);
	cim=getTitle();
	npart=getHeight();
	if(refshiftx.length==1 && refshiftx[0]==0 && refshifty.length==1 && refshifty[0]==0)
		rshift=false;
	else
		rshift=true;
	xcor=newArray(npart);
	ycor=newArray(npart);
	for(i=0;i<npart;i++){
		xcor[i]=getPixel(0,i);
		ycor[i]=getPixel(1,i);
	}
	close(cim);
	setnum=setend-setstrt;
	newImage(newimtitle,"32-bit",cnum*pnum,npart,setnum);
	saveim=getTitle();
	for(sets=setstrt;sets<setend;sets++){
		if(aroute==amode[2])
			in=sets-1;
		else
			in=sets;
		selectWindow(saveim);
		setSlice(in+1);
		for(cs=0;cs<cnum;cs++){
			ROI1=ROIcoord(0,r1[cs]);
			ROI2=ROIcoord(r1[cs],r2[cs]);
			for(ps=0;ps<pnum;ps++){
				nidx=ps+cs*pnum+in*cnum*pnum;
				open(impath+signam[in]+"_"+chans[cs]+polinsert+polnames[ps]+".tif");
				sim=getTitle();
				for(ns=0;ns<npart;ns++){
					selectWindow(sim);
					if(aroute!=amode[2]){
						if((rshift==false && sets==0) || sets==1) {
							crossex=DRext(xcor[ns]-driftx[nidx],ycor[ns]-drifty[nidx],ROI1,ROI2,sim,areacal,Nsig);
							if(sets==1)
								crossex=tau*zeta*crossex/eta;
						}
						if(rshift==true && sets==0)
							crossex=avgsinext(xcor[ns]-driftx[nidx],ycor[ns]-drifty[nidx],refshiftx[cs],refshifty[cs],ROI1,sim,areacal);
					}
					else{
						crossex=DRext(xcor[ns]-driftx[nidx],ycor[ns]-drifty[nidx],ROI1,ROI2,sim,areacal,Nsig);
						crossex=tau*zeta*crossex/eta;
					}
					selectWindow(saveim);
					setPixel(ps+cs*pnum,ns,crossex);
				}
				close(sim);
			}
		}
	}
	saveAs("Tiff",procdir+newimtitle);
	close();
	return 0;
}
////////////////////////////////////////////write data image to xls table
function imtoxls(procdir,coordim,datim,savpath,savname,cnum,pnum,tablabs,aroute,amode,setstrt,setend){
	open(procdir+coordim);
	tcoord=getTitle();
	open(procdir+datim);
	tdat=getTitle();
	npart=getHeight();
	selectWindow(tcoord);
  	Table.create(savname);
  	for(row=0;row<npart;row++){
  		Table.set("Data type",row,"Datapoint",savname);
  		Table.set("Datapoint index", row, row+1,savname);
  		xs=getPixel(0,row);
  		ys=getPixel(1,row);
  		Table.set("X",row,xs,savname);
  		Table.set("Y",row,ys,savname);
  	}
  	Table.set("Data type",npart,"Mean",savname);
  	Table.set("Data type",npart+1,"Standard Deviation",savname);
  	close(tcoord);
  	Table.update;
  	selectWindow(tdat);
  	for(sets=setstrt;sets<setend;sets++){
  		if(aroute==amode[2])
  			in=sets-1;
  		else
  			in=sets;
  		setSlice(in+1);
		for(cs=0;cs<cnum;cs++){
			for(ps=0;ps<pnum;ps++){
				didx=ps+cs*pnum;
				for (row=0; row<npart; row++) {
					datum=getPixel(didx,row);
					Table.set(tablabs[didx+in*cnum*pnum],row,datum,savname); //modified in v4.1.2 on 26/03/2021 changed tablabs indexing variable to didx+in*cnum*pnum
				}
				makeRectangle(didx,0,1,npart);
				getRawStatistics(nPixels, mean, min, max, std, histo);
				Table.set(tablabs[didx+in*cnum*pnum],npart,mean,savname); //modified in v4.1.2 on 26/03/2021 changed tablabs indexing variable to didx+in*cnum*pnum
				Table.set(tablabs[didx+in*cnum*pnum],npart+1,std,savname); //modified in v4.1.2 on 26/03/2021 changed tablabs indexing variable to didx+in*cnum*pnum
			}
		}
	}
	Table.update;
	Table.save(savpath+savname+".xls");
	close(savname);
	close(tdat);
	return 0;
}
///////////////////////////////////////////////////////draw custom histogram (capable of drawing one histogram from a data image, or two histograms where the second has preset characteristisc assuming it is a noise distribution (background).
function histogram(htitle,hxtitle,hytitle,vals,cts,maxs,mins,means,stds,bins,bvals,bcts,colors,usefit){
	plotwi=500;
	plothe=plotwi*9/16;
	Array.getStatistics(cts, mn, cmx, cmean, cstd);
	ydispmax=cmx;
	if(bcts.length>1){
		Array.getStatistics(bcts,bmn,bcmx,bcmean,bcstd);
		if(bcmx>cmx){
			for(i=0;i<bcts.length;i++){
				bcts[i]=bcts[i]*cmx/bcmx;
			}
		}
	}
	if(bcts.length>1 && usefit==false)//modified v4.0.7
		len=2;
	else
		len=1;
	binwidth=newArray(len);
	increment=newArray(len);
	numdraw=newArray(len);
	if(len>1){
		if(mins[1]<=mins[0])
			xdispmin=mins[1];
		else
			xdispmin=mins[0];
		if(maxs[0]>=maxs[1])
			xdispmax=maxs[0];
		else
			xdispmax=maxs[1];
	}
	else if(len==1 && usefit==true){///////////////////////////////////////////added v4.0.7
		Array.getStatistics(bcts, bmin, bmax, bmean, bstdDev);
		iniguess=newArray(bmax,0,stds[1]);
		Fit.doFit(21, bvals, bcts, iniguess);// eqn 21  is Gaussian no-offset
		gamp=Fit.p(0);
		gcen=Fit.p(1);
		gwid=Fit.p(2);
		xfit=newArray(100);
		yfit=newArray(100);
		gstrt=gcen-3*gwid;
		for(i=0;i<100;i++){
			xfit[i]=gcen-3*gwid+i*6*gwid/100+(bvals[1]-bvals[0])/2;
			yfit[i]=cmx*exp(-(xfit[i]-gcen)*(xfit[i]-gcen)/(2*gwid*gwid));
		}
		if(gcen-3*gwid<=mins[0])
			xdispmin=gcen-3*gwid;
		else
			xdispmin=mins[0];
		if(maxs[0]>=gcen+3*gwid)
			xdispmax=maxs[0];
		else
			xdispmax=gcen+3*gwid;
	}///////////////////////////////////////////////////
	else{
		xdispmin=mins[0];
		xdispmax=maxs[0];
	}
	Plot.create(htitle, hxtitle, hytitle);
	Plot.setFrameSize(plotwi,plothe);
	Plot.setLimits(xdispmin,xdispmax,0,ydispmax+0.2*ydispmax);
	for(ds=0;ds<len;ds++){
		binwidth[ds]=(maxs[ds]-mins[ds])/bins[ds]; //determine the binwidth
		increment[ds]=(maxs[ds]-mins[ds])/(2*plotwi); //find draw increment
		numdraw[ds]=floor(binwidth[ds]/increment[ds]);
		if(ds==0){
			valin=vals;
			countin=cts;
		}
		else{
			valin=bvals;
			countin=bcts;
		}
		Plot.setColor(colors[ds]);	
		for(i=0;i<bins[ds];i++){
			for(j=floor(0.1*numdraw[ds]);j<=numdraw[ds]-floor(0.1*numdraw[ds]);j++){
				x1=valin[i]+j*increment[ds];
				Plot.drawLine(x1,0,x1,countin[i]);
			}
		}
	}
	if(usefit==true){///////////added v4.0.7
		Plot.setLineWidth(2);
		Plot.setColor("gray");
		Plot.add("line",xfit, yfit);
	}/////////
	Plot.setJustification("center");
	Plot.setColor("black");
	Plot.setFontSize(14, "bold")
	Plot.addText(htitle,0.5,0);
	Plot.setJustification("right");
	Plot.addText("Mean="+means[0],0.99,0.07);
	if(bcts.length==1)
		Plot.addText("Std. Dev.= "+stds[0],0.99,0.12);
	else if(bcts.length>1 || usefit==true){//modified v4.0.7
		if(bcts.length>1){//added v4.0.7
			disent=sqrt(pow(stds[0],2)-pow(stds[1],2));//original
			dispstd=stds[1];
			dispnoise="Std. Dev of the noise=";
		}////////
		if(usefit==true){//////added v4.0.7
			disent=sqrt(pow(stds[0],2)-pow(gwid,2));
			dispstd=gwid;
			dispnoise="Std. Dev of the Gaussian fitted noise=";
		}///////////////
		Plot.addText("Sample Std. Dev. without noise= "+disent,0.99,0.12);
		Plot.addText("Sample Std. Dev. with noise= "+stds[0],0.99,0.17);
		Plot.addText(dispnoise+dispstd,0.99,0.22);//modified v4.0.7
	}
	Plot.show();
	return 0;
}
/////////////////////////////////////////////////////////////////////////////////////////////////function obtains statistical data from data image and calls histogram to display the statistics
function imtohist(procdir,datim,dstart,dwidth,pstart,pend,full,savpath,savname,htitle,hxtitle,hytitle,pcolor,savehist,closehist,slice,newbins,openim){
	if(openim==true)
		open(procdir+datim);
	tdat=getTitle();
	setSlice(slice+1);
	npart=getHeight();
	if(full==true)
		makeRectangle(dstart,0,dwidth,npart);
	else
		makeRectangle(dstart,pstart,dwidth,pend);
	getRawStatistics(npixels,mean,min,max,std,histo); //obtain statistics on the data array
	if(newbins==0){
		nBins=2*round(log(npixels)/log(2)+1); //determine the number of bins from data
		bins=newArray(nBins,0);
	}
	else
		bins=newArray(newbins,0);
	getHistogram(vals, cts, bins[0]); // determine the counts of data array for the bins
	if(openim==true)
		close(tdat);
	bvals=newArray(1);
	bcts=newArray(1);
	maxs=newArray(max,0);
	mins=newArray(min,0);
	stds=newArray(std,0);
	means=newArray(mean,0);
	colors=newArray(pcolor,"gray");
	histogram(htitle,hxtitle,hytitle,vals,cts,maxs,mins,means,stds,bins,bvals,bcts,colors,false);
	if(savehist==true)
		saveAs("png",savpath+savname);
	if(closehist==true)
		close();
	return bins[0];
}
/////////////////////////////////////////////////////////////////////////////////////////////////function obtains statistical data from data image and background data image and calls histogram to display the dual statistics
function imtohistbg(procdir,datim,bgdatim,dstart,dwidth,polarnum,colen,avginx,savpath,savname,htitle,hxtitle,hytitle,pcolor,savehist,closehist,slice,newbins){
	ims=newArray(datim,bgdatim);
	maxs=newArray(2);
	mins=newArray(2);
	means=newArray(2);
	stds=newArray(2);
	bins=newArray(2);
	colors=newArray(2);
	colors[0]=pcolor;
	colors[1]="gray";
	for(i=0;i<2;i++){
		open(procdir+ims[i]);
		tdat=getTitle();
		setSlice(slice+1);
		npart=getHeight();
		if(avginx==true)
			iavgx=avgx(procdir,tdat,procdir,"iavgx",polarnum,colen,false,false);
		makeRectangle(dstart,0,dwidth,npart);
		getRawStatistics(npixels,mean,min,max,std,histo); //obtain statistics on the data array
		if(newbins==0 || i>0)
			nBins=2*round(log(npixels)/log(2)+1); //determine the number of bins from data
		else if (newbins>0 && i==0)
			nBins=newbins;
		getHistogram(vals, cts, nBins); // determine the counts of data array for the bins
		if(avginx==true)
			close(iavgx);
		close(tdat);
		maxs[i]=max;
		mins[i]=min;
		stds[i]=std;
		means[i]=mean;
		bins[i]=nBins;
		if(i==0){
			svals=vals;
			scts=cts;
		}
		if(i==1){
			bvals=vals;
			bcts=cts;
		}
	}
	histogram(htitle,hxtitle,hytitle,svals,scts,maxs,mins,means,stds,bins,bvals,bcts,colors,true);
	if(savehist==true)
		saveAs("png",savpath+savname);
	if(closehist==true)
		close();
	return bins[0];
}
///////////////////////////////////////////////////////function which returns the range defined by 3*sigma below and above the mean from a region of pixels in an image
function imtostat(path,im,dstrt,dwidth,slice){
	open(path+im);
	t1=getTitle();
	setSlice(slice+1);
	npart=getHeight();
	makeRectangle(dstrt,0,dwidth,npart);
	getRawStatistics(nPixels, mean, min, max, std, histo);
	close(t1);
	lowlim=mean-3*std;
	uplim=mean+3*std;
	range=newArray(lowlim,uplim);
	return range;
}
/////////////////////////////////////////////////////function which returns the indices of filtered particles
function finalfilter(procdir,datim,dstrt,dwidth,lowlim,uplim,ranks,colen,polarnum,findhow,signam,aroute,amode,slice){
	open(procdir+datim);
	t1=getTitle();
	npart=getHeight();
	if(findhow==signam[0])
		sidx=0;
	else{
		if(aroute==amode[1])
			sidx=1;
		else if(aroute==amode[2])
			sidx=0;
	}
	numoccur=0;
	setSlice(slice+1);
	for(i=0;i<npart;i++){
		makeRectangle(dstrt,i,dwidth,1);
		List.setMeasurements;
		mean=List.getValue("Mean");
		if(lowlim<=mean && mean<=uplim)
			numoccur++; 
	}
	if(numoccur==0){
		finfilt=newArray(1);
		finfilt[0]=-1;
		close(t1);
		return finfilt;
	}
	else{
		ranfilt=newArray(numoccur);
		numoccur=0;
		for(i=0;i<npart;i++){
			makeRectangle(dstrt,i,dwidth,1);
			List.setMeasurements;
			mean=List.getValue("Mean");
			if(lowlim<=mean && mean<=uplim){
				ranfilt[numoccur]=i;
				numoccur++;
			}
		}
		newlen=ranfilt.length;
		numoccur=0;
		for(i=0;i<newlen;i++){
			means=newArray(colen);
			for(cs=0;cs<colen;cs++){
				makeRectangle(cs*polarnum,ranfilt[i],polarnum,1); //the 2nd argument was changed from i to ranfilt[i] in v4.1.4 fixing an error causing the filter length to be the wrong length (too long)
				List.setMeasurements;
				means[cs]=List.getValue("Mean");
			}
			check=checkRanks(ranks,means);
			if(check)
				numoccur++;
		}
		if(numoccur==0){
			finfilt=newArray(1);
			finfilt[0]=-2;
			close(t1);
			return finfilt;
		}
		else{
			finfilt=newArray(numoccur);
			numoccur=0;
			for(i=0;i<newlen;i++){
				means=newArray(colen);
				for(cs=0;cs<colen;cs++){
					makeRectangle(cs*polarnum,ranfilt[i],polarnum,1);
					List.setMeasurements;
					means[cs]=List.getValue("Mean");
				}
				check=checkRanks(ranks,means);
				if(check==true){
					finfilt[numoccur]=ranfilt[i];
					numoccur++;
				}
			}
			close(t1);
			return finfilt;
		}
	}
}
////////////////////////////////////////////a function to filter a data image removing cases via an array of indices, reducing original image height to filter.length
function imfilter(path,im,filter,savpath,savname){
	open(path+im);
	t1=getTitle();
	nexps=getWidth();
	nslice=nSlices();
	newImage(savname,"32-bit",nexps,filter.length,nslice);
	t2=getTitle();
	for(set=0;set<nslice;set++){
		selectWindow(t1);
		setSlice(set+1);
		selectWindow(t2);
		setSlice(set+1);
		for(i=0;i<filter.length;i++){
			selectWindow(t1);
			makeRectangle(0,filter[i],nexps,1);
			run("Copy");
			selectWindow(t2);
			makeRectangle(0,i,nexps,1);
			run("Paste");
		}
	}
	close(t1);
	saveAs("Tiff",savpath+savname);
	close();
	return 0;
}
////////////////////////////////////////function creates a new image compressed in x by the polarizer angle count in each color, resulting width is number of color channels. (averages polarization data).
function avgx(path,im,savpath,savname,polarnum,colen,openim,savavg){
	if(openim==true)
		open(path+im);
	t1=getTitle();
	npart=getHeight();
	nexp=getWidth();
	//nslice=nSlices(); //removed in v4.1.2 on 26/03/2021, not needed
	newImage("avgx","32-bit",colen,npart,1); //modified in v4.1.2 on 26/03/2021, changed stack size to "1" from "nslice"
	t2="avgx";
	//for(set=0;set<nslice;set++){ // removed for-loop in v4.1.2 on 26/03/2021, since data image slice number is iterated outside of function
	//selectWindow(t1);
	//setSlice(set+1);
	//selectWindow(t2);
	//setSlice(set+1);
	for(cs=0;cs<colen;cs++){
		for(i=0;i<npart;i++){
			selectWindow(t1);
			makeRectangle(cs*polarnum,i,polarnum,1);
			List.setMeasurements;
			mean=List.getValue("Mean");
			selectWindow(t2);
			setPixel(cs,i,mean);
		}
	}
	//}
	if(openim==true)
		close(t1);
	if(savavg==true)
		saveAs("Tiff",savpath+savname);
	//close(t2);
	return t2;//savname;
}
////////////////////////////////////////////function to adjust coordinate array to Gaussian fitted coordinates
function useGauss(procdir,coordim,extpath,extim,savpath,savname,radius){
	open(procdir+coordim);
	t1=getTitle();
	npart=getHeight();
	x0=newArray(npart);
	y0=newArray(npart);
	for(i=0;i<npart;i++){
		x0[i]=getPixel(0,i);
		y0[i]=getPixel(1,i);
	}
	close();
	open(extpath+extim);
	t1=getTitle();
	for(i=0;i<npart;i++){	
		xygauss=gaussCoord(x0[i],y0[i],radius,t1);
		x0[i]=xygauss[0];
		y0[i]=xygauss[1];
	}
	close();
	newImage(savname,"32-bit",2,npart,1);
	for(i=0;i<npart;i++){
		setPixel(0,i,x0[i]);
		setPixel(1,i,y0[i]);
	}
	saveAs("Tiff",savpath+savname);
	close();
	return 0;
}
////////////////////////////////////////////function to generate color gamut
function colorgen(){
	x0=newArray(-293,182,522);
	cw=newArray(114,30,90);
	arr=newArray(800-380);
	xarr=newArray(800-380);
	blue=newArray(800-380);
	green=newArray(800-380);
	red=newArray(800-380);
	for(j=0;j<(800-380);j++){
		val=100000*exp(-(j-x0[0])*(j-x0[0])/(2*cw[0]*cw[0]));
		if(val>=255)
			blue[j]=255;
		else
			blue[j]=val;
	}
	for(j=0;j<(800-380);j++){
		if(j<102){
			val=10000*exp(-(j-x0[1])*(j-x0[1])/(2*cw[1]*cw[1]));
			if(val>=255)
				green[j]=255;
			else
				green[j]=val;
		}
		else if(j>=102 && j<=210)
			green[j]=255;
		else if(j>210){
			val=10000*exp(-(j-x0[1]+214)*(j-x0[1]+214)/(2*90*90));
			if(val>=255)
				green[j]=255;
			else
				green[j]=val;
		}
	}
	for(j=0;j<(800-380);j++){
		val=100000*exp(-(j-x0[2])*(j-x0[2])/(2*cw[2]*cw[2]))+15000*exp(-(j+320)*(j+320)/(2*110*110));
		if(val>=255)
			red[j]=255;
		else
			red[j]=val;
	}
	return Array.concat(red,green,blue);
}
////////////////////////////////////////function to generate hex names from color generator
function getHex(lambda,colarr){
	red=Array.slice(colarr,0,colarr.length/3);
	green=Array.slice(colarr,colarr.length/3,2*colarr.length/3);
	blue=Array.slice(colarr,2*colarr.length/3,colarr.length);
	R=toHex(round(red[lambda-380]));
	G=toHex(round(green[lambda-380]));
	B=toHex(round(blue[lambda-380]));
	if(lengthOf(R)==1)
		R=IJ.pad(R,2);
	if(lengthOf(G)==1)
		G=IJ.pad(G,2);
	if(lengthOf(B)==1)
		B=IJ.pad(B,2);
	nhex="#"+R+G+B;
	return nhex;
}
///////////////////////////////////////////polarization fit
function polfit(fitmodel,xvals,yvals){
	Fit.doFit(fitmodel, xvals, yvals);
	afit=Fit.p(0);
	bfit=Fit.p(1);
	cfit=Fit.p(2);
	rfit=Fit.rSquared;
	if(afit<0){
		afit=-afit;
		bfit=bfit+90;
		while(bfit<0)
			bfit=bfit+180;
		while(bfit>=180)
			bfit=bfit-180;
	}
	if(afit>0){
		while(bfit<0)
			bfit=bfit+180;
		while(bfit>=180)
			bfit=bfit-180;
	}
	//yfit=newArray(xvals.length);  //This section added and commented out to be used in future release (v4.1.4 on 20/05/2021)
	//RMSEdifsq=newArray(xvals.length);
	//for(i=0;i<xvals.length;i++)
	//	yfit[i]=cfit*(1+afit*cos(2*PI*(xvals[i]-bfit)/180));
	//for(i=0;i<xvals.length;i++)
	//	RMSEdifsq[i]=pow(yfit[i]-yvals[i],2);
	//RMSEsum=arraySum(RMSEdifsq);
	//RMSE=sqrt(RMSEsum/(xvals.length));
	fitvals=newArray(afit,bfit,cfit,rfit);
	return fitvals;
}
/////////////////////////////////////////Polarization fit plots
function polarplot(fitmodel,xvals,yvals,lambda,pcolor,signam,pnum,CS,units,savpath,savname){
	fitvals=polfit(fitmodel,xvals,yvals);
	afit=fitvals[0];
	bfit=fitvals[1];
	cfit=fitvals[2];
	rfit=fitvals[3];
	xfit=newArray(181);
	yfit=newArray(181);
	for(i=0;i<181;i++){
		xfit[i]=i;
		yfit[i]=cfit*(1+afit*cos(2*PI*(xfit[i]-bfit)/180));
	}
	//Array.getStatistics(yfit, ymin, ymax, mean, stdDev); //commented out in v4.1.4 on 20/05/2021
	ymax=cfit*(1+afit); // replaced definition of ymax in v4.1.4 on 20/05/2021
	ymin=cfit*(1-afit); // replaced definition of ymin in v4.1.4 on 20/05/2021
	amp=ymax-ymin;
	RMSEdifsq=newArray(xvals.length);
	for(i=0;i<xvals.length;i++)
		RMSEdifsq[i]=pow(yfit[xvals[i]]-yvals[i],2);
	RMSEsum=arraySum(RMSEdifsq);
	RMSE=sqrt(RMSEsum/(xvals.length));
	Plot.create(signam+" vs Theta at"+lambda+"nm"+"for p"+pnum, "Polarizer angle, theta (Deg)", "sigma_"+CS+units);
	Plot.setLimits(0, 180, ymin-0.2*abs(ymin), ymax+0.2*abs(ymax));
	Plot.setFontSize(14,"bold");
	Plot.setLineWidth(2);
	Plot.setColor("gray");
	Plot.add("line",xfit, yfit);
	Plot.setColor(pcolor);
	Plot.setLineWidth(3);
	Plot.add("circles",xvals, yvals);
	Plot.setColor("black");
	setJustification("center");
	Plot.addText(signam+" cross-section vs. polarizer angle (theta)", 0.5, 0.0);
	Plot.setJustification("right");
	Plot.addText("a="+afit,0.99,0.075);
	Plot.addText("b="+bfit,0.99,0.130);
	Plot.addText("c="+cfit,0.99,0.185);
	Plot.addText("R^2="+rfit,0.99,0.240);
	Plot.addText("RMSE="+RMSE,0.99,0.295);
	Plot.addText("Amp(PtoP)="+amp,0.99,0.35);
	Plot.show();
	saveAs("PNG",savpath+savname);
	close();
	fitvals=newArray(afit,bfit,cfit,rfit,RMSE);
	return fitvals;
}
//////////////////////////////////////////////simulated noise fit function
function addnoise(polarnum,pdata,bgdat){
	randommean=newArray(polarnum);
	for(jr=0;jr<polarnum;jr++){
		randomvalue=0;
		for(kr=0;kr<12;kr++)
			randomvalue+=(random()-0.5);
		randommean[jr]=randomvalue*bgdat[jr];
	}
	randata=newArray(polarnum);
	for(jr=0;jr<polarnum;jr++)
		randata[jr]=pdata[jr]+randommean[jr];
	return randata;
}
////////////////////////////////////////function to find standard deviation of values in y direction from data image
function istdy(path,im,savpath,savname,polarnum,colen,openim,savavg){
	if(openim==true)
		open(path+im);
	t1=getTitle();
	npart=getHeight();
	nexp=getWidth();
	nslice=nSlices();
	newImage("stdy","32-bit",colen*polarnum,1,nslice);
	t2="stdy";
	for(set=0;set<nslice;set++){
		selectWindow(t1);
		setSlice(set+1);
		selectWindow(t2);
		setSlice(set+1);
		for(cs=0;cs<colen;cs++){
			for(ps=0;ps<polarnum;ps++){
				selectWindow(t1);
				makeRectangle(ps+cs*polarnum,0,1,npart);
				List.setMeasurements;
				std=List.getValue("StdDev");
				selectWindow(t2);
				setPixel(ps+cs*polarnum,0,std);
			}
		}
	}
	if(openim==true)
		close(t1);
	if(savavg==true)
		saveAs("Tiff",savpath+savname);
	//close(t2);
	return t2;//savname;
}
////////////////////////////////////////////////////function to run fit simulations and save to additional array resource
function fitim(procdir,im,bgdatim,setstrt,setend,colen,chans,polnums,signam,pcolor,plotfit,psavpath,datsav,aroute,amode,fitmodel,polfits,subscript,usescale){
	if(plotfit==false)
		tbg=istdy(procdir,bgdatim,"","",polarnum,colen,true,false);
	setnum=setend-setstrt;
	polarnum=polnums.length;
	open(procdir+im);
	t1=getTitle();
	npart=getHeight();
	yvals=newArray(polarnum);
	bgs=newArray(polarnum);
	if (polfits>1)   // added in v4.1.4 on 21/05/2021
		numparams=3; //
	else 			 //
		numparams=5; //
	newImage(datsav,"32-bit",numparams*polfits*colen,npart,2); //changed 3rd argument from 3*polfits*colen to numparams in v4.1.4 on 21/05/2021
	t2=getTitle();
	for(sets=setstrt;sets<setend;sets++){
		if(aroute!=amode[2])
			in=sets;
		else
			in=sets-1;
		subscript0=subscript[in];
		if(sets==0){
			if (usescale==true)
				units=" (nm^2)";
			else
				units=" (unscaled)";
		}
		else{
			if((tau==1 && zeta==1 && eta==1) || usescale==false)
				units=" (unscaled");
			else
				units=" (nm^2)";
		}
		selectWindow(t1);
		setSlice(in+1);
		selectWindow(t2);
		setSlice(in+1);
		if(plotfit==false){
			selectWindow(tbg);
			setSlice(in+1);
		}
		for(cs=0;cs<colen;cs++){
			savdir=psavpath[cs];
			for(ns=0;ns<npart;ns++){
				selectWindow(t1);
				for(ps=0;ps<polarnum;ps++)
					yvals[ps]=getPixel(ps+cs*polarnum,ns);
				if(plotfit==true){
					pname=buffer(npart,ns+1);
					psavname="Particle_"+pname+"_AngleDependence_"+chans[cs]+"nm";
					fitvals=polarplot(fitmodel,polnums,yvals,chans[cs],pcolor[cs],signam[sets],ns+1,subscript0,units,savdir,psavname);
					selectWindow(t2);
					for(i=0;i<numparams;i++) //changed length from 3 to numparams in v4.1.4 on 21/05/2021
						setPixel(i+cs*numparams,ns,fitvals[i]); //changed 1st argument from i+cs*3 to i+cs*numparams in v4.1.4 on 21/05/2021
				}
				else{
					selectWindow(tbg);
					for(ps=0;ps<polarnum;ps++)
						bgs[ps]=getPixel(ps+cs*polarnum,0);
					selectWindow(t2);
					for(j=0;j<polfits;j++){
						newvals=addnoise(polarnum,yvals,bgs);
						fitvals=polfit(fitmodel,polnums,newvals);
						for(i=0;i<numparams;i++){ //changed length from 3 to numparams in v4.1.4 on 21/05/2021
							setPixel(j+i*polfits+cs*numparams*polfits,ns,fitvals[i]); //changed 1st argument from i+cs*3 to i+cs*numparams in v4.1.4 on 21/05/2021
						}
					}
				}
			}
		}
	}
	close(t1);
	if(plotfit==false)
		close(tbg);
	selectWindow(t2);
	saveAs("Tiff",procdir+datsav);
	close();
}
/////////////////////////////function to draw one histogram per parameter per channel for all particles
function fithist(procdir,datim,setstrt,setend,chans,tau,zeta,eta,savpath,pcolor,subscript,usescale){
	open(procdir+datim);
	colen=chans.length;
	pname=newArray(3);
	for(i=setstrt;i<setend;i++){
		pname[0]="Alpha";
		pname[1]="Theta0";
		if(aroute!=amode[2])
			in=i;
		else
			in=i-1;
		pname[2]="MeanSigma_"+subscript[in];
		if(i==0){
			if (usescale==true)
				units=newArray("","(degrees)"," (nm^2)");
			else
				units=newArray("","(degrees)"," (unscaled)");
		}
		else{
			if((tau==1 && zeta==1 && eta==1) || usescale==false)
				units=newArray("","(degrees)","(unscaled)");
			else
				unitsnewArray("","(degrees)"," (nm^2)");
		}
		for(cs=0;cs<colen;cs++){
			for(ps=0;ps<3;ps++){
				xaxname=pname[ps]+units[ps];
				imtohist(procdir,datim,ps+cs*3,1,0,1,true,savpath[cs],"DistributionOf"+pname[ps]+"_"+chans[cs]+"nm","DistributionOf"+pname[ps]+"_"+chans[cs]+"nm",xaxname,"Occurrence",pcolor[cs],true,true,in,0,false);
			}
		}
	}
	close();
}
////////////////////////function to draw one histogram per parameter per channel per particle
function noisehist(procdir,datim,setstrt,setend,chans,polfits,tau,zeta,eta,savpath,pcolor,subscript,usescale){
	colen=chans.length;
	pname=newArray(3);
	open(procdir+datim);
	npart=getHeight();
	for(i=setstrt;i<setend;i++){
		pname[0]="Alpha";
		pname[1]="Theta0";
		if(aroute!=amode[2])
			in=i;
		else
			in=i-1;
		pname[2]="MeanSigma_"+subscript[in];
		if(i==0){
			if (usescale==true)
				units=newArray("","(degrees)"," (nm^2)");
			else
				units=newArray("","(degrees)"," (unscaled)");
		}
		else{
			if(tau==1 && zeta==1 && eta==1)
				units=newArray("","(degrees)","(unscaled)");
			else
				unitsnewArray("","(degrees)"," (nm^2)");
		}
		for(cs=0;cs<colen;cs++){
			//pname[0]="Alpha";
			//pname[1]="Theta0";
			//pname[2]="MeanSigma_"+subscript[in];
			for(ps=0;ps<3;ps++){
				for(np=0;np<npart;np++){
					pID=buffer(npart,np+1);
					xaxname=pname[ps]+units[ps];
					imtohist(procdir,datim,ps*polfits+cs*3*polfits,polfits,np,1,false,savpath[cs],"Particle"+pID+"_SimulatedDistributionOf"+pname[ps]+"_"+chans[cs]+"nm","Particle"+pID+"_SimulatedDistributionOf"+pname[ps]+"_"+chans[cs]+"nm",xaxname,"Occurrence",pcolor[cs],true,true,in,0,false);
				}
			}
		}
	}
	close();
}
////////////////////////function to draw histogram per parameter per channel for all simulated fit parameters across all particles
function histhist(procdir,datim,setstrt,setend,chans,polfits,tau,zeta,eta,savpath,pcolor,subscript,usescale){
	colen=chans.length;
	pname=newArray(3);
	open(procdir+datim);
	npart=getHeight();
	for(i=setstrt;i<setend;i++){
		if(aroute!=amode[2])
			in=i;
		else
			in=i-1;
		pname[0]="Alpha";
		pname[1]="Theta0";
		pname[2]="MeanSigma_"+subscript[in];
		if(i==0){
			if (usescale==true)
				units=newArray("","(degrees)"," (nm^2)");
			else
				units=newArray("","(degrees)"," (unscaled)");
		}
		else{
			if(tau==1 && zeta==1 && eta==1)
				units=newArray("","(degrees)","(unscaled)");
			else
				unitsnewArray("","(degrees)"," (nm^2)");
		}
		for(cs=0;cs<colen;cs++){
			//pname[0]="Alpha";
			//pname[1]="Theta0";
			//pname[2]="MeanSigma_"+subscript;
			for(ps=0;ps<3;ps++){
				xaxname=pname[ps]+units[ps];
				imtohist(procdir,datim,ps*polfits+cs*3*polfits,polfits,0,1,true,savpath[cs],"FullSimulatedDistributionOf"+pname[ps]+"_"+chans[cs]+"nm","FullSimulatedDistributionOf"+pname[ps]+"_"+chans[cs]+"nm",xaxname,"Occurrence",pcolor[cs],true,true,in,0,false);
			}
		}
	}
	close();
}
//////////////////////function to write fit-data images to xls
function fimtoxls(procdir,datim,savpath,colen,phenom,tablabs,polfits,aroute,amode,setstrt,setend,noise,subscript){//modified by addition of "phenom" as argument in v4.0.7
	open(procdir+datim);
	tdat=getTitle();
	npart=getHeight();
	if(noise==true){
		savname0="SimulatedFitData";
		npars=3;//added in v4.1.4 on 20/05/2021
	}
	else{
		savname0="FitData";
		npars=5;//added in v4.1.4 on 20/05/2021
	  	///removed 8 lines in v4.0.7
  	}  	
  	//savname0=savname;//removed in v4.0.7
  	selectWindow(tdat);
  	for(sets=setstrt;sets<setend;sets++){
  		if(aroute==amode[2])
  			in=sets-1;
  		else
  			in=sets;
  		setSlice(in+1);
  		pname=newArray(npars);
  		pname[0]="Simulated_Alpha";
		pname[1]="Simulated_Theta0";
		pname[2]="Simulated_MeanSigma_"+subscript[in];
		if(noise==false){
			savname=phenom[in]+"_"+savname0;
			Table.create(savname);
		  	for(row=0;row<npart;row++){
		  		Table.set("Data type",row,"Datapoint",savname);
		  		Table.set("Datapoint index", row, row+1,savname);
		  	}
		  	Table.set("Data type",npart,"Mean",savname);
		  	Table.set("Data type",npart+1,"Standard Deviation",savname);
	  		Table.update;
		}
		for(cs=0;cs<colen;cs++){
			//if(in==0 && noise==true){// removed in v4.0.7
			if(noise==true){
				savname=phenom[in]+"_"+savname0+"_"+chans[cs]+"nm";
				Table.create(savname);
			  	for(row=0;row<npart;row++){
			  		Table.set("Data type",row,"Datapoint",savname);
			  		Table.set("Datapoint index", row, row+1,savname);
			  	}
			}
			//if(noise==true) //removed in v4.0.7
			//	savname=savname0+"_"+chans[cs]+"nm"; //removed in v4.0.7
			for(ps=0;ps<npars;ps++){ ////changed from 3 to npars in v4.1.4 on 20/05/2021
				if(noise==false){//added v4.0.7
					pidx=ps+cs*npars;//added v4.0.7.  changed from 3 to npars in v4.1.4 on 20/05/2021
					lidx=pidx;//added v4.0.7
				}//added v4.0.7
				for(i=0;i<polfits;i++){
					if(noise==true){//added v4.0.7
						pidx=i+ps*polfits+cs*3*polfits;
						lidx=i+ps*polfits+in*3*polfits;//added in v4.0.7
					}//added in v4.0.7
					for (row=0; row<npart; row++) {
						datum=getPixel(pidx,row);
						Table.set(tablabs[lidx],row,datum,savname);
					}
				}
				if(noise==false){
					makeRectangle(ps+cs*npars,0,1,npart); //changed from 3 to npars in v4.1.4 on 20/05/2021
					getRawStatistics(nPixels, mean, min, max, std, histo);
					Table.set(tablabs[lidx],npart,mean,savname);
					Table.set(tablabs[lidx],npart+1,std,savname);
				}
				else{
					for(row=0;row<npart;row++){
						makeRectangle(ps*polfits+cs*3*polfits,row,polfits,1);
						getRawStatistics(nPixels, mean, min, max, std, histo);
						Table.set("Avg_"+pname[ps],row,mean,savname);
						Table.set("StdDev_"+pname[ps],row,std,savname);
					}
				}
			}
			Table.update;
			if(noise==true){
				//for(cs=0;cs<colen;cs++){//removed in v4.0.7
				//	savname=savname0+"_"+chans[cs]+"nm";//removed in v4.0.7
				//	selectWindow(savname);//removed in v4.0.7
					Table.save(savpath[cs]+savname+".xls");
					close(savname);
				//} /removed in v4.0.7
			}
		}
	}
	//removed 8 lines here in v4.0.7
	if(noise==false){//modified in v4.0.7 from else to if
		//for(cs=0;cs<colen;cs++){ //removed v4.0.7
			Table.save(savpath+savname+".xls");
			close(savname);
		//} //removed v4.0.7
	}
	close(tdat);
	return 0;
}
//////////////////////////////////////////////////////function to label particles in the image
function labelim(impath,savpath,procdir,coordim,refshiftx,refshifty,mainidx){
	open(impath);
	t1=getTitle();
	open(procdir+coordim);
	t2=getTitle();
	npart=getHeight();
	xs=newArray(npart);
	ys=newArray(npart);
	for(i=0;i<npart;i++){
		xs[i]=getPixel(0,i);
		ys[i]=getPixel(1,i);
	}
	close(t2);
	selectWindow(t1);
	for(i=0;i<npart;i++){
		setBackgroundColor(0,0,0);
		setForegroundColor(255,255,255);
		setFont("Times New Roman", 16, " anti-aliased");
		drawString(i+1, xs[i]-refshiftx[mainidx], ys[i]-refshifty[mainidx]);
	}
	saveAs("Tiff",finpath+"LabeledParticles");
	close();
}
////////////////////////////////////////////
////////////Begin new addition
if(extinctionsuite==true){
	Dialog.create("Extinction Suite");
	Dialog.addCheckbox("Perform Image Conversion?",false);
	Dialog.addCheckbox("Perform Image Averaging?",false);
	Dialog.addCheckbox("Perform Extinction Analysis?",false);
	Dialog.addCheckbox("Perform Particle Analysis?",false);
	Dialog.show();
	converter=Dialog.getCheckbox();
	averager=Dialog.getCheckbox();;
	extanalysis=Dialog.getCheckbox();;;
	partanalysis=Dialog.getCheckbox();;;;

	if(converter|averager==true){
		specrec=false;
		if(extanalysis==true || extanalysis==false){
			expmode=newArray("Extinction","Extinction + Darkfield","Darkfield");
			darkBGsubopt=newArray("No darkfield BG correction","Subtract BG Image","Subtract Numerical offset");
			formats=newArray("Consumer RAW Formats", "TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "8-bit TIFF");
			imstyle=newArray("Single frame per input image file","Multiple frames (a stack) per input image file");
			coloropt=newArray("M-Channel Grayscale","RGB","M-Channel Color");
			Dialog.create("Input Image Options");
			Dialog.addChoice("Specify Input Image File Format",formats,formats[0]);
			Dialog.addChoice("Specify Input Image Color Format",coloropt,coloropt[0]);
			Dialog.addMessage("Please indicate below, the method by wich the acquired images have been saved.\nSome cameras, e.g. the Canon 40D, save individual frames per saved file.\nOther cameras, e.g. the PCO Edge 5.5 sCMOS save multiple frames (an image stack) per saved file.");
			Dialog.addChoice("Save Style of Input Images",imstyle,imstyle[0]);
			Dialog.addChoice("Choose type of experiment:",expmode,expmode[0]);
			Dialog.addString("Enter center wavelength of spectral ranges (channels) as comma separated values (no spaces):", "450,550,650");
			Dialog.addMessage("If you acquired darkfield images, you can choose how you would like to correct the BG offset\nof the darkfield image. You can choose not to correct it, to correct it via BG images acquired using darkfield\nexposure settings, or to correct it with a simple numerical offset");
			Dialog.addChoice("Choose Dark BG Subtraction Option",darkBGsubopt,darkBGsubopt[0]);
			Dialog.show();
	
			inform=Dialog.getChoice();
			usrcolor=Dialog.getChoice();
			usrstyle=Dialog.getChoice();
			exproute=Dialog.getChoice();
			chans=getString();
			extdarkchoice=Dialog.getChoice();

			chans=CSV(chans);
			imageoption=newArray("M-Channel Grayscale Images","RAW","RGB","M-Channel Color Images");
			if(inform!=formats[0] && usrcolor==coloropt[0])
				raw=imageoption[0];
			else{
				if(inform==formats[0])
					raw=imageoption[1];
				else{
					if(inform!=formats[0] && usrcolor==coloropt[1])
						raw=imageoption[2];
					else if(inform!=formats[0] && usrcolor==coloropt[2])
						raw=imageoption[3];
				}
			}
			splitter=false;
				
			if(inform==formats[0]) exten=newArray(".3FR",".ARI",".ARW",".SRF",".SR2",".BAY",".CRW",".CR2",".CAP",".IIQ",".EIP",".DCS",".DCR",".DRF",".K25",".KDC",".DNG",".ERF",".FFF",".MEF",".MDC",".MOS",".MRW",".NEF",".NRW",".ORF",".PEF",".PTX",".PXN",".R3D",".RAF",".RAW",".RW2",".RWL",".RWZ",".SRW",".X3F");
			if(inform==formats[1] || inform==formats[10]) exten=newArray(".tif", ".tiff");
			if(inform==formats[2]) exten=newArray(".jpg", ".jpeg");
			if(inform==formats[3]) exten=newArray(".gif");
			if(inform==formats[4]) exten=newArray(".png");
			if(inform==formats[5]) exten=newArray(".pgm");
			if(inform==formats[6]) exten=newArray(".bmp");
			if(inform==formats[7]) exten=newArray(".fits");
			if(inform==formats[8]) exten=newArray(".txt");
			if(inform==formats[9]) exten=newArray(".zip");

			refdef=true;
			bgdef=true;
			refchoice=true;
			background=true;
			if(exproute!=expmode[2])
				focdef=true;
			else{
				focdef=false;
				focuschoice=false;
			}
			if(exproute==expmode[0]){
				darkdefault=false;
				darkchoice=false;
				darkBGchoice=false;
				avgdarkchoice=false;
				avgdarkBGchoice=false;
				darkfolder="";
				darkBGfolder="";
			}
			else{
				darkdefault=true;
				if(extdarkchoice==darkBGsubopt[0]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";	
				}
				if(extdarkchoice==darkBGsubopt[1])
					darkBGdefault=true;
				if(extdarkchoice==darkBGsubopt[2]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";
					Dialog.create("Dark BG Offset");
					Dialog.addNumber("Input Offset Level to be subtracted from Darkfield Image",100);
					Dialog.show();
					DBGoffset=Dialog.getNumber();
				}
			}
		}
	}
		
	if(converter==false && averager==false){
		if(extanalysis==true){
			expmode=newArray("Extinction","Extinction + Darkfield","Darkfield");
			darkBGsubopt=newArray("No darkfield BG correction","Subtract BG Image","Subtract Numerical offset");
			Dialog.create("Extinction and Darkfield Options");
			imageoption=newArray("M-Channel Grayscale Images","RAW","RGB","M-Channel Color Images","Stage-camera synchronized continuous recording");
			Dialog.addChoice("Image Type",imageoption,imageoption[0]);
			Dialog.addChoice("Choose type of experiment:",expmode,expmode[0]);
			Dialog.addString("Enter center wavelength for color channels as comma separated values (no spaces):", "450,550,650");
			Dialog.addMessage("If you acquired darkfield images, you can choose how you would like to correct the BG offset\nof the darkfield image. You can choose not to correct it, to correct it via BG images acquired using darkfield\nexposure settings, or to correct it with a simple numerical offset");
			Dialog.addChoice("Choose Dark BG Subtraction Option (if applicable):",darkBGsubopt,darkBGsubopt[0]);
			Dialog.show();
			//extchoices=Dialog.getChoice();
			raw=Dialog.getChoice();
			exproute=Dialog.getChoice();
			chans=Dialog.getString();
			extdarkchoice=Dialog.getChoice();
			//raw=extchoices;
			colorstr=chans;
			chans=CSV(chans);
			if(raw==imageoption[4])
				specrec=true;
			else
				specrec=false;
			refdef=true;
			bgdef=true;
			refchoice=true;
			background=true;
			if(exproute!=expmode[2])
				focdef=true;
			else{
				focdef=false;
				focuschoice=false;
			}
			if(exproute==expmode[0]){
				darkdefault=false;
				darkchoice=false;
				darkBGchoice=false;
				avgdarkchoice=false;
				avgdarkBGchoice=false;
				darkfolder="";
				darkBGfolder="";
			}
			else{
				darkdefault=true;
				if(extdarkchoice==darkBGsubopt[0]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";	
				}
				if(extdarkchoice==darkBGsubopt[1])
					darkBGdefault=true;
				if(extdarkchoice==darkBGsubopt[2]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";
					Dialog.create("Dark BG Offset");
					Dialog.addNumber("Input Offset Level to be subtracted from Darkfield Image",100);
					Dialog.show();
					DBGoffset=Dialog.getNumber();
				}
			}
//			if(specrec==true){
//				if(polarization==true){
//					Dialog.create("Polarization Options");
//					Dialog.addNumber("Initial Angle",0);
//					Dialog.addNumber("Final Angle",150);
//					Dialog.addNumber("Angle Step Size",30);
//					Dialog.show();
//					
//					polstart=Dialog.getNumber();
//					polend=Dialog.getNumber();;
//					polstep=Dialog.getNumber();;;
//			
//					polarnum=((polend-polstart)/polstep)+1;
//					polnums=newArray(polarnum);
//					
//					for(poloop=0;poloop<polarnum;poloop++)
//						polnums[poloop]=polstart+poloop*polstep; ////numeric array of polarizer angles
//					polnames=polbuffer(polstart,polend,polstep); /////string array of polarizer angles
//					polnames=Array.concat(polnames,"BG");
//					polnames=Array.concat(polnames,"DarkBG");
//				}
//				else{
//					polnames=newArray("");
//					polarnum=1;
//				}
//			}
		}
	}
		
		
	if(converter==true){
		Dialog.create("Converter");
		Dialog.setInsets(10,20,7);
		Dialog.addMessage("Choose Image sets to Convert");
		if(focdef==true)
			Dialog.addCheckbox("Complete for focus image set?",focdef);
		Dialog.addCheckbox("Complete for reference image set?",refdef);
		Dialog.addCheckbox("Complete for background image set?",bgdef);
		if(darkdefault==true){
			Dialog.addCheckbox("Complete for darkfield image set?",darkdefault);
			if(darkBGdefault==true)
				Dialog.addCheckbox("Complete for darkfield background image set?",darkBGdefault);
		}
		Dialog.show();
		if(focdef==true)
			focuschoice=Dialog.getCheckbox();
		else
			focuschoice=false;
		refchoice=Dialog.getCheckbox();
		background=Dialog.getCheckbox();
		
		if(darkdefault==true){
			darkchoice=Dialog.getCheckbox();
			if(darkBGdefault==true)
				darkBGchoice=Dialog.getCheckbox();
			else
				darkBGchoice=false;
		}
		else
			darkchoice=false;
	}
	
	if(converter==true && averager==true){
			avgfocuschoice=focuschoice;
			avgrefchoice=refchoice;
			avgbackground=background;	
			avgdarkchoice=darkchoice;
			avgdarkBGchoice=darkBGchoice;
	}
	else if(converter==false && averager==true){
		Dialog.create("Averager");
		Dialog.setInsets(10,20,7);
		Dialog.addMessage("Choose Image sets to Average");
		if(focdef==true)
			Dialog.addCheckbox("Complete for focus image set?",true);
		Dialog.addCheckbox("Complete for reference image set?",true);
		Dialog.addCheckbox("Complete for background image set?",true);
		if(darkdefault==true){
			Dialog.addCheckbox("Complete for darkfield image set?",darkdefault);
			if(darkBGdefault==true)
				Dialog.addCheckbox("Complete for darkfield background image set?",darkBGdefault);
		}
		Dialog.show();
		if(focdef==true)
			avgfocuschoice=Dialog.getCheckbox();
		else
			avgfocuschoice=false;
		avgrefchoice=Dialog.getCheckbox();
		avgbackground=Dialog.getCheckbox();
		if(darkdefault==true){
			avgdarkchoice=Dialog.getCheckbox();
			if(darkBGdefault==true)
				avgdarkBGchoice=Dialog.getCheckbox();
			else
				avgdarkBGchoice=false;
		}
		else
			avgdarkchoice=false;
	}
	
	
	if(converter==true && averager==false){
		if(raw!=imageoption[0]){
			Dialog.create("Color Conversion Options");
			Dialog.addCheckbox("Split Color Image?",true);
			Dialog.show();
			splitter = Dialog.getCheckbox();
		}
		if(raw==imageoption[0])
			splitter=false;
		avgfocuschoice=false;
		avgrefchoice=false;
		avgbackground=false;
		avgdarkchoice=false;
		avgdarkBGchoice=false;
	}
	else if(converter==true){
		if(raw!=imageoption[0]){
			if(averager==true)
				splitter=true;
		}
		if(raw==imageoption[0])
			splitter = false;
	}
	if(converter==false){
		focuschoice=false;
		refchoice=false;
		background=false;
		darkchoice=false;
		darkBGchoice=false;
	}
	if(converter==false && averager==false){
		focuschoice=false;
		refchoice=false;
		background=false;
		darkchoice=false;
		avgfocuschoice=false;
		avgrefchoice=false;
		avgbackground=false;
		avgdarkchoice=false;
		avgdarkBGchoice=false;
	}
	if(extanalysis==false)
		specrec=false;
		
	if(mapping==true)
		usermapping=false;
	if(mapping==false){
		polbase=getDirectory("Choose BASE Folder");
		if((converter|averager|extanalysis==true) && specrec==false){
			Dialog.create("Folder Information");
			Dialog.addCheckbox("Did you create your folder mapping using the Extinction Suite?",true);
			Dialog.show();
			usermapping=Dialog.getCheckbox();
			
			if(polarization==false){
				//BASEdirec=polbase;
				polarnum=1;
			}
			if(usermapping==true){
				if(focdef==true)
					focusfolder="Focus";
				reffolder="Reference";
				BGfolder="BG";
				if(darkdefault==true){
					darkfolder="Dark";
					if(darkBGdefault==true)
						darkBGfolder="DarkBG";
				}
			}
			if(usermapping==false){
				if(converter|averager|extanalysis|partanalysis==true){
					Dialog.create("Folder Titles");
					if(focdef==true)
						Dialog.addString("Type title of Focus Folder", "Focus");
					Dialog.addString("Type title of Reference Folder", "Reference");
					Dialog.addString("Type title of BG Folder", "BG");
					if(darkdefault==true){
						Dialog.addString("Type title of Darkfield Folder","Dark");
						if(darkBGdefault==true)
							Dialog.addString("Type title of Darkfield-BG Folder","DarkBG");
					}
					Dialog.show();
					if(focdef==true)
						focusfolder=Dialog.getString();
					reffolder=Dialog.getString();
					BGfolder=Dialog.getString();
					if(darkdefault==true){
						darkfolder=Dialog.getString();
						if(darkBGdefault==true)
							darkBGfolder=Dialog.getString();
					}
				}
			}
		}
	}
	if(converter|averager|extanalysis|partanalysis==true && polarization==true){
		Dialog.create("Polarization Options");
		Dialog.addNumber("Initial Angle",0);
		Dialog.addNumber("Final Angle",150);
		Dialog.addNumber("Angle Step Size",30);
		Dialog.show();
		
		polstart=Dialog.getNumber();
		polend=Dialog.getNumber();;
		polstep=Dialog.getNumber();;;

		polarnum=((polend-polstart)/polstep)+1;
		polnums=newArray(polarnum);
		
		for(poloop=0;poloop<polarnum;poloop++)
			polnums[poloop]=polstart+poloop*polstep; ////numeric array of polarizer angles
		polnames=polbuffer(polstart,polend,polstep); /////string array of polarizer angles
		polnames=Array.concat(polnames,"BG");
		polnames=Array.concat(polnames,"DarkBG");
	}
	if(polarization==false){
		polnames=newArray("");
		polarnum=1;
	}
	if(partanalysis==true){//when particle analysis is selected, check to see if the Processing folder already exists and if it contains a text file with parameters and coordinates
		procdir=polbase+"Processing";
		procexst=File.exists(procdir);
		if(procexst==0)
			File.makeDirectory(procdir);
		procdir=procdir+"/";
		txtexst=File.exists(procdir+"Parameters.txt");
		if(txtexst){
			parstr=File.openAsString(procdir+"Parameters.txt");
			parlines=split(parstr,"\n");
			parfile="";
		}
		else{
			parfile=File.open(procdir+"Parameters.txt");
			usetxt=false;
		}
		strcnt=0;
	}
	else{
		txtexst=false;
		usetxt=false;
	}
	if(txtexst){//if the text file with parameters and coordinates exists, ask the user if they want to load these files and run automatically
		Dialog.create("Data Input Options");
		Dialog.addMessage("Parameter and coordinate files were found in the source directory.");
		Dialog.addCheckbox("Use parameters and peak/background point locations from files", true);
		Dialog.show();
		usetxt=Dialog.getCheckbox();
		if(usetxt){
			convertTextToIm("signal",procdir,"PeakCoordinates.txt");
			convertTextToIm("filter",procdir,"FilteredCoordinates.txt");
			convertTextToIm("BG",procdir,"BGCoordinates.txt");
			parstr=File.openAsString(procdir+"Parameters.txt");
			txtlins=split(parstr,"\n");
			aroute=substring(txtlins[0],indexOf(txtlins[0],"\t")+1,lengthOf(txtlins[0]));
			bgroute=substring(txtlins[1],indexOf(txtlins[1],"\t")+1,lengthOf(txtlins[1]));
			usrspc=parseFloat(substring(txtlins[2],indexOf(txtlins[2],"\t")+1,lengthOf(txtlins[2])));
			datapoints=parseInt(substring(txtlins[3],indexOf(txtlins[3],"\t")+1,lengthOf(txtlins[3])));  
			numext=substring(txtlins[4],indexOf(txtlins[4],"\t")+1,lengthOf(txtlins[4]));
			chans=substring(txtlins[5],indexOf(txtlins[5],"\t")+1,lengthOf(txtlins[5]));
			ranks=substring(txtlins[6],indexOf(txtlins[6],"\t")+1,lengthOf(txtlins[6]));
			NA=parseFloat(substring(txtlins[7],indexOf(txtlins[7],"\t")+1,lengthOf(txtlins[7])));
			mag=parseFloat(substring(txtlins[8],indexOf(txtlins[8],"\t")+1,lengthOf(txtlins[8])));
			pixs=parseFloat(substring(txtlins[9],indexOf(txtlins[9],"\t")+1,lengthOf(txtlins[9])));
			fwc=parseFloat(substring(txtlins[10],indexOf(txtlins[10],"\t")+1,lengthOf(txtlins[10])));
			polfits=parseInt(substring(txtlins[11],indexOf(txtlins[11],"\t")+1,lengthOf(txtlins[11])));
			timereg=parseInt(substring(txtlins[12],indexOf(txtlins[12],"\t")+1,lengthOf(txtlins[12])));
			gaussfit=parseInt(substring(txtlins[13],indexOf(txtlins[13],"\t")+1,lengthOf(txtlins[13])));
			radiuscalibration=parseInt(substring(txtlins[14],indexOf(txtlins[14],"\t")+1,lengthOf(txtlins[14])));
			fitmodel=substring(txtlins[15],indexOf(txtlins[15],"\t")+1,lengthOf(txtlins[15]));
			scale=parseFloat(substring(txtlins[16],indexOf(txtlins[16],"\t")+1,lengthOf(txtlins[16])));
			usescale=parseInt(substring(txtlins[17],indexOf(txtlins[17],"\t")+1,lengthOf(txtlins[17])));
			areacal=parseFloat(substring(txtlins[18],indexOf(txtlins[18],"\t")+1,lengthOf(txtlins[18])));
			regpts=parseInt(substring(txtlins[19],indexOf(txtlins[19],"\t")+1,lengthOf(txtlins[19])));
			flofitmodel=substring(txtlins[20],indexOf(txtlins[20],"\t")+1,lengthOf(txtlins[20]));
			tau=parseFloat(substring(txtlins[21],indexOf(txtlins[21],"\t")+1,lengthOf(txtlins[21])));
			zeta=parseFloat(substring(txtlins[22],indexOf(txtlins[22],"\t")+1,lengthOf(txtlins[22])));
			eta=parseFloat(substring(txtlins[23],indexOf(txtlins[23],"\t")+1,lengthOf(txtlins[23])));
			exptime=parseFloat(substring(txtlins[24],indexOf(txtlins[24],"\t")+1,lengthOf(txtlins[24])));
			Nsig=parseFloat(substring(txtlins[25],indexOf(txtlins[25],"\t")+1,lengthOf(txtlins[25])));
			radius=CSVf(substring(txtlins[26],indexOf(txtlins[26],"\t")+1,lengthOf(txtlins[26])));
			radius2=CSVf(substring(txtlins[27],indexOf(txtlins[27],"\t")+1,lengthOf(txtlins[27])));
			mainidx=parseInt(substring(txtlins[28],indexOf(txtlins[28],"\t")+1,lengthOf(txtlins[28])));
			refshiftx=CSVf(substring(txtlins[29],indexOf(txtlins[29],"\t")+1,lengthOf(txtlins[29])));
			refshifty=CSVf(substring(txtlins[30],indexOf(txtlins[30],"\t")+1,lengthOf(txtlins[30])));
			shapename=substring(txtlins[31],indexOf(txtlins[31],"\t")+1,lengthOf(txtlins[31]));
			rectlocx=parseInt(substring(txtlins[32],indexOf(txtlins[32],"\t")+1,lengthOf(txtlins[32])));
			rectlocy=parseInt(substring(txtlins[33],indexOf(txtlins[33],"\t")+1,lengthOf(txtlins[33])));
			ovalwi=parseInt(substring(txtlins[34],indexOf(txtlins[34],"\t")+1,lengthOf(txtlins[34])));
			ovalhe=parseInt(substring(txtlins[35],indexOf(txtlins[35],"\t")+1,lengthOf(txtlins[35])));
			findhow=substring(txtlins[36],indexOf(txtlins[36],"\t")+1,lengthOf(txtlins[36]));
			usrtol=parseFloat(substring(txtlins[37],indexOf(txtlins[37],"\t")+1,lengthOf(txtlins[37])));
			driftx=CSVf(substring(txtlins[38],indexOf(txtlins[38],"\t")+1,lengthOf(txtlins[38])));
			drifty=CSVf(substring(txtlins[39],indexOf(txtlins[39],"\t")+1,lengthOf(txtlins[39])));
			sidx=parseInt(substring(txtlins[40],indexOf(txtlins[40],"\t")+1,lengthOf(txtlins[40])));
			lowlim=parseFloat(substring(txtlins[41],indexOf(txtlins[41],"\t")+1,lengthOf(txtlins[41])));
			uplim=parseFloat(substring(txtlins[42],indexOf(txtlins[42],"\t")+1,lengthOf(txtlins[42])));
		}
	}
	if(converter==false && averager==false && extanalysis==false && partanalysis==true){
		print("\\Clear");
		finpath=polbase+"ProcessedImages"+"/";
		finims=getFileList(finpath);
		len=finims.length/polarnum;
		num=0;
		dnum=0;
		for(i=0;i<len;i++){
			if(indexOf(finims[i],"Extinction")>=0)
				num++;
			if(indexOf(finims[i],"Darkfield")>=0)
				dnum++;
		}
		if(dnum>0 && num>0){
			if(dnum!=num)
				waitForUser("Note there are extinction and darkfield images in the folder,\nbut the number of color channels of each are not consistent.");
		}
		//polarization=false;
		amode=newArray("Extinction","Extinction + Darkfield", "Darkfield");
		bgmeth=newArray("Defocus","Shifted Sample");		
		if(usetxt==false || txtexst==false){
			Dialog.create("Analysis Options");
			Dialog.addString("Input color channel titles (comma-separated, no spaces)","600,550,500,450",30);
			Dialog.setInsets(10,5,5);
			Dialog.addMessage("Input a rank for each color in list above. Highest rank is 1. Colors may have equal rank.\nLowest equivalent rank colors may be left at an arbitrarily high value, e.g. 1000.\nIf there is only 1 channel, you can enter 1 as its rank.");
			Dialog.addString("Input ranks of channels (comma-separated, no spaces):","1,1,1,1",30);
			Dialog.addString("Input total number of brightfield images taken per channel for extinction if applicable (comma-separated, no spaces).","2560,2560,2560,2560",30);
			Dialog.addChoice("Choose analysis route",amode,amode[0]);
			Dialog.addChoice("Choose background subtraction method",bgmeth,bgmeth[1]);
			Dialog.addNumber("Proximity restriction (n*Ri): ",2);
			Dialog.addNumber("Desired Number of BG datapoints to examine",1000);
			//Dialog.addNumber("Number of particle species/nominal sizes expected",1);
			Dialog.addNumber("Numerical Aperture",0.95);
			Dialog.addNumber("Magnification",40);
			Dialog.addNumber("Pixel size (um)",6.5);
			Dialog.addNumber("Full well capacity",30000);
			if(polarization==true)
					Dialog.addNumber("Specify Number of Polarization Fit Noise Simulations",100);
			Dialog.setInsets(15,0,10);
			Dialog.addCheckbox("Perform time series analysis?",false);
			Dialog.addCheckbox("Perform Gaussian fit to particle intensity\nprofile for improved coordinate centering?",true);
			if(polarization==false){
				Dialog.addCheckbox("Calibrate the particle measurement radius (Ri)?",false);
			}
			Dialog.show();
			
			chans=Dialog.getString();
			ranks=Dialog.getString();
			numext=Dialog.getString();
			aroute=Dialog.getChoice();
			bgroute=Dialog.getChoice();
			usrspc=Dialog.getNumber();
			datapoints=Dialog.getNumber();
			//NPsize=Dialog.getNumber;
			NA=Dialog.getNumber();
			mag=Dialog.getNumber();
			pixs=Dialog.getNumber();
			fwc=Dialog.getNumber();
			if(polarization==true)
				polfits=Dialog.getNumber();
			else
				polfits=0;
			timereg=Dialog.getCheckbox();
			gaussfit=Dialog.getCheckbox();
			if(polarization==false)
				radiuscalibration=Dialog.getCheckbox();
			else
				radiuscalibration=false;
			strcnt=dualprint("Analysis path:\t"+aroute,parfile,"",0,strcnt); //0 parameters entry ID
			strcnt=dualprint("BG subtraction type:\t"+bgroute,parfile,"",0,strcnt); //1
			strcnt=dualprint("Proximity restriction, N*Ri, N=\t"+d2s(usrspc,2),parfile,"",0,strcnt); //2
			strcnt=dualprint("# BG datapoints taken=\t"+d2s(datapoints,0),parfile,"",0,strcnt); //3
			strcnt=dualprint("Total # brightfield images taken per channel:\t"+numext,parfile,"",0,strcnt); //4
			//print("# nominal particle sizes expected in sample= "+NPsize);
			strcnt=dualprint("Center wavelength of spectral ranges:\t"+chans,parfile,"",0,strcnt); //5
			strcnt=dualprint("Ranks of spectral ranges:\t"+ranks,parfile,"",0,strcnt); //6
			strcnt=dualprint("Numerical aperture=\t"+d2s(NA,3),parfile,"",0,strcnt); //7
			strcnt=dualprint("Magnification=\t"+d2s(mag,1),parfile,"",0,strcnt); //8
			strcnt=dualprint("Pixel size=\t"+d2s(pixs,3),parfile,"",0,strcnt); //9
			strcnt=dualprint("Full well capacity=\t"+d2s(fwc,0),parfile,"",0,strcnt); //10
			strcnt=dualprint("Number of simulations for polarization fits=\t"+d2s(polfits,0),parfile,"",0,strcnt); //11
			strcnt=dualprint("Perform time series analysis?\t"+d2s(timereg,0),parfile,"",0,strcnt); //12
			strcnt=dualprint("Used Gaussian Fit for Particle Center Coordinates?\t"+d2s(gaussfit,0),parfile,"",0,strcnt); //13
			strcnt=dualprint("Ri dependence examined?\t"+radiuscalibration,parfile,"",0,strcnt); //14
		}

		chans=CSV(chans);
		ranks=CSV(ranks);
		ranks=listparse(ranks);
		numext=CSV(numext);
		numext=listparse(numext);
		for(i=0;i<chans.length;i++){
			numext[i]=numext[i]/2;
		}
		if(aroute!=amode[0])
			darkdefault=true;
		else
			darkdefault=false;
	}
	//////Begin Converter/Averager/Extinction Polarization Loop
	if(converter|averager|extanalysis==true){
		//progcount=0;
		//progname=newArray(converter,averager,extanalysis);
		//for(prog=0;prog<3;prog++){
		//	if(progname[prog]==true) progcount++;
		//}
		//proghe=60;
		//progwi=100;
		//newImage("Image Processing Completion","RGB",
		if(specrec==false){
			setBatchMode(true);
			for(poloop=0;poloop<polarnum;poloop++){
				BASEdirec=polbase+polnames[poloop]+"/";
				if(usermapping==true){
					if(focdef==true)
						focusfolder="Focus";
					reffolder="Reference";
					BGfolder="BG";
					if(darkdefault==true){
						darkfolder="Dark";
						if(darkBGdefault==true)
							darkBGfolder="DarkBG";
					}
				}	
				if(usermapping==true || usermapping==false){
					if(raw==imageoption[0]){
						if(focdef==true){
							focsrc=BASEdirec+focusfolder+"/";
							focbase=focsrc;
						}
						refsrc=BASEdirec+reffolder+"/";
						BGsrc=polbase+BGfolder+"/";
						refbase=refsrc;
						BGdest=BGsrc;
						if(darkdefault==true){
							darsrc=BASEdirec+darkfolder+"/";
							darbase=darsrc;
							if(darkBGdefault==true){
								darkBGsrc=polbase+darkBGfolder+"/";
								darkBGdest=darkBGsrc;
							}
						}
						else{
							darsrc="";
							darbase=darsrc;
							darkBGsrc="";
							darkBGdest=darkBGsrc;
						}
					}
					else{
						if(focdef==true){
							focsrc=BASEdirec+focusfolder+"/"+"RAW"+"/";
							focbase=BASEdirec+focusfolder+"/";
						}
						refsrc=BASEdirec+reffolder+"/"+"RAW"+"/";
						BGsrc=polbase+BGfolder+"/"+"RAW"+"/";
						refbase=BASEdirec+reffolder+"/";
						BGdest=polbase+BGfolder+"/";
						if(darkdefault==true){
							darsrc=BASEdirec+darkfolder+"/"+"RAW"+"/";
							darbase=BASEdirec+darkfolder+"/";
							if(darkBGdefault==true){
								darkBGsrc=polbase+darkBGfolder+"/"+"RAW"+"/";
								darkBGdest=polbase+darkBGfolder+"/";
							}
						}
					}
				}
				if(raw!=imageoption[0]){
					if(focdef==true){
						focusrawexists=File.exists(BASEdirec+focusfolder+"/"+"RAW");
						if((focuschoice==true || (focuschoice==false && avgfocuschoice==true && raw==imageoption[2])) && focusrawexists==false){
							File.makeDirectory(BASEdirec+"Focus_processing");
							File.rename(BASEdirec+focusfolder,BASEdirec+"Focus_processing"+"/"+"RAW");
							wait(100);
							File.rename(BASEdirec+"Focus_processing",BASEdirec+focusfolder);
						}
					}
					refrawexists=File.exists(BASEdirec+reffolder+"/"+"RAW");
					BGrawexists=File.exists(polbase+BGfolder+"/"+"RAW");	
					if((refchoice==true || (refchoice==false && avgrefchoice==true && raw==imageoption[2])) && refrawexists==false){
						File.makeDirectory(BASEdirec+"Reference_processing");
						File.rename(BASEdirec+reffolder,BASEdirec+"Reference_processing"+"/"+"RAW");
						wait(100);
						File.rename(BASEdirec+"Reference_processing",BASEdirec+reffolder);
					}
					if((background==true || (background==false && avgbackground==true && raw==imageoption[2])) && BGrawexists==false && poloop==0){
							File.makeDirectory(polbase+"BG_processing");
							File.rename(polbase+BGfolder,polbase+"BG_processing"+"/"+"RAW");
							wait(100);
							File.rename(polbase+"BG_processing",polbase+BGfolder);
					}
					if(darkdefault==true){
						darkrawexists=File.exists(BASEdirec+darkfolder+"/"+"RAW");
						if((darkchoice==true || (darkchoice==false && avgdarkchoice==true && raw==imageoption[2])) && darkrawexists==false){
							File.makeDirectory(BASEdirec+"Dark_processing");
							File.rename(BASEdirec+darkfolder,BASEdirec+"Dark_processing"+"/"+"RAW");
							wait(100);
							File.rename(BASEdirec+"Dark_processing",BASEdirec+darkfolder);
						}
						if(extdarkchoice==darkBGsubopt[2]){
							darkBGrawexists=File.exists(polbase+darkBGfolder+"/"+"RAW");
							if((darkBGchoice==true || (darkBGchoice==false && avgdarkBGchoice==true && raw==imageoption[2])) && poloop==0){
								if(darkBGrawexists==false){
									File.makeDirectory(polbase+"darkBG_processing");
									File.rename(polbase+darkBGfolder,polbase+"darkBG_processing"+"/"+"RAW");
									wait(100);
									File.rename(polbase+"darkBG_processing",polbase+darkBGfolder);
								}
							}
						}
					}
				}
				imagesets=newArray(focuschoice,refchoice,background,darkchoice,darkBGchoice);
				setnames=newArray("focuschoice","refchoice","background","darkchoice","darkBGchoice");
				setTitles=newArray("FOCUS","REFERENCE","BG","DARKFIELD","DARKFIELD BG");
				dest=newArray(focbase,refbase,BGdest,darbase,darkBGdest);
				srcs=newArray(focsrc,refsrc,BGsrc,darsrc,darkBGsrc);
				avgsets=newArray(avgfocuschoice,avgrefchoice,avgbackground,avgdarkchoice,avgdarkBGchoice);
				avgsetnames=newArray("avgfocuschoice","avgrefchoice","avgbackground","avgdarkchoice","avgdarkBGchoice");
	/////////////////////////////////////////////////////Converter portion of macro
					if(averager|converter==true){
					///Determine Number of sets to be converted and averaged
						if(converter==true){
							setnum=0;
							for(i=0;i<imagesets.length;i++)
								if(imagesets[i]==true)
									setnum++;
							setnames=rebuildmatching(setnames,imagesets);
							setTitles=rebuildmatching(setTitles,imagesets);
							dest=rebuildmatching(dest,imagesets);
							srcs=rebuildmatching(srcs,imagesets);
							if(averager==true){
								avgsetnum=setnum;
								avgsetnames=rebuildmatching(avgsetnames,imagesets);
								asetTitles=setTitles;
								adest=dest;
								asrcs=srcs;
							}
						}
						if(converter==false && averager==true){
							avgsetnum=0;
							for(i=0;i<imagesets.length;i++)
								if(avgsets[i]==true)
									avgsetnum++;
							setnum=avgsetnum;
							asetT=rebuildmatching(setTitles,avgsets);
							avgsetnames=rebuildmatching(avgsetnames,avgsets);
							adest=rebuildmatching(dest,avgsets);
							asrcs=rebuildmatching(srcs,avgsets);
						}
					// Obtain input information - Number of channels, Channel names
					// color images
						if(raw!=imageoption[0]){
							if(splitter==true){
								if(raw==imageoption[1] || raw==imageoption[2]){
									chcnt0=3;
									chcnt=3*setnum;
									prenames=newArray("B","G","R");
									chnames=newArray("B","G","R");
									for(cname=0;cname<setnum-1;cname++)
										chnames=Array.concat(chnames,prenames);
								}
								if(raw==imageoption[3]){
									Dialog.create("M-Channel Color Image: Channel Info (1)");
									Dialog.addNumber("Enter number of channels in images",4);
									Dialog.show();
									chcnt0=Dialog.getNumber();
									chnames=newArray(chcnt0);
									Dialog.create("M-Channel Color Image: Channel Info (2)");
									suggestednames=newArray(chcnt0);
									for(i=0;i<chcnt0;i++){
										suggestednames[i]="C"+i;
										Dialog.addString("Channel "+i+1+" Name",suggestednames[i]);
									}
									Dialog.show();
									for(i=0;i<chcnt0;i++)
										chnames[i]=Dialog.getString();
									chcnt=chcnt0*setnum;
									prenames=chnames;
									for(cname=0;cname<setnum-1;cname++)
										chnames=Array.concat(chnames,prenames);
								}
							}
							if(splitter==false){
								chcnt0=setnum;
								chcnt=setnum;
								chnames=newArray(setnum);
								chnames=listfill(chnames,"");
								dirlen=newArray(setnum);
								dirlen=listfill(dirlen,1);
							}
							else{
								dirlen=newArray(setnum);
								dirlen=listfill(dirlen,chcnt0);
							}
							incnt=chcnt0;
							//mchans=newArray(incnt*dirlen[0]);
							mchans=newArray(setnum*chcnt0);
							mchans=listfill(mchans,"");
						}
						
						///Grayscale images
						if(raw==imageoption[0]){
							/// check number of channel directories contained in source folders and obtain names
							mchans=newArray(setnum);
							dirlen=newArray(setnum);
							sumins=0;
							for(sets=0;sets<setnum;sets++){
								if(((setnames[sets]!="background") && (setnames[sets]!="darkBGchoice")) || (setnames[sets]=="background" && poloop==0) || (setnames[sets]=="darkBGchoice" && poloop==0)){
									conexists=File.exists(dest[sets]+"Converted");
									if(conexists==false){
										prelist=foldReject(srcs[sets],"Averages");
										//prelist=getFileList(srcs[sets]);
									}
									else{
										//prelist=getFileList(dest[sets]+"Converted");
										prelist=foldReject(dest[sets]+"Converted","Averages");
									}
									newcond=newArray(prelist.length);
									for(i=0;i<prelist.length;i++){
										if(conexists==false){
											if(File.isDirectory(srcs[sets]+prelist[i]))
												newcond[i]=true;
											else
												newcond[i]=false;
										}
										if(conexists==true){
											if(File.isDirectory(dest[sets]+"Converted/"+prelist[i]))
												newcond[i]=true;
											else
												newcond[i]=false;
										}
									}
									chanlist=rebuildmatching(prelist,newcond);
									dirlen[sets]=chanlist.length;
									if(dirlen[sets]!=1){
										if(sets==0)
											mchans=chanlist;
										if(sets>0)
											mchans=Array.concat(mchans,chanlist);
									}
									if(dirlen[sets]==1)
										mchans[sets]="";
									sumins+=dirlen[sets];
								}
							}
			
							incnt=sumins;
							chcnt=incnt;
							if(chcnt==1){
								chnames=newArray(setnum);
								chnames=listfill(chnames,"");
							}
							if(chcnt>1)
								chnames=stripchar(mchans,"/");
						}
							
						// Prepare some arrays for output/input directories and place-keeping
						/// For Conversion
						outputs=newArray(chcnt);
						if(raw==imageoption[0])
							filperchan=newArray(chcnt);
						else
							filperchan=newArray(setnum);
						/// For Averaging
						avgins=newArray(chcnt);
						//if(raw==imageoption[0]) avgfilperchan=newArray(chcnt);
						//else avgfilperchan=newArray(setnum);
						avgfilperchan=newArray(chcnt);
							
						// Make Output directories and prepare list of all output conversion directories, and all input images to be converted.
						// Make all averaging output directories and determine sources for averaging.
						sumins=0;
						numoccur=0;
						avgsumins=0;
						avgnumoccur=0;
						if(raw!=imageoption[0] && splitter==false){
							dirlen=newArray(setnum);
							dirlen=listfill(dirlen,1);
						}
					}
					
					if(converter==true){
						for(sets=0;sets<setnum;sets++){
							if(((setnames[sets]!="background") && (setnames[sets]!="darkBGchoice")) || (setnames[sets]=="background" && poloop==0) || (setnames[sets]=="darkBGchoice" && poloop==0)){
								numoccur++;
								File.makeDirectory(dest[sets]+"Converted");
								for(j=0;j<dirlen[sets];j++){
									File.makeDirectory(dest[sets]+"Converted/"+chnames[j+sumins]);
									if(raw!=imageoption[0] && splitter==false)
										outputs[j+sumins]=dest[sets]+"Converted/"+chnames[j+sumins];
									else
										outputs[j+sumins]=dest[sets]+"Converted/"+chnames[j+sumins]+"/";
									if(raw==imageoption[0]){
										prelist=getFileList(srcs[sets]+mchans[j+sumins]);
										interlist=duallistrebuilder(prelist,exten);
										filperchan[j+sumins]=interlist.length;
										if(numoccur==1){
											if(j==0)
												Lists=interlist;
											else
												Lists=Array.concat(Lists,interlist);
										}
										if(numoccur>1)
											Lists=Array.concat(Lists,interlist);
									}
								}
								sumins+=dirlen[sets];
								if(raw!=imageoption[0]){
									prelist=getFileList(srcs[sets]);
									interlist=duallistrebuilder(prelist,exten);
									filperchan[sets]=interlist.length;
									if(numoccur==1)
										Lists=interlist;
									else
										Lists=Array.concat(Lists,interlist);
								}
							}
						}
					}
					if(averager==true){
						for(sets=0;sets<avgsetnum;sets++){
							if(((avgsetnames[sets]!="avgbackground") && (avgsetnames[sets]!="avgdarkBGchoice")) || ((avgsetnames[sets]=="avgbackground") && (poloop==0)) || ((avgsetnames[sets]=="avgdarkBGchoice") && (poloop==0))){
								avgnumoccur++;
								File.makeDirectory(adest[sets]+"Averages");
								if(converter==false)
									conexists=File.exists(adest[sets]+"Converted");
								for(j=0;j<dirlen[sets];j++){
									if(converter==true)
										avgins[j+avgsumins]=dest[sets]+"Converted/"+chnames[j+avgsumins]+"/";
									if(converter==false){
										if(conexists==true && File.exists(adest[sets]+"Converted/"+chnames[j+avgsumins]))
											avgins[j+avgsumins]=adest[sets]+"Converted/"+chnames[j+avgsumins]+"/";
										else
											avgins[j+avgsumins]=asrcs[sets]+chnames[j+avgsumins]+"/";
										prelist=getFileList(avgins[j+avgsumins]);
										avgexten=newArray(".tif",".tiff",".jpg", ".jpeg",".gif",".png",".pgm",".bmp",".fits",".txt",".zip",".raw");
										interlist=duallistrebuilder(prelist,avgexten);
										avgfilperchan[j+avgsumins]=interlist.length;
										//if(numoccur==1){
										//	if(j==0) aLists=interlist;
										//	else aLists=Array.concat(Lists,interlist);
										//}
										//if(numoccur>1) aLists=Array.concat(Lists,interlist);
									}
								}
								avgsumins+=dirlen[sets];
							}
						}
					}
					////////////Conversion Option Dialog
					if((converter==true) && (poloop==0)){
						formats = newArray("TIFF", "8-bit TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "Raw");
						Dialog.create("Image Conversion Options");
						if(raw==imageoption[0])
							sumins=0;
						for(sets=0;sets<setnum;sets++){
							if(raw==imageoption[0])
								imcount=sumins;
							if(raw!=imageoption[0])
								imcount=sets;
							Dialog.setInsets(15,0,5);
							Dialog.addMessage(setTitles[sets]+" OPTIONS");
							Dialog.setInsets(0,30,0);
							Dialog.addChoice("Convert to:", formats,"TIFF");
							Dialog.setInsets(0,30,0);
							Dialog.addNumber("Initial Image",1);
							Dialog.setInsets(0,30,0);
							Dialog.addNumber("Number of Images", filperchan[imcount]);
							if(raw==imageoption[0])
								sumins+=dirlen[sets];
						}
						Dialog.show();	
						usrformats=newArray(setnum);
						imagestart=newArray(setnum);
						imagenumber=newArray(setnum);
						for(sets=0;sets<setnum;sets++){
							usrformats[sets]=Dialog.getChoice();
							imagestart[sets]=Dialog.getNumber();
							imagenumber[sets]=Dialog.getNumber();;
						}
					}
					////////////Averaging Option Dialog
					if((averager==true) && (poloop==0)){
						Dialog.create("Image Averaging Options");
						avgsumins=0;
						for(sets=0;sets<avgsetnum;sets++){
							Dialog.setInsets(15,0,5);
							Dialog.addMessage(setTitles[sets]+" OPTIONS");
							if(raw==imageoption[1] || raw==imageoption[2]){
								Dialog.setInsets(0,30,0);
								Dialog.addCheckbox("Include Merged Averaged Image?",false);
							}
							if(converter==true){
								Dialog.setInsets(0,30,0);
								Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
							}
							if(converter==false){
								Dialog.setInsets(0,30,0);
								Dialog.addNumber("Initial Image",1);
								if(usrstyle==imstyle[0]){
									Dialog.setInsets(0,30,0);
									Dialog.addNumber("Number to average", avgfilperchan[avgsumins]);
								}
								if(usrstyle==imstyle[1]){
									Dialog.setInsets(0,30,0);
									Dialog.addNumber("Number of Image Stacks to open", avgfilperchan[avgsumins]);
									Dialog.setInsets(0,30,0);
									Dialog.addNumber("Number of Slices to Average",3);
								}
							}
							avgsumins+=dirlen[sets];
						}
						Dialog.show();
						if(raw==imageoption[1] || raw==imageoption[2]) merge=newArray(setnum);
						if(converter==true) condelete=newArray(avgsetnum);
						avgstart=newArray(avgsetnum);
						avgnum=newArray(avgsetnum);
						if(usrstyle==imstyle[1])
							slicecount=newArray(avgsetnum);
						for(sets=0;sets<avgsetnum;sets++){
							if(raw==imageoption[1] || raw==imageoption[2])
								merge[sets]=Dialog.getCheckbox();
							if(converter==true){
								condelete[sets]=Dialog.getCheckbox();;
								avgstart[sets]=imagestart[sets];
								avgnum[sets]=imagenumber[sets];
							}
							if(converter==false){
								avgstart[sets]=Dialog.getNumber();;;
								avgnum[sets]=Dialog.getNumber();;;;
							}
							if(usrstyle==imstyle[1])
								slicecount[sets]=Dialog.getNumber();;;;;
						}
					}
					////////////////////
		
					////////////Converter Execution
					if(converter==true){
						if(raw!=imageoption[0]){
							dirlen=newArray(setnum);
							dirlen=listfill(dirlen,1);
							if(raw==imageoption[1] || raw==imageoption[3]){
								sectremove=newArray(chcnt);
								for(i=0;i<chcnt;i++)
									sectremove[i]="C"+i+1+"-";
							}
							if(raw==imageoption[2])
								sectremove=newArray("(red)","(green)","(blue)");
						}	
						
						//if(raw==imageoption[0])
						sumins=0;
						sumimgs=0;
						for(sets=0;sets<setnum;sets++){
							if(raw==imageoption[0])
								imcount=sumins;
							if(raw!=imageoption[0])
								imcount=sets;
							Format=usrformats[sets];
							for(j=0;j<dirlen[sets];j++){
								//setBatchMode(true);
								inlist=Array.slice(Lists,sumimgs,filperchan[imcount]+sumimgs);
								for(i=imagestart[sets]-1; i<imagestart[sets]-1+imagenumber[sets]; i++){
									showProgress(i-imagestart[sets]+1, imagenumber[sets]);
									pathFull=srcs[sets]+mchans[j+sumins]+inlist[i];
									if(raw==imageoption[1])
										run("DCRaw Reader...", "open="+pathFull+" use_temporary_directory white=[Camera white balance] output_colorspace=raw document_mode_without_scaling read_as=[16-bit linear] interpolation=[High-speed, low-quality bilinear] half_size");
									else
										open(pathFull);
									if(averager==false && (splitter==false || raw==imageoption[0])){
										outdir=outputs[j+sumins];
										if(raw==imageoption[0] && dirlen[sets]>1)
											insert="_";
										else
											insert="";
										if(usrstyle==imstyle[0]){
											Titles=getCleanTitle();
											if(Format=="8-bit TIFF" || Format=="GIF")
												convertTo8Bit();
											saveAs(Format, outdir+chnames[j]+insert+Titles);
											close();
										}
										if(usrstyle==imstyle[1]){
											Titles=getTitle();
											Titles=chnames[j]+insert+Titles;
											if (Format=="8-bit TIFF" || Format=="GIF")
												convertTo8Bit();
											run("Image Sequence... ", "dir=&outdir format=&Format name=&Titles");
											close();
										}
									}
									if(averager==true && raw==imageoption[0]){
										outdir=outputs[j+sumins];
										if(raw==imageoption[0] && dirlen[sets]>1)
											insert="_";
										else
											insert="";
										if(usrstyle==imstyle[0]){
											Titles=getCleanTitle();
											if(Format=="8-bit TIFF" || Format=="GIF")
												convertTo8Bit();
											saveAs(Format, outdir+chnames[j]+insert+Titles);
											close();
										}
										if(usrstyle==imstyle[1]){
											Titles=getTitle();
											Titles=chnames[j]+insert+Titles;
											if(Format=="8-bit TIFF" || Format=="GIF")
												convertTo8Bit();
											run("Image Sequence... ", "dir=&outdir format=&Format name=&Titles");
											close();
										}
									}
									if(splitter==true){
										run("Split Channels");
										if(raw!=imageoption[1]){
											if(Format=="8-bit TIFF" || Format=="GIF")
												convertTo8Bit();
										}
										for(k=0;k<chcnt0;k++){
											outdir=outputs[k+3*sets];
											if(raw==imageoption[1] || raw==imageoption[3]){
												///outdir=outputs[k+sumins];
												if(usrstyle==imstyle[0]){
													titles=getCleanTitle();
													removeindex=findmatchindex(titles,sectremove);
													titles=replace(titles,sectremove[removeindex],"");
													//newtitle=chnames[k]+"_"+titles;
													newtitle=chnames[k]+"_"+titles;
													saveAs(Format,outdir+newtitle);
													close();
												}
												if(usrstyle==imstyle[1]){
													numslice=nSlices();
													for(l=0;l<(numslice/3);l++){
														sliceID=3*l+1+k;
														setSlice(sliceID);
														titles=getCleanSliceTitle();
														run("Make Substack...","delete slices=sliceID");
														removeindex=findmatchindex(titles,sectremove);
														newtitle=chnames[k]+"_"+titles;
														saveAs(Format,outdir+newtitle);
														close();
													}
												}
											}
											if(raw==imageoption[2]){
												///outdir=outputs[k+sumins]
												if(usrstyle==imstyle[0]){
													///titles=getCleanTitle();
													titles=getTitle();
													if(endsWith(titles,sectremove[2-k])){
														titles=getCleanTitle();
														newtitle=chnames[k]+"_"+titles;
														print(outdir+newtitle);
														saveAs(Format,outdir+newtitle);
													    close();
													}
												}
												if(usrstyle==imstyle[1]){
													numslice=nSlices();
													for(l=0;l<numslice;l++){
														setSlice(l+1);
														titles=getCleanSliceTitle();
														newtitle=chnames[k]+"_"+titles;
														run("Image Sequence... ", "dir=&outdir format=&Format name=&newtitle");
														close();
													}
												}
											}
										}
									}
								}
								sumimgs+=filperchan[sumins];
							}
							sumins+=dirlen[sets];
						}
					}
		//////////////////////Converter END
		////////////////////////////////////////////////////Averager execution
				if(averager==true){
					if(converter==true){
						if(raw!=imageoption[0]){
							dirlen=newArray(setnum);
							dirlen=listfill(dirlen,chcnt0);
						}
					}
					//setBatchMode(true);
					avgsumins=0;
					for(sets=0;sets<setnum;sets++){
						q=avgstart[sets]-1;
						finavgnum=avgstart[sets]-1+avgnum[sets];
						if(usrstyle==imstyle[1]) finslice=slicecount[sets];
						for(j=0;j<dirlen[sets];j++){
							avgpath=avgins[j+avgsumins];
							
							run("Image Sequence...", "dir=&avgpath start=&q count=&finavgnum sort");
							avgtitle=getTitle();
							selectWindow(avgtitle);
							rename(chnames[j+avgsumins]);
							if(usrstyle==imstyle[0])
								run("Z Project...", "start=&q stop=&finavgnum projection=[Average Intensity]");
							if(usrstyle==imstyle[1])
								run("Z Project...", "start=&q stop=&finslice projection=[Average Intensity]");
							selectWindow("AVG_"+chnames[j+avgsumins]);
							saveAs("TIFF",adest[sets]+"Averages"+"/"+"AVG_"+chnames[j+avgsumins]);
							close();
							close();
		
							if(converter==true){
								if(condelete[sets]==true){
									dellist=getFileList(avgins[j+avgsumins]);
									for(del=0;del<finavgnum;del++)
										File.delete(avgins[j+avgsumins]+dellist[del]);
									File.delete(avgins[j+avgsumins]);
								}
							}
						}
						if(raw==imageoption[1] || raw==imageoption[2]){
							if(merge[sets]==true){
								for(k=0;k<incnt;k++)
									open(adest[sets]+"Averages"+"/"+"AVG_"+chnames[k]+".tif");
								run("Merge Channels...", "red=AVG_R.tif green=AVG_G.tif blue=AVG_B.tif gray=*None* create");
								saveAs("TIFF",adest[sets]+"Averages"+"/"+"merged_avg");
								close();
							}
						}
						avgsumins+=dirlen[sets];
						if(converter==true){
							if(condelete[sets]==true)
								File.delete(adest[sets]+"Converted");
						}
					}
				}
		/////////////////////Averager portion END
										//////////Standard Extinction Analysis Portion of macro
				if(extanalysis==true && specrec==false){
					finpath=polbase+"ProcessedImages";
					File.makeDirectory(finpath);
					finpath=finpath+"/";
					//if(converter==true || averager==true)
						//extchoices=raw;
					if(exproute==expmode[0]){
						setstrt=0;
						setend=3;
					}
					else if(exproute==expmode[1]){
						setstrt=0;
						if(extdarkchoice==darkBGsubopt[0] || extdarkchoice==darkBGsubopt[2])
							setend=4;
						else if(extdarkchoice==darkBGsubopt[1])
							setend=5;
					}
					else if(exproute==expmode[2]){
						setstrt=1;
						if(extdarkchoice==darkBGsubopt[0] || extdarkchoice==darkBGsubopt[2])
							setend=4;
						else if(extdarkchoice==darkBGsubopt[1])
							setend=5;
					}
					extsets=setend-setstrt;
					//extsets=3;
					//if(darkdefault==true){
					//	extsets=4;
					//	if(darkBGdefault==true)
					//		extsets=5;
					//}
					extsrcs=newArray(extsets);
					for(sets=setstrt;sets<setend;sets++){
						if(File.exists(dest[sets]+"Averages") && File.isDirectory(dest[sets]+"Averages")){
							extsrcs[sets]=dest[sets]+"Averages/";
							prelist=getFileList(dest[sets]+"Averages");
							if(sets==0){
								exttest=0;
								Lists=prelist;
								channels=newArray(Lists.length);
								for(l=0;l<Lists.length;l++){
									buff=getNewStr(Lists[l],"_",".");
									channels[l]=buff+"/";
								}
								extinter=newArray(channels.length);
								for(l=0;l<channels.length;l++){
									extinter[l]="";
								}
							}
							else
								Lists=Array.concat(Lists,prelist);
						}
						else if(File.exists(dest[sets]+"Converted") && File.isDirectory(dest[sets]+"Converted")){
							extinter="";
							extsrcs[sets]=dest[sets]+"Converted/";
							prelist=getFileList(dest[sets]+"Converted");
							if(sets==0 && File.isDirectory(extsrcs[sets]+"/"+prelist[0])){
								exttest=1;
								channels=newArray(prelist.length);
								for(j=0;j<prelist.length;j++)
									channels[j]=prelist[j];
								extinter=newArray(channels.length);
								for(l=0;l<channels.length;l++){
									extinter[l]="";
								}
							}
							else{
								exttest=2;
								interlist=newArray(prelist[0]);
								if(sets==0){
									Lists=interlist;
									channels=newArray(extsets);
								}
								else 
									Lists=Array.concat(Lists,interlist);
								extinter=newArray(channels.length);
								for(l=0;l<channels.length;l++){
									extinter[l]="";
								}
							}
						}
						else{
							if(raw==imageoption[0]){
								extsrcs[sets]=dest[sets];
								prelist=getFileList(dest[sets]);
							}
							else{
								extsrcs[sets]=srcs[sets];
								prelist=getFileList(srcs[sets]);
							}
							if(sets==0){
								if(File.isDirectory(extsrcs[sets]+prelist[0])){
									exttest=3;
									channels=newArray(prelist.length);
									for(j=0;j<prelist.length;j++)
										channels[j]=prelist[j];
									extinter=channels;
								}
								else{
									exttest=4;
									if(sets==0){
										interlist=newArray(extsets);
										interlist[0]=prelist[0];
										Lists=interlist;
										channels=newArray(extsets);
										extinter=newArray(channels.length);
										for(l=0;l<channels.length;l++){
											extinter[l]="";
										}
									}
									else{
										interlist[sets]=prelist[sets];
										Lists=Array.concat(Lists,interlist);
									}
								}
							}
						}
					}
					if(exttest==1 || exttest==3){
						Lists=newArray(extsets*channels.length);
						inexten=newArray(".tif",".tiff",".jpg", ".jpeg",".gif",".png",".pgm",".bmp",".fits",".txt",".zip",".raw");
						for(sets=setstrt;sets<setend;sets++){
							for(j=0;j<channels.length;j++){
								prelist=getFileList(extsrcs[sets]+"/"+channels[j]);
								interlist=duallistrebuilder(prelist,inexten);
								Lists[sets*channels.length+j]=interlist[0];
								//if(sets==0){
								//	Lists[0]=prelist[0];
								//	Array.print(Lists);
								//}
								//else
									//Lists=Array.concat(Lists,interlist);
							}
						}
					}
					//setBatchMode(true);
					sumins=0;
					//if(channels.length==1)
						//colinsert="";
					//else
					colinsert="_";
					if(polarnum==1)
						polinsert="";
					else
						polinsert="_";
					namchans=stripchar(channels,"/");
					if(exproute!=expmode[2])
						srcindx=newArray(0,1,2,3,4);
					else{
						srcindx=newArray(0,0,1,2,3);
					}
					colen=channels.length;
					for(j=0;j<colen;j++){
						open(extsrcs[srcindx[2]]+extinter[j]+Lists[j+srcindx[2]*colen]);
						rename("BG");
						if(exproute!=expmode[2]){
							//Open focus image and subtract BG
						 	open(extsrcs[srcindx[0]]+extinter[j]+Lists[j+srcindx[0]*colen]);
						 	rename("focus");
							imageCalculator("Subtract create 32-bit","focus","BG");
							selectWindow("focus");
							close();
						}
						//Open reference image and subtract BG
						open(extsrcs[srcindx[1]]+extinter[j]+Lists[j+srcindx[1]*colen]);
						rename("ref");
						imageCalculator("Subtract create 32-bit","ref","BG");
						selectWindow("ref");
						close();
						selectWindow("BG");
						close();
						if(exproute!=expmode[0])
							saveAs("Tiff", BASEdirec+reffolder+"/"+"Reference"+colinsert+chans[j]+polinsert+polnames[poloop]);
						newdef=getTitle();
						
						if(exproute!=expmode[0]){
							//Open darkfield image without BG correction (if requested), convert to 32-bit, save image
							if(extdarkchoice==darkBGsubopt[0]){
								open(extsrcs[srcindx[3]]+extinter[j]+Lists[j+srcindx[3]*colen]);
								rename("darkfield");
								run("32-bit");
								selectWindow("darkfield");
								//saveAs("Tiff", finpath+namchans[j]+colinsert+"Darkfield"+polinsert+buffpol);
								//close();
							}
							//Open darkfield image, and subtract darkBG image (if requested)
							if(extdarkchoice==darkBGsubopt[1]){
								open(extsrcs[srcindx[3]]+extinter[j]+Lists[j+srcindx[3]*colen]);
								rename("darkfield");
								open(extsrcs[srcindx[4]]+extinter[j]+Lists[j+srcindx[4]*colen]);
								rename("darkBG");
								imageCalculator("Subtract create 32-bit","darkfield","darkBG");
								//selectWindow("Result of darkfield");
								//saveAs("Tiff", finpath+namchans[j]+colinsert+"Darkfield"+polinsert+buffpol);
								//close();
								selectWindow("darkfield");
								close();
								selectWindow("darkBG");
								close();
								selectWindow("Result of darkfield");
								rename("darkfield");
							}
							//Open darkfield, convert to 32-bit, subtract numeric offset (if requested), save image
							if(extdarkchoice==darkBGsubopt[2]){
								open(extsrcs[srcindx[3]]+extinter[j]+Lists[j+srcindx[3]*colen]);
								rename("darkfield");
								run("32-bit");
								run("Subtract...", "value=DBGoffset");
								selectWindow("darkfield");
								//saveAs("Tiff", finpath+namchans[j]+colinsert+"Darkfield"+polinsert+buffpol);
								//close();
							}
							imageCalculator("Divide create 32-bit", "darkfield", newdef); //normalize darkfield image using brightfield reference
							selectWindow("darkfield");
							close();
							selectWindow("Result of darkfield");
							saveAs("Tiff", finpath+"Darkfield"+colinsert+chans[j]+polinsert+polnames[poloop]);
							close();
						}
						if(exproute!=expmode[2]){
							//Extinction Analysis
							imageCalculator("Divide create 32-bit", "Result of focus", newdef);
							selectWindow("Result of focus");
							close();
							selectWindow(newdef);
							close();
							selectWindow("Result of Result of focus");
							run("Multiply...", "value=-1");
							run("Add...", "value=1");
							rename(extinter[j]+"Extinction");
							selectWindow(extinter[j]+"Extinction");
							//saveAs("Tiff", BASEdirec+"Extinction"+"/"+namchans[j]+colinsert+polnames[poloop]+polinsert+"Extinction");
							saveAs("Tiff", finpath+"Extinction"+colinsert+chans[j]+polinsert+polnames[poloop]);
							close();
						}
					}
				}	////////////Standard extinction portion end
			} 	////End polarization loop
			setBatchMode(false);
		}
		if(specrec==true){/////////////////////Begin special record development of images
//			if(polarization==false)
//				poldef=false;
			Dialog.create("Acquisition Parameter Inputs");
			Dialog.setInsets(15,15,15);
			Dialog.addMessage("(1) Save Options");
			Dialog.setInsets(0,52,0);
			Dialog.addCheckbox("(i) Save all extinction timepoints?",false);
			if(exproute!=expmode[0]){
				Dialog.setInsets(0,52,0);
				Dialog.addCheckbox("(ii) Save all darkfield timepoints?",false);
			}
			Dialog.setInsets(15,15,5);
			Dialog.addMessage("(2) Image nomenclature:");
			Dialog.setInsets(0,0,2);
			Dialog.addString("(i) Input brightfield identifier (name/prefix/suffix)            ", "bright",10);
			Dialog.setInsets(0,65,2);
			Dialog.addString("(ii) Input brightfield offset identifier (name/prefix/suffix)", "BG",10);
			if(exproute!=expmode[0]){
				Dialog.setInsets(0,0,2);
				Dialog.addString("(iii) Input darkfield identifier (name/prefix/suffix)             ", "dark",10);
				if(extdarkchoice==darkBGsubopt[1]){
					Dialog.setInsets(0,0,2);
					Dialog.addString("(iv) Input darkfield offset identifier (name/prefix/suffix)  ", "dbg",10);
				}
			}
			Dialog.setInsets(15,0,0);
			Dialog.addCheckbox("(3) Single record for all polarization angles (if applicable)?",polarization);
			Dialog.setInsets(15,15,5);
			Dialog.addMessage("(4) Spectral acquisition parameters:");
			Dialog.setInsets(0,54,2);
			Dialog.addCheckbox("(i) Single record for all color channels?",true);
			Dialog.setInsets(0,0,2);
			Dialog.addString("(ii) Input Color channels                                                    ","450,500,550",10);
			if(exproute!=expmode[2]){
				Dialog.setInsets(0,0,2);
				Dialog.addNumber("(iii) # brightfield images taken per position                         ",128,0,10,"");
				Dialog.setInsets(0,0,2);
				Dialog.addString("(iv) # brightfield repetitions per channel                         ","4,2,2",10);
			}
			if(exproute!=expmode[0]){
				Dialog.setInsets(0,0,2);
				Dialog.addString("(v) # darkfield repetitions per channel                           ","16,4,4",10);
			}
			Dialog.show();
			tp=Dialog.getCheckbox();
			if(exproute!=expmode[0])
				tpd=Dialog.getCheckbox();
			bstr=Dialog.getString();
			offstr=Dialog.getString();
			if(exproute!=expmode[0]){
				dstr=Dialog.getString();
				if(extdarkchoice==darkBGsubopt[1])
					doffstr=Dialog.getString();
			}
			polrec=Dialog.getCheckbox();
			colrec=Dialog.getCheckbox();
			colorstr=Dialog.getString();
			if(exproute!=expmode[2]){
				posnum=Dialog.getNumber();
				colnum=Dialog.getString();
			}
			else
				colnum="0,0,0,0,0,0,0,0,0,0";
			if(exproute!=expmode[0])
				dcolnum=Dialog.getString();
			else
				dcolnum="0,0,0,0,0,0,0,0,0,0";	
			
			//if(polarnum>1 && polrec==false && colrec==true){
			//	waitForUser("You have indicated that the experiment was performed as a single recording over the color channels but\nas separate recordings over polarizer angles. This is not allowable in this version of Extinction Suite.\nThe allowable options are:\n(1) A separate recording per polarizer angle per colour.\n(2) A single recording over all polarizer angles per color channel.\n(3) A single recording over all polarizer angles and color channels,\nwith rotation of angles first, then change of color channel following.");
			//}
			
			if(exproute==expmode[0]){
				dstr="dark"; /////dummy variable if not applicable
				doffstr="dbg";///as above
			}
			else{
				refpath=polbase+"Reference";
				File.makeDirectory(polbase+"Reference");
				refpath=refpath+"/";
			}
			
			sigsuf=newArray(bstr,dstr);
			bgsuf=newArray(offstr,doffstr);
			
			signam=newArray("Extinction","Darkfield");
			
			if(exproute!=expmode[2]){
				slstrt=0;
				if(exproute==expmode[0])
					slend=1;
				else
					slend=2;
			}
			else {
				slstrt=0;
				slend=2;
			}
			chans=CSV(colorstr);
			//chans=Array.reverse(chans);
			colen=chans.length;
			list=getFileList(polbase);
			list=listrebuilder(list,".tif");
			finpath=polbase+"ProcessedImages";
			if(File.exists(finpath)==false)
				File.makeDirectory(finpath);
			finpath=finpath+"/";
			rawpath=polbase+"Raw";
			if(File.exists(rawpath)==false){
				File.makeDirectory(rawpath);
				for(i=0;i<list.length;i++){
					File.rename(polbase+list[i],rawpath+"/"+list[i]);
				}
			}
			else{
				olist=getFileList(rawpath);
				if(olist.length==0){
					for(i=0;i<list.length;i++){
						File.rename(polbase+list[i],rawpath+"/"+list[i]);
					}
				}
			}
			rawpath=rawpath+"/";
			list=getFileList(rawpath);
			
			if(polarnum>1 && colen>1 && (colrec==true || polrec==true)){
				exporder=newArray("Polarizer rotated before swap of spectral range","Spectral range swapped before rotation of polarizer");	
				Dialog.create("Order of recording");
				Dialog.addMessage("User has indicated that experiments were performed for multiple polarizer angles and\nmultiple spectral ranges, with a single recording over at least one of these modes.\nPlease choose the order of these operations below.");
				Dialog.addChoice("Choose order of recording:",exporder,exporder[0]);
				Dialog.show();
			
				recorder=Dialog.getChoice();
				if(recorder==exporder[0]){
					recdef=true;
					outer=colen;
					inner=polarnum;
				}
				else{
					recdef=false;
					outer=polarnum;
					inner=colen;
				}
			}
			else{
				recdef=true;
				outer=colen;
				inner=polarnum;
			}
			
			if((colrec==true && polrec==true) || (colrec==true && polarnum==1))
				char="";
			else
				char="_";
			
			bglens=newArray(2);
			bgcontent=newArray(2);
			for(sl=slstrt;sl<slend;sl++){
				bglist=listRebwExt(list,bgsuf[sl],char,true,true,".tif");
				bglens[sl]=bglist.length;
				bgcontent[sl]=bglist[0];
			}
			if((bglens[0]==1 && bgcontent[0]==0) || (bglens[1]==1 && bgcontent[1]==0)){
				Dialog.create("Background subtraction options")
				if(bglens[0]==1 && bgcontent[0]==0){
					Dialog.addMessage("No image with a title matching the name input for brightfield sensor-offset was found.\nPlease input a value below to be subtracted instead; 0 is valid.");
					Dialog.addNumber("Sensor offset (brightfield):",0);
				}
				if(bglens[1]==1 && bgcontent[1]==0){
					Dialog.addMessage("No image with a title matching the name input for darkfield sensor-offset was found.\nPlease input a value below to be subtracted instead; 0 is valid.");
					Dialog.addNumber("Sensor offset (darkfield):",0);
				}
				if((bglens[0]==1 && bgcontent[0]==0) && (bglens[1]==1 && bgcontent[1]==0)){
					Dialog.addMessage("No images with titles matching the names input for brightfield/darkfield sensor-offsets were found.\nPlease input a value below to be subtracted instead; 0 is valid.");
					Dialog.addNumber("Sensor dark offset (brightfield):",0);
					Dialog.addNumber("Sensor dark offset (darkfield):",0);
				}
				Dialog.show();
			
				if((bglens[0]==1 && bgcontent[0]==0) && (bglens[1]>=1 && bgcontent[1]!=0)){
					bgoff=Dialog.getNumber();
					dbgoff=false;
				}
				else if((bglens[0]>=1 && bgcontent[0]!=0) && (bglens[1]==1 && bgcontent[1]==0)){
					bgoff=false;
					dbgoff=Dialog.getNumber();
				}
				else if((bglens[0]==1 && bgcontent[0]==0) && (bglens[1]==1 && bgcontent[1]==0)){
					bgoff=Dialog.getNumber();
					dbgoff=Dialog.getNumber();;
				}
			}
			else{
				bgoff=false;
				dbgoff=false;
			}
			offsets=newArray(bgoff,dbgoff);
			
			//reps=Array.reverse(reps);
			
			temp=polbase+"temp";
			File.makeDirectory(temp);
			temp=temp+"/";
			
			if(polarnum==1)
				char1="";
			else
				char1="_";
			timedirec=newArray(polarnum*colen*slend);
			
			setBatchMode(true);
			for(sl=slstrt;sl<slend;sl++){
				inlist=listRebwExt(list,sigsuf[sl],char,true,true,".tif");
				if(sl==0)
					clexps=colnum;
				else if (sl==1)
					clexps=dcolnum;
				reps=CSV(clexps);
				//if(polrec==true || colrec==true)
				//	reps=listparse(reps);/////when either imaging of polarizer angles or imaging of color channels, or both, are recorded in a single stack, this is an array of the number of repetitions per color channel/polarizer angle with each index corresponding to color channel 
				//else
				//	reps=arrfill(colen,1);/////when each color and each polarizer angle is recorded separately number of repetitions treated as 1  with each index corresponding to a color channel
				reps=listparse(reps); //above 4 lines changed to this on 01/08/2020. Unclear previous reasoning. This definition needs to be examined again along with lines 4386-4391 & 4504 & 4513
				if(sl==0){ // if statement on 26/03/2021 for v4.1.2
					for(ii=0;ii<reps.length;ii++){
						//if((polrec==true && colrec==true) || (polrec==true && colrec==false))
						//	reps[ii]=4*reps[ii]/polarnum;
						//else if((polrec==false && colrec==true) || (polrec==false && colrec==false)) //////removed in v4.0.7 the definition of reps in the line above is outdated, and based on the expectation it does not originate from an input value from the user.
							reps[ii]=4*reps[ii];
					}
					numext=newArray(colen);
					//posnum=CSV(posnum);
					//posnum=listparse(posnum);
					for(i=0;i<colen;i++)
						numext[i]=reps[i]*posnum/2;
				}
				else{// as in if statement above, 26/03/2021 for v4.1.2
					numext[i]=reps[i];
				}
				if((colrec==true && polrec==true) || (colrec==true && polarnum==1) || (polrec==true && colen==1) || (colrec==false && polrec==false && polarnum==1 && colen==1)){
					prim="sigstack";
					open(rawpath+inlist[0]);
					rename(prim);
					stindx=subind(outer,inner,reps);
					ind1=Array.slice(stindx,0,stindx.length/2);
					ind2=Array.slice(stindx,stindx.length/2,stindx.length);
				}
				
				if(offsets[sl]==false){
					bgti="bgstack";
					bgavti="AvgBG";
					bglist=listRebwExt(list,bgsuf[sl],char,true,true,".tif");
					bgavti=arrindstr(bgavti,bglist.length);
					for(ii=0;ii<bglist.length;ii++){
						open(rawpath+bglist[ii]);
						rename(bgti);
						numslice=nSlices();
						if(numslice==1)
							rename(bgavti[ii]);
						else
							avgbgst(bgti,bgavti[ii]);
					}
				}
				if((colrec==true && polrec==true) || (colrec==true && polarnum==1) || (polrec==true && colen==1) || (colrec==false && polrec==false && polarnum==1 && colen==1)){
					sigti="subsigstack";
					if(offsets[sl]==false){
						imageCalculator("Subtract create 32-bit stack", prim, bgavti[0]);
						rename(sigti);
						wait(100);
						close(prim);
						close(bgavti[0]);
					}
					else{
						run("Subtract...", "value="+offset[sl]);
						rename(sigti);
					}
				}
				for(ii=0;ii<colen;ii++){
					for(jj=0;jj<polarnum;jj++){
						File.makeDirectory(polbase+signam[sl]+"_timepoints_"+chans[ii]+char1+polnames[jj]);
						timedirec[jj+ii*polarnum+sl*colen*polarnum]=polbase+signam[sl]+"_timepoints_"+chans[ii]+char1+polnames[jj]+"/";
					}
				}
				towrite="subtracted_";
				for(ii=0;ii<outer;ii++){
					if((colrec==true && polrec==false && polarnum>1) || (colrec==false && polrec==true && colen>1)){
						if(colrec==true && polrec==false && polarnum>1)
							name1=sigsuf[sl]+"_"+polnames[ii];
						else if(colrec==false && polrec==true && colen>1)
							name1=sigsuf[sl]+"_"+chans[ii];
						open(rawpath+name1+".tif");
						sigti="sigstack";
						rename(sigti);
						reparr=newArray(1); //added v4.0.7
						reparr[0]=reps[ii]; //added v4.0.7
						stindx=subind(1,inner,reparr); //modified v4.0.7
						ind1=Array.slice(stindx,0,stindx.length/2);
						ind2=Array.slice(stindx,stindx.length/2,stindx.length);
					}
					if((colrec==false && polrec==true) || (colrec==false && polrec==false)){
						if(recdef==true && offsets[sl]==false && bglist.length>1)
							subbg=bgavti[ii];
						else if(recdef==true && offsets[sl]==false && bglist.length==1)
							subbg=bgavti[0];
					}
					for(jj=0;jj<inner;jj++){
						if(colrec==true && polrec==false){
							if(recdef==false && offsets[sl]==false && bglist.length>1)
								subbg=bgavti[jj];
							else if(recdef==false && offsets[sl]==false && bglist.length==1)
								subbg=bgavti[0];
						}
						if(colrec==true && polrec==true){ //(colrec==true || polrec==true && polarnum|colen>1){   ///modified in v4.0.7
							selectWindow(sigti);
							run("Make Substack...", "channels=1 slices="+ind1[jj+ii*inner]+"-"+ind2[jj+ii*inner]);
							//rename(towrite);
						}
						else if(((colrec==true && polrec==false) || (colrec==false && polrec==true)) && inner>1){// added in v4.0.7
							selectWindow(sigti);                                                                //added in v4.0.7
							run("Make Substack...", "channels=1 slices="+ind1[jj]+"-"+ind2[jj]);               //added in v4.0.7
						}                                                                                     //added in v4.0.7
						else if(((colrec==true && polrec==false) || (colrec==false && polrec==true)) && inner==1){//added in v4.1.1
							selectWindow(sigti);                                                                  //added in v4.1.1
							run("Make Substack...", "channels=1 slices="+ind1[ii]+"-"+ind2[ii]);                 //added in v4.1.1
						}                                                                                       //added in v4.1.1
						if(colrec==false && polrec==false){
							if(recdef==true){
								name1=chans[ii];
								if(polarization==true)
									name2=buffer(polend,polnums[jj]);
								else
									name2="";
							}
							else{
								name1=chans[jj];
								if(polarization==true)
									name2=buffer(polend,polnums[jj]);
								else
									name2="";
							}
							open(rawpath+sigsuf[sl]+"_"+name1+char1+name2+".tif"); //added .tif on 01/08/2020
							//rename(jj+ii*inner);
						}
						rename("hold");
						if((colrec==true && polrec==false && polarnum>1) || (colrec==false && polrec==false) || (colrec==false && polrec==true && colen>1)){
							if(offsets[sl]==false){
								imageCalculator("Subtract create 32-bit stack", "hold", subbg);
								//selectWindow(jj+ii*inner);
								//rename(towrite);
								close("hold");
							}
							else
								run("Subtract...", "value="+offset[sl]);
						}
						rename(towrite);
						run("Image Sequence... ", "dir="+temp+" format=TIFF name="+towrite+"_"+" use");
						//run("Image Sequence... ", "format=TIFF save=temp");
						close(towrite);
						tlist=getFileList(temp);
						
						if(recdef==true){
							ntemp=reps[ii]/4;
							cname=chans[ii];
							if(polarization==true)
								pname=buffer(polend,polnums[jj]);
							else
								pname="";
							idxmulti=polarnum;
						}
						else{
							ntemp=reps[jj]/4;
							cname=chans[jj];
							if(polarization==true)
								pname=buffer(polend,polnums[ii]);
							else
							pname="";
							idxmulti=colen;
						}
						if(polarnum>1)
							tchar="_";
						else
							tchar="";
						if(sl==0){
							if(exproute!=expmode[2])
								syncext(temp,ntemp,tlist,cname,tchar,pname,finpath,tp,timedirec[jj+ii*idxmulti+sl*polarnum*colen]);
							if(exproute==expmode[1])
								syncref(temp,ntemp,tlist,cname,tchar,pname,refpath);
							else if(exproute==expmode[2])
								stref(temp,list,cname,char,pname,refpath);
						}
						else if(sl==1 && exproute!=expmode[0])
							stdark(temp,tlist,cname,char,pname,refpath,finpath,tpd,timedirec[jj+ii*idxmulti+sl*polarnum*colen]);
						for(tt=0;tt<tlist.length;tt++){
							File.delete(temp+tlist[tt]);
						}
					}
					if((colrec==true && polrec==false && polarnum>1) || (colrec==false && polrec==true && colen>1)) //moved into ii loop in v4.0.7  //changed on 01/08/2020 //((colrec==true && polrec==false && polarnum>1) || (colrec==false && polrec==false) || (colrec==false && polrec==true && colen>1))
						close(sigti); //moved into ii loop in v4.0.7
				}
				if((colrec==true && polrec==false && polarnum>1) || (colrec==false && polrec==false) || (colrec==false && polrec==true && colen>1) && (offsets[sl]==false)){
					for(ii=0;ii<bglist.length;ii++)
						close(bgavti[ii]);
				}
			}
			if((colrec==true && polrec==true) || (colrec==true && polarnum==1) || (polrec==true && colen==1) || (colrec==false && polrec==false && polarnum==1 && colen==1))
				close(sigti);
			File.delete(temp);
		}
	}	//////////End Converter/Averager/Extinction	
/////////////////////////////////////////////////////////////////////Begin Particle analysis section
	if(partanalysis==true){
		if(extanalysis==true){///////////////////////These options are presented only if particle analysis begins immediately after extinction/darkfield development during the same run of the macro.
			adef=exproute;
			amode=newArray("Extinction", "Extinction + Darkfield", "Darkfield");
			bgmeth=newArray("Defocus","Shifted Sample");
			if(usetxt==false || txtexst==false){
				Dialog.create("Analysis Options");
				Dialog.addChoice("Choose Analysis route",amode, adef);
				Dialog.addChoice("Choose background subtraction method",bgmeth,bgmeth[1]);
				Dialog.addNumber("Proximity restriction (n*Ri): ",2);
				Dialog.addNumber("Desired Number of BG datapoints to examine",1000);
				//Dialog.addNumber("Number of particle species/nominal sizes expected", 1);
				Dialog.setInsets(30,20,10);
				if(specrec==false)
					Dialog.addString("Total # brightfield images taken per channel", "128,128,128",30);
				Dialog.addMessage("Input ranks of colors. Highest rank is 1. Colors may have equal rank.\nLowest equivalent rank colors may be left at an arbitrarily high value, e.g. 1000.\nIf there is only 1 channel, use 1 for its rank.");
				Dialog.addString("Input ranks of channels","1,1,1",30);
				Dialog.addNumber("Numerical Aperture",0.95);
				Dialog.addNumber("Magnification",40);
				Dialog.addNumber("Pixel Pitch um)",6.5);
				Dialog.addNumber("Full well capacity",30000);
				if(polarization==true)
					Dialog.addNumber("Specify Number of Polarization Fit Noise Simulations",100);
				Dialog.setInsets(15,0,10);
				Dialog.addCheckbox("Perform time series analysis?",false);
				Dialog.addCheckbox("Perform Gaussian fit to particle intensity\nprofile for improved coordinate centering?",true);
				if(adef!=expmode[0]){
					Dialog.addNumber("Darkfield scaling parameter, Tau",1);
					Dialog.addNumber("Darkfield scaling parameter, Zeta",1);
					Dialog.addNumber("Darkfield scaling parameter, Eta",1);
				}
				if(polarization==false){
					Dialog.setInsets(30,20,10);
					Dialog.addMessage("Calibration for new optics");
					Dialog.addCheckbox("Calibrate the particle measurement radius (Ri)?",false);
				}
				Dialog.show();
				aroute=Dialog.getChoice();
				bgroute=Dialog.getChoice();
				usrspc=Dialog.getNumber();
				datapoints=Dialog.getNumber();
				//NPsize=Dialog.getNumber();
				if(specrec==false)
					numext=Dialog.getString();
				ranks=Dialog.getString();
				NA=Dialog.getNumber();
				mag=Dialog.getNumber();
				pixs=Dialog.getNumber();
				fwc=Dialog.getNumber();
				if(polarization==true)
					polfits=Dialog.getNumber();
				else
					polfits=0;
				timereg=Dialog.getCheckbox();
				gaussfit=Dialog.getCheckbox();
				if(aroute!=amode[0]){
					tau=Dialog.getNumber();
					zeta=Dialog.getNumber();
					eta=Dialog.getNumber();
				}
				if(polarization==false)
					radiuscalibration=Dialog.getCheckbox();
				else
					radiuscalibration=false;

				if (specrec==true){
					nextstr=d2s(2*numext[0],0); 
					for(i=1;i<chans.length;i++) 
						nextstr=nextstr+","+d2s(2*numext[i],0); //addded comma in v4.1.5
				}
				else{
					nextstr=numext;
				}
				strcnt=dualprint("Analysis path:\t"+aroute,parfile,"",0,strcnt); //0 parameter entry ID
				strcnt=dualprint("BG subtraction type:\t"+bgroute,parfile,"",0,strcnt); //1
				strcnt=dualprint("Proximity restriction, N*Ri, N=\t"+d2s(usrspc,2),parfile,"",0,strcnt); //2
				strcnt=dualprint("# BG datapoints taken=\t"+d2s(datapoints,0),parfile,"",0,strcnt); //3
				strcnt=dualprint("Total # brightfield images taken per channel:\t"+nextstr,parfile,"",0,strcnt); //4
				//print("# nominal particle sizes expected in sample= "+NPsize);
				strcnt=dualprint("Center wavelength of spectral ranges:\t"+colorstr,parfile,"",0,strcnt); //5
				strcnt=dualprint("Ranks of spectral ranges:\t"+ranks,parfile,"",0,strcnt); //6
				strcnt=dualprint("Numerical aperture=\t"+d2s(NA,3),parfile,"",0,strcnt); //7
				strcnt=dualprint("Magnification=\t"+d2s(mag,1),parfile,"",0,strcnt); //8
				strcnt=dualprint("Pixel size=\t"+d2s(pixs,3),parfile,"",0,strcnt); //9
				strcnt=dualprint("Full well capacity=\t"+d2s(fwc,0),parfile,"",0,strcnt); //10
				strcnt=dualprint("Number of simulations for polarization fits=\t"+d2s(polfits,0),parfile,"",0,strcnt); //11
				strcnt=dualprint("Perform time series analysis?\t"+timereg,parfile,"",0,strcnt); //12
				strcnt=dualprint("Used Gaussian Fit for Particle Center Coordinates?\t"+gaussfit,parfile,"",0,strcnt); //13
				strcnt=dualprint("Ri dependence examined?\t"+radiuscalibration,parfile,"",0,strcnt); //14
	//			if(aroute!=amode[0]){
	//				dualprint("Tau= "+tau);
	//				dualprint("Zeta= "+zeta);
	//				dualprint("Eta= "+eta);	
	//			}
	//			else(aroute==amode[0]){
	//				tau=1;
	//				zeta=1;
	//				eta=1;
	//			}
				if(specrec==false){
					numext=CSV(numext); //moved from above the print-to-file section in v4.1.5 into if statement here
					numext=listparse(numext); //moved from above the print-to-file section in v4.1.5 into if statement here
					for(i=0;i<chans.length;i++){
						numext[i]=numext[i]/2;
					}
				}
			}

			ranks=CSV(ranks);
			ranks=listparse(ranks);
			if(aroute!=amode[0])
				darkdefault=true;
			else
				darkdefault=false;
		}
		/////////////////////////No dialog options appearing after this point appeared in any previous section.
		fitmodel = "y = c*(1+a*cos(2*0.017453292519943*(x-b)))";
		scale=1000*pixs/mag;
		if(bgroute==bgmeth[1])
			refshift=true;
		else
			refshift=false;
		signam=newArray("Extinction","Darkfield");
		colen=chans.length;
		colorarr=colorgen();
		chint=listparse(chans);
		Ri=newArray(colen);
		pcolor=newArray(colen);
		for(i=0;i<colen;i++){
			Ri[i]=3*chint[i]/(2*NA);
			hexcols=getHex(chint[i],colorarr);
			pcolor[i]=hexcols;
		}
		
		if(usetxt==false || txtexst==false){
			radius=newArray(colen);
			radius2=newArray(colen);
			
			Dialog.create("Additional analysis options");
			Dialog.addCheckbox("Scale results in units of area?",true);
			if(timereg==true){
				Dialog.addNumber("Number of peaks to identify pattern (registration)",1);
				timemods=newArray("Exponential decay: I=I0•exp(-t/T)","Linear: y=at+b");
				Dialog.addChoice("Select model to fit time-series data",timemods,timemods[0]);
			}
			if(aroute!=amode[0]){
				Dialog.addNumber("Darkfield scaling parameter, Tau",1);
				Dialog.addNumber("Darkfield scaling parameter, Zeta",1);
				Dialog.addNumber("Darkfield scaling parameter, Eta",1);
			}
			if(timereg==1 || (aroute!=amode[0]))
				Dialog.addNumber("Exposure time (s)", 0.005);
			if(refshift==false){
				Dialog.setInsets(10,0,10);
				Dialog.addMessage("Below, input N for sigma exclusion. For the\ndouble radius method, pixels in the outer radius\nwill be excluded from measurement if they\nare outside of the (Mean +/- N*sigma).");
				Dialog.setInsets(10,0,0);
				Dialog.addNumber("N ",2);
			}
			for(i=0;i<colen;i++){
				ripix=Ri[i]*mag/(pixs*1000);
				Dialog.addNumber("Radius of Region of Interest (ROI) for "+chans[i]+"nm channel (pixels)",ripix);
				Dialog.setInsets(0,0,15);
				Dialog.addNumber("Radius of Outer ROI (if applicable) for "+chans[i]+"nm channel (pixels)",2*ripix);
			}
			Dialog.show();
	
			usescale=Dialog.getCheckbox();
			if(timereg==true){
				regpts=Dialog.getNumber();
				timemodcho=Dialog.getChoice();
				if (timemodcho==timemods[0])
					flofitmodel = "y = a*exp(-x/b)";
				else
					flotfitmodel= "y = a*x+b";
			}
			else{
				regpts=0;
				flofitmodel="N/A";
			}
			if(aroute!=amode[0]){
				tau=Dialog.getNumber();
				zeta=Dialog.getNumber();
				eta=Dialog.getNumber();
			}
			else{
				tau=1;
				zeta=1;
				eta=1;
			}
			
			if(timereg==1 || (aroute!=amode[0]))
				exptime=Dialog.getNumber();
			else
				exptime=1;
			if(refshift==false)
				Nsig=Dialog.getNumber();
			else
				Nsig=5;
			for(i=0;i<colen;i++){
				radius[i]=Dialog.getNumber();
				radius2[i]=Dialog.getNumber();
			}
	
			
			if(usescale==true)
				areacal=pow(scale,2);
			else
				areacal=1;
			
			//if(polarization==true)
			//	radiuscalibration=false;
			highranks=rank1(ranks);
			if(highranks.length>1){
				mainchans=newArray(highranks.length);
				for(ii=0;ii<highranks.length;ii++)
					mainchans[ii]=chans[highranks[ii]];
				Dialog.create("");
				Dialog.addMessage("The highest rank value in your rankings occurs for more than one color channel.\nPlease indicate below which you would like to use as main image for peak-\n-finding, calbration, etc.");
				Dialog.addChoice("Color channel used for calibration",chans);
				Dialog.show();
				usermaincol=Dialog.getChoice();
				mainidx=arrayvalueindices(chans,usermaincol);
				mainidx=mainidx[0];
			}
			else{
				mainidx=highranks[0];
			}
			strcnt=dualprint("Polarization Fit Model:\t"+fitmodel,parfile,"",0,strcnt); //15
			strcnt=dualprint("Length scale calibration (nm/pixel)=\t"+scale,parfile,"",0,strcnt); //16
			strcnt=dualprint("Scale integrated signal in area units?\t"+usescale,parfile,"",0,strcnt);//17
			strcnt=dualprint("ROI area =\t"+areacal,parfile,"",0,strcnt);//18
			strcnt=dualprint("Number of points to identify pattern =\t"+d2s(regpts,0),parfile,"",0,strcnt);//19
			strcnt=dualprint("Fit model is:\t"+flofitmodel,parfile,"",0,strcnt);//20
			strcnt=dualprint("Tau=\t"+d2s(tau,3),parfile,"",0,strcnt);//21
			strcnt=dualprint("Zeta=\t"+d2s(zeta,3),parfile,"",0,strcnt);//22
			strcnt=dualprint("Eta=\t"+d2s(eta,3),parfile,"",0,strcnt);//23
			strcnt=dualprint("Exposure time =\t"+d2s(exptime,4),parfile,"",0,strcnt);//24
			strcnt=dualprint("N-sigma=\t"+d2s(Nsig,2),parfile,"",0,strcnt);//25
			fillr="Measurement radius Ri for each channel (pixels):";
			strcnt=dualprint(radius,parfile,fillr,1,strcnt);//26
			fillr="Outer radius for each channel (pixels):";
			strcnt=dualprint(radius2,parfile,fillr,1,strcnt);//27
			strcnt=dualprint("Main color channel index=\t"+mainidx,parfile,"",0,strcnt);//28
		}
		maincol=chans[mainidx];

		//if(channels.length==1)
			//colinsert="";
		//else
		colinsert="_";
		if(polarnum==1)
			polinsert="";
		else
			polinsert="_";
		if(aroute!=amode[2])
			setstrt=0;
		else
			setstrt=1;
		if(aroute==amode[0])
			setend=1;
		else
			setend=2;
		setnum=setend-setstrt;
		if(aroute!=amode[1])
			noisetol1=newArray(colen);
		else
			noisetol1=newArray(colen*2);
		for(j=0;j<colen;j++){
			if(aroute==amode[0])
				noisetol1[j]=10*sqrt(2/(numext[j]*fwc));
			else if (aroute==amode[1]){
				noisetol1[j]=10*sqrt(2/(numext[j]*fwc));
				noisetol1[j+colen]=0.02;
			}
			else
				noisetol1[j]=0.02;
		}

		if (timereg==true){
			procsrc=File.exists(polbase+"ProcessedImages");
			if (procsrc!=true){
				File.makeDirectory(polbase+"ProcessedImages");
				avsav=1;
			}
			else{
				avsav=0;
			}
			timeSave(polbase,finpath,signam,setstrt,setend,chans,polinsert,polnames,polinsert,aroute,amode,radius,avsav);
			run("Clear Results");
		}
		maintitle="Extinction"+colinsert+maincol+polinsert+polnames[0]+".tif";
		mainimage=finpath+maintitle;
		maindtitle="Darkfield"+colinsert+maincol+polinsert+polnames[0]+".tif";
		maindimage=finpath+maindtitle;
		///////////////////////Reference shift determination
		setBatchMode(true);
		if(usetxt==false || txtexst==false){
			if(refshift==false || aroute==amode[2]){
				refshiftx=newArray(1);
				refshifty=newArray(1);
			}
			else{
				shiftSelect=newArray("Auto","Manual"); ////added on 19/04/2021 in v4.1.3
				Dialog.create("Shift determination options");////
					Dialog.addChoice("Select the method of shift distance determination",shiftSelect,shiftSelect[0]);////
					Dialog.addCheckbox("Use the same shift for all color channels?",false);////
				Dialog.show();////
				shiftmeth=Dialog.getChoice();////
				singleshift=Dialog.getCheckbox();////
				if(singleshift==true)////
					shiftlen=1;////
				else////
					shiftlen=colen;////
				
				if(shiftmeth==shiftSelect[0]){////
					if(singleshift==true)////
						shifts=shiftRef(finpath,colinsert,chans,polinsert,polnames[0],noisetol1,noisetol1,0.5,15,mainidx);////
					else////
						shifts=shiftRef(finpath,colinsert,chans,polinsert,polnames[0],noisetol1,noisetol1,0.5,15,1000000);//updated in v4.1.3 on 19/04/2021
					refshiftx=Array.slice(shifts,0,shifts.length/2);
					refshifty=Array.slice(shifts,shifts.length/2,shifts.length);
				}////
				else{////
					refshiftx=newArray(colen);
					refshifty=newArray(colen);
				}////
				
				Dialog.create("Reference shift options:")
					for(i=0;i<shiftlen;i++){////
						if(singleshift==1)////
							shiftindex=mainidx;////
						else////
							shiftindex=i;////
						if(shiftmeth==shiftSelect[0]){////
							if(i==0)////
								Dialog.addMessage("The reference shifts have been automatically characterized. Please examine\nthe results and check that they are reasonable. If so, press return to accept.\nIf not you may manually enter the shift for each channel.");
							Dialog.addNumber("Reference shift in x-direction for "+chans[shiftindex]+"nm channel = ",refshiftx[i]);//argument of chans updated from i to shiftindex in v4.1.3 on 19/04/2021
							Dialog.addNumber("Reference shift in y-direction for "+chans[shiftindex]+"nm channel = ",refshifty[i]);//
						}////
						else{////
							Dialog.addNumber("Reference shift in x-direction for "+chans[shiftindex]+"nm channel = ",-(100+3*chint[shiftindex]/NA)/scale);////
							Dialog.addNumber("Reference shift in y-direction for "+chans[shiftindex]+"nm channel = ",0);////
						}////
					}
				Dialog.show();

				if(singleshift==true){////
					xrefhold=Dialog.getNumber();////
					yrefhold=Dialog.getNumber();////
					for(i=0;i<colen;i++){////
						refshiftx[i]=xrefhold;////
						refshifty[i]=yrefhold;////
					}////
				}////
				else{////
					for(i=0;i<colen;i++){
						refshiftx[i]=Dialog.getNumber();
						refshifty[i]=Dialog.getNumber();
					}
				}////
			}
			fillr="Reference shift in x per color channel:";
			strcnt=dualprint(refshiftx,parfile,fillr,1,strcnt);//29
			fillr="Reference shift in y per color channel:";
			strcnt=dualprint(refshifty,parfile,fillr,1,strcnt);//30
		}
		///////////////////////////////Measurement radius calibration
		if(radiuscalibration==true){
			setBatchMode(false);
			if(aroute!=amode[2]){
				open(mainimage);
				intitle=maintitle;
			}
			else{
				open(maindimage);
				intitle=maindtitle;
			}
			waitForUser("Scan image for a reasonably intense nanoparticle.\nPlace cursor over center of particle and note X and Y coordinates of the brightest pixel\n(appearing in the ImageJ toolbar). The particle should be well isolated.\nWhen ready, close this window and enter values in next dialog.");
			Dialog.create("Measurement radius calibration options");
			Dialog.addNumber("Maximum range over which to check measurement radius (pixels):",100);
			Dialog.addNumber("Particle X-coordinate",0);
			Dialog.addNumber("Particle Y-coordinate",0);
			Dialog.show();
			radext=Dialog.getNumber();
			xopt=Dialog.getNumber();
			yopt=Dialog.getNumber();
			close(maintitle);
			setBatchMode(true);
			radcal(finpath,intitle,xopt,yopt,radext,radius[mainidx],radius2[mainidx],mag,pixs,refshiftx[mainidx],refshifty[mainidx],2);
		}
		/////////////////////Initiate folders
		resdir=polbase+"Results";
		File.makeDirectory(resdir);
		resdir=resdir+"/";
		//procdir=polbase+"Processing";
		//File.makeDirectory(procdir);
		//procdir=procdir+"/";
		if(polarization==true){
			presdir=resdir+"RawStatistics";
			fitdir=resdir+"PolarizationDependentFits";
			fitstat=resdir+"FitStatistics";
			File.makeDirectory(presdir);
			File.makeDirectory(fitdir);
			File.makeDirectory(fitstat);
			presdir=presdir+"/";
			fitdir=fitdir+"/";
			fitstat=fitstat+"/";
			presdirs=newArray(colen);
			fitdirs=newArray(colen);
			fitstats=newArray(colen);
			if(colen>1){
				for(cs=0;cs<colen;cs++){
					presdirs[cs]=presdir+chans[cs];
					File.makeDirectory(presdirs[cs]);
					presdirs[cs]=presdirs[cs]+"/";
					fitdirs[cs]=fitdir+chans[cs];
					File.makeDirectory(fitdirs[cs]);
					fitdirs[cs]=fitdirs[cs]+"/";
					fitstats[cs]=fitstat+chans[cs];
					File.makeDirectory(fitstats[cs]);
					fitstats[cs]=fitstats[cs]+"/";
				}
			}
			else{
				presdirs[0]=presdir;
				fitdirs[0]=fitdir;
				fitstats[0]=fitstat;
			}
		}
		else{
			presdir=resdir; //added in v4.1.2 on 26/03/2021
			presdirs=newArray(1);
			presdirs[0]=resdir;
		}
		if(timereg==true){
			tfitdir=resdir+"TimeDependentFits";
			File.makeDirectory(tfitdir);
			tfitdir=tfitdir+"/";
			tfitdirs=newArray(setnum*colen*polarnum);
			for(sets=setstrt; sets<setend; sets++){
				if(aroute==amode[2])
					in=sets-1;
				else
					in=sets;
				for(cs=0;cs<colen;cs++){
					for(ps=0;ps<polarnum;ps++){
						tfitdirs[ps+cs*polarnum+in*colen*polarnum]=tfitdir+signam[in]+colinsert+chans[cs]+polinsert+polnames[ps];
						File.makeDirectory(tfitdirs[ps+cs*polarnum+in*colen*polarnum]);
						tfitdirs[ps+cs*polarnum+in*colen*polarnum]=tfitdirs[ps+cs*polarnum+in*colen*polarnum]+"/";
					}
				}
			}
			print("Time-registration completed");
		}
		//////////////////////Open main image, determine greater region of interest
		if(usetxt==false || txtexst==false){
			setBatchMode(false);
			if(aroute!=amode[2])
				open(mainimage);
			else
				open(maindimage);
			run("Select None");
			run("Clear Results");
			run("Set Scale...", "distance=0");
			imageH=getHeight();
			imageW=getWidth();
			Dialog.create("ROI Format");
			roiform=newArray("Oval","Rectangle");
			Dialog.addChoice("ROI Format", roiform,roiform[1]);
			Dialog.show();
			roicho=Dialog.getChoice();
			
			if(roicho==roiform[0]){
				makeOval(0.125*imageW,0.125*imageH,0.75*imageW,0.75*imageH);
				waitForUser("Reformat Selection Oval");
			}
			else {
				makeRectangle(0.125*imageW,0.125*imageH,0.75*imageW,0.75*imageH);
				waitForUser("Reformat Selection Rectangle");
			}
			getSelectionBounds(t,r,width,height);
			rectlocx=t;
			rectlocy=r;
			ovalwi=width;
			ovalhe=height;
			centerx=((ovalwi/2)+rectlocx);
			centery=((ovalhe/2)+rectlocy);
			if(aroute!=amode[2])
				close(maintitle);
			else
				close(maindtitle);
			if(roicho==roiform[0])
				shapename="Oval";
			else
				shapename="Rectangle";
			strcnt=dualprint("ROI selection shape:\t"+shapename,parfile,"",0,strcnt);//31
			strcnt=dualprint("X Coordinate of ROI =\t"+rectlocx,parfile,"",0,strcnt);//32
			strcnt=dualprint("Y Coordinate of ROI =\t"+rectlocy,parfile,"",0,strcnt);//33
			strcnt=dualprint("Width of ROI =\t"+ovalwi,parfile,"",0,strcnt);//34
			strcnt=dualprint("Height of ROI =\t"+ovalhe,parfile,"",0,strcnt);//35
			////////////////////////////////////////////open main image and allow user to examine noise tolerance of find maxima function
			for(sets=setstrt;sets<setend;sets++){
				if(sets==0)
					impath=mainimage;
				else
					impath=maindimage;
				if(mag/(2*pixs*1000)>(2*NA)/chint[mainidx]){
					lam=chint[mainidx]-20;
					setBatchMode(true);
					inband(impath,NA,pixs,mag,lam,0.4);
					setBatchMode("exit and display");
					setBatchMode(false);
					saveAs("Tiff",finpath+"InBand_"+signam[sets]);
				}
				else
					open(impath);
				if(roicho==roiform[0])
					makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
				else
					makeRectangle(rectlocx,rectlocy,ovalwi,ovalhe);
				waitForUser(signam[sets]+" Noise Tolerance:\n \nUse 'Find Maxima' ONLY TO PREVIEW selection and calibrate\nnoise tolerance, but do NOT run.\n \nRecord appropriate noise tolerance number such that all\nparticles are marked in the 'Preview' including any debris\nor small scatterers, then hit CANCEL.\n \nDo NOT PRESS 'OK' on this window until finishing those steps.");
				close();
			}
			
			Dialog.create("Particle Finding Options");
			if(aroute==amode[0])
				Dialog.addNumber("Noise tolerance: ",noisetol1[mainidx]);
			else if(aroute==amode[1]){
				Dialog.addMessage("You have just been presented with both extinction and darkfield images in order\nto determine the best noise tolerance for each. You can now choose which\nyou would prefer to use to find determine particle locations. Standard\npractice is to use extinction, but this is at the user's discretion.\nThe decision will carry through to cross-section gating of particles.");
				Dialog.addChoice("Find particles using: ",signam,signam[0]);
				Dialog.addNumber("Enter the noise tolerance according to your selection above: ",noisetol1[mainidx]);
			}
			else if(aroute==amode[2])
				Dialog.addNumber("Noise tolerance: ",noisetol1[mainidx+colen]);
			Dialog.show();
	
			if(aroute==amode[1])
				findhow=Dialog.getChoice();
			usrtol=Dialog.getNumber();
	
			if(aroute==amode[0])
				findhow=signam[0];
			if(aroute==amode[2])
				findhow=signam[1];
	
			strcnt=dualprint("Find particles using:\t"+findhow,parfile,"",0,strcnt);//36
			strcnt=dualprint("Noise tolerance to find peaks =\t"+usrtol,parfile,"",0,strcnt);//37
	///////////////////////Registration
			setBatchMode(true);
			driftx=newArray(setnum*colen*polarnum);
			drifty=newArray(setnum*colen*polarnum);
			regcode=newArray(setnum*colen*polarnum);
			if(setnum>1 || colen>1 || polarnum>1){
				repeat=true;
				regloop=0;
				shiftol=0.5;
				while(repeat==true){
					if(regloop>0){
						exntol=noisetol1[0];
						if(aroute==amode[1])
							exdntol=noisetol1[colen];
						Dialog.create("Registration options");
						if(aroute==amode[0]){
							Dialog.addMessage("You have chosen to RE-register the images. You can now adjust the noise tolerance for\nExtinction as well the tolerance allowable in the shift determinations.");
							Dialog.addNumber("Extinction noise tolerance",noisetol1[0]);
						}
						if(aroute==amode[1]){
							Dialog.addMessage("You have chosen to RE-register the images. You can now adjust the noise tolerance for\nExtinction and Darkfield as well the tolerance allowable in the shift determinations.");
							Dialog.addNumber("Extinction noise tolerance",noisetol1[0]);
							Dialog.addNumber("Darkfield noise tolerance",noisetol1[colen]);
						}
						if(aroute==amode[2]){
							Dialog.addMessage("You have chosen to RE-register the images. You can now adjust the noise tolerance for\nDarkfield as well the tolerance allowable in the shift determinations.");
							Dialog.addNumber("Darkfield noise tolerance",noisetol1[colen]);
						}
						Dialog.addNumber("Shift tolerance",shiftol);
						Dialog.show();
		
						
						if(aroute!=amode[2]){
							noisetol1[0]=Dialog.getNumber();
							if(aroute==amode[1])
								noisetol1[colen]=Dialog.getNumber();
						}
						if(aroute==amode[2])
							noisetol1[colen]=Dialog.getNumber();
						shiftol=Dialog.getNumber();
		
						if(aroute!=amode[2])
							noisetol1=difAdd(exntol,noisetol1,1,colen);
						if(aroute!=amode[0])
							noisetol1=difAdd(exdntol,noisetol1,colen+1,2*colen);
					}
					for(sets=setstrt;sets<setend;sets++){
						if(aroute==amode[2])
							in=sets-1;
						else
							in=sets;
						open(mainimage); //finpath+signam[sets]+"_"+chans[0]+polinsert+polnames[0]+".tif"); altered in v4.1.5 (08/09/2021)
						t1=getTitle();
						for(cs=0;cs<colen;cs++){
							for(ps=0;ps<polarnum;ps++){
								open(finpath+signam[sets]+"_"+chans[cs]+polinsert+polnames[ps]+".tif");
								t2=getTitle();
								shifts=imageReg(t1,t2,noisetol1[cs+in*colen],noisetol1[cs+in*colen],shiftol,"","",9);
								if (shifts[4]>=1){
									driftx[ps+cs*polarnum+in*colen*polarnum]=shifts[0];
									drifty[ps+cs*polarnum+in*colen*polarnum]=shifts[2];
									regcode[ps+cs*polarnum+in*colen*polarnum]=d2s(shifts[4],0);
								}
								else if (shifts[4]==0.5){
									Dialog.create("Registration check");
									if(polarnum>1)
										Dialog.addMessage("No matching peak-pairs were found to register channel: "+chans[cs]+"nm at angle: "+polnames[ps]+".\nOne peak was used instead.");
									else
										Dialog.addMessage("No matching peak-pairs were found to register channel: "+chans[cs]+"nm.\nOne peak was used instead.");
									Dialog.addMessage("You can enter a shift for this image manually below if desired.");
									Dialog.addNumber("Shift in x-direction:",shifts[0]);
									Dialog.addNumber("Shift in y-direction:",shifts[2]);
									Dialog.show();
									driftx[ps+cs*polarnum+in*colen*polarnum]=Dialog.getNumber();
									drifty[ps+cs*polarnum+in*colen*polarnum]=Dialog.getNumber();
									regcode[ps+cs*polarnum+in*colen*polarnum]=d2s(shifts[4],1);
								}
								else{
									Dialog.create("Registration Problem");
									if(polarnum>1)
										Dialog.addMessage("No points were found to register channel: "+chans[cs]+"nm at angle: "+polnames[ps]+".");
									else
										Dialog.addMessage("No points were found to register channel: "+chans[cs]+"nm.");
									Dialog.addMessage("You can enter a shift for this image manually below if desired.");
									Dialog.addNumber("Shift in x-direction:",0);
									Dialog.addNumber("Shift in y-direction:",0);
									Dialog.show();
									driftx[ps+cs*polarnum+in*colen*polarnum]=Dialog.getNumber();
									drifty[ps+cs*polarnum+in*colen*polarnum]=Dialog.getNumber();
									regcode[ps+cs*polarnum+in*colen*polarnum]=d2s(shifts[4],0);
								}
							}
						}
					}
					print("Pass #"+regloop+1+" Drift detected in x direction (pixels): ");
					Array.print(driftx);
					print("Pass #"+regloop+1+" Drift detected in y direction (pixels): ");
					Array.print(drifty);
					print("Pass #"+regloop+1+" Exit codes: ");
					Array.print(regcode);
					driftstrx="x1 = "+d2s(driftx[0],2);
					driftstry="y1 = "+d2s(drifty[0],2);
					excdstr="Ex 1 = "+regcode[0];
					for(i=1;i<driftx.length;i++){
						driftstrx=driftstrx+",  "+"x"+i+1+" = "+d2s(driftx[i],2);
						driftstry=driftstry+",  "+"y"+i+1+" = "+d2s(drifty[i],2);
						excdstr=excdstr+",  "+"Ex "+i+1+" = "+regcode[i];
					}
					repopts=newArray("Yes","No");
					Dialog.create("Registration Exam");
					Dialog.addMessage("Check Registration Shifts below for abnormalities. The order is\npolarizer angle, then channel, e.g. 400:(000-150), 500:(000-150), etc.\nNote the exit code of associated with a given channel/angle.\nThis value indicates the number of peak-pairs used to register the image.\n A value of 0.5 ndicates that 1 peak was used to register the image.\nA value of 0 indicates no peaks were used to register this channel/angle.\nIn this case, an option was presented to you to choose the shift for that channel/angle.\nWhen a large percentage of exit codes are <2 take the opportunity to rerun the registration\nwith a lower noise tolerance to try to find more peaks to register with.");
					Dialog.addMessage("Drift in x-direction:  "+driftstrx);
					Dialog.addMessage("Drift in y-direction:  "+driftstry);
					Dialog.addMessage("Exit codes : "+excdstr);
					Dialog.setInsets(5,20,0);
					Dialog.addChoice("Repeat Registration",repopts,repopts[1]);
					Dialog.show();
					repchoice=Dialog.getChoice;
					if(repchoice==repopts[0])
						repeat=true;
					else
						repeat=false;
					regloop++;
				}
			}
			
			strcnt=dualprint(driftx,parfile,"Drift detected in x direction (pixels):",1,strcnt);//38
			strcnt=dualprint(drifty,parfile,"Drift detected in y direction (pixels):",1,strcnt);//39
			//strcnt=dualprint(regcode,parfile,"Exit codes:",1,strcnt); //to be added in future release
		/////////////////////////////////////////////Begin particle and background identification
			if(aroute==amode[0]){
				if(File.exists(finpath+"InBand_Extinction.tif")){
					ftitle="InBand_Extinction.tif";
					open(finpath+ftitle);
				}
				else{
					open(mainimage);
					ftitle=maintitle;
				}
			}
			else if(aroute==amode[1]){
				if(findhow==signam[0]){
					if(File.exists(finpath+"InBand_Extinction.tif")){
						ftitle="InBand_Extinction.tif";
						open(finpath+ftitle);
					}
					else{
						open(mainimage);
						ftitle=maintitle;
					}
				}
				else{
					if(File.exists(finpath+"InBand_Darkfield.tif")){
						ftitle="InBand_Darkfield.tif";
						open(finpath+ftitle);
					}
					else{
						open(maindimage);
						ftitle=maindtitle;
					}
				}
			}
			else{
				if(File.exists(finpath+"InBand_Darkfield.tif")){
					ftitle="InBand_Darkfield.tif";
					open(finpath+ftitle);
				}
				else{
					open(maindimage);
					ftitle=maindtitle;
				}
			}
			if(shapename=="Oval")//(roicho==roiform[0])
				makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
			if(shapename=="Rectangle")//(roicho==roiform[1])
				makeRectangle(rectlocx,rectlocy,ovalwi,ovalhe);//updated to makeRectangle from makeOval in v4.1.3 on 17/04/2021

			//run("Find Maxima...", "noise=usrtol output=List exclude"); //deprecated as of v4.1.2 (see next line)
			run("Find Maxima...", "prominence="+usrtol+" strict exclude output=List"); //updated in v4.1.3 on 16/04/2021
			numresults=nResults;
			run("Select None");
			////////////////////////
			coords=coordRes(numresults,"X","Y");
			xprime=Array.slice(coords,0,coords.length/2);
			yprime=Array.slice(coords,coords.length/2,coords.length);
			
			if(refshift==true){
				coords=coordShift(numresults,xprime,yprime,refshiftx[mainidx],refshifty[mainidx]);
				xprimeref=Array.slice(coords,0,coords.length/2);
				yprimeref=Array.slice(coords,coords.length/2,coords.length);
				selectWindow(ftitle);
				if(shapename=="Oval")//(roicho==roiform[0])
					makeOval(rectlocx, rectlocy, ovalwi, ovalhe);
				if(shapename=="Rectangle")//(roicho==roiform[1])
					makeRectangle(rectlocx, rectlocy, ovalwi, ovalhe);
				
				//run("Find Maxima...", "noise=usrtol output=List exclude light"); //deprecated as of v4.1.2 (see next line)
				run("Find Maxima...", "prominence="+usrtol+" strict exclude light output=List"); //updated in v4.1.3 on 16/04/2021
				numresdark=nResults;
				coords=coordRes(numresdark,"X","Y");
				xdarkpeaks=Array.slice(coords,0,coords.length/2);
				ydarkpeaks=Array.slice(coords,coords.length/2,coords.length);
				
				xrootpeaks=Array.concat(xprime,xdarkpeaks);
				yrootpeaks=Array.concat(yprime,ydarkpeaks);
				selectWindow(ftitle);
				run("Select None");
				run("Clear Results");
				close(ftitle);
				
				xprime=Array.concat(xprime,xprimeref);
				yprime=Array.concat(yprime,yprimeref);
				totresults=numresults*2;
				dx=newArray(totresults);
				cx=newArray(totresults);
			}
			if(refshift==false){
				xrootpeaks=xprime;
				yrootpeaks=yprime;
				dx=newArray(numresults);
				cx=newArray(numresults);
				totresults=numresults;
			}
			rootplen=xrootpeaks.length;
			/////////////////////////////////Identify all maxima and run proximity filter
			dDR=newArray(numresults);
			spacing=usrspc*radius[mainidx];
			for(i=0;i<totresults;i++){
				for(j=0;j<totresults;j++){
					if(xprime[i]==xprime[j] && yprime[i]==yprime[j])
								cx[j]=1;
					else{
						proximity=sqrt(pow(xprime[j]-xprime[i],2)+pow(yprime[j]-yprime[i],2));
						if(proximity<spacing)
							cx[j]=0;
						else
							cx[j]=1;
					}
				}
				Array.getStatistics(cx,min,max,mean,stdDev);
				dx[i]=mean;
			}
			
			if(refshift==true){
				for(i=0;i<numresults;i++){
					if(dx[i]+dx[i+numresults]==2)
						dDR[i]=1;
					else
						dDR[i]=0;
				}
			}
			if(refshift==false){
				for(i=0;i<numresults;i++)
					dDR[i]=dx[i];
			}
			if(refshift==true){
				xprime=Array.slice(xprime,0,numresults);
				yprime=Array.slice(yprime,0,numresults);
			}
			proxfiltDR=arrayvalueindices(dDR,1);
			xDR=newArray(proxfiltDR.length);
			yDR=newArray(proxfiltDR.length);
			intcheckDR=newArray(proxfiltDR.length);
			meanfiltDR=newArray(proxfiltDR.length);
			
			print("Successfully Identified Maxima");
	//////////////////////////////////////////////////////// create new coordinate arrays filtering proximal cases.
			Proxfilter=proxfiltDR;
			if(refshift==false)
				sigmaquality=newArray(Proxfilter.length);
			for(i=0;i<Proxfilter.length;i++){
				showProgress(i+1,Proxfilter.length);
				xDR[i]=xprime[Proxfilter[i]];
				yDR[i]=yprime[Proxfilter[i]];
				if(refshift==false){
					ROI2=ROIcoord(radius[mainidx],radius2[mainidx]);
					sigmaquality[i]=sigcheck(xDR[i],yDR[i],ROI2,ftitle,Nsig);
				}
			}
			if(refshift==false){
				xDR=arrayrebuild(xDR,sigmaquality,true);
				yDR=arrayrebuild(yDR,sigmaquality,true);
				Proxfilter=arrayrebuild(Proxfilter,sigmaquality,true);
			}
			arrtoim(xDR,yDR,procdir,"PartCoord");
			
			print("Successfully completed promixity filtering");
	/////////////////////////////////////////////////////////// BG point identification
			//if(refshift==false)
			//newImage("BGCoord","32-bit",datapoints,2,1);
			//if(refshift==true)
				//newImage("BGCoord","32-bit",datapoints,2*2,1);
			bgcoord="BGCoord";
			newImage(bgcoord,"32-bit",2,datapoints,1);
			//saveAs("Tiff",procdir+"BGCoord");
			
			centerx=((ovalwi/2)+rectlocx);
			centery=((ovalhe/2)+rectlocy);
	
			kRadlimit=usrspc*radius[mainidx];
			xrad=xprime;
			yrad=yprime;
			c=newArray(numresults);
			AllResults=numresults;
			if(refshift==true){
				xrad=xrootpeaks;
				yrad=yrootpeaks;
				AllResults=rootplen;
				c=newArray(rootplen);
			}
			//open(procdir+bgcoord);
			//open(mainimage);
			
			print("Beginning background point identification...");
			j=0;
			BGdatacount=0;
			while(BGdatacount<datapoints){
				showProgress(BGdatacount, datapoints);
				bx=rectlocx+random()*ovalwi;
				by=rectlocy+random()*ovalhe;
				if(shapename=="Oval"){//(roicho==roiform[0]){
					bRad=sqrt(pow(bx-centerx,2)+pow(by-centery,2));
					btheta=atan2(centery-by,bx-centerx);
				}
				for(k=0;k<AllResults;k++){
					kRad1=sqrt(pow(xrad[k]-bx,2)+pow(yrad[k]-by,2));
					if(refshift==true)
						kRad2=sqrt(pow(xrad[k]-(bx-refshiftx[mainidx]),2)+pow(yrad[k]-(by-refshifty[mainidx]),2));
					else
						kRad2=sqrt(pow(xrad[k]-bx,2)+pow(yrad[k]-by,2));
						//sigmaquality=sigcheck(xrad[k],yrad[k],ROI2,maintitle,Nsig);
					if(kRad1<kRadlimit || kRad2<kRadlimit) // || sigmaquality==false){
						c[k]=0;
					else
						c[k]=1;
				}
				Array.getStatistics(c,min,max,mean,stdDev);
				d=mean;
				if(shapename=="Oval"){//(roicho==roiform[0]){
						//sigmaquality=sigcheck(xrad[k],yrad[k],ROI2,maintitle,Nsig);
						if(bRad>ellipseRad(ovalwi/2-radius2[mainidx],ovalhe/2-radius2[mainidx],btheta))// || sigmaquality==false){
							d=d;
						else
							d++;
				}
				if(shapename=="Rectangle"){//(roicho==roiform[1]){
						//sigmaquality=sigcheck(xrad[k],yrad[k],ROI2,maintitle,Nsig);
						if((bx<(rectlocx+radius2[mainidx]) || bx>(rectlocx+ovalwi-radius2[mainidx])) || (by<(rectlocy+radius2[mainidx]) || by>(rectlocy+ovalhe-radius2[mainidx])))// || sigmaquality==false){
							d=d;
						else
							d++;
				}
				selectWindow(bgcoord);
				if(d==2){
					setPixel(0,j,bx);//2*i,bx);
					setPixel(1,j,by);//2*i+1,by);
					BGdatacount++;
					j++;
				}
				if(d!=2){
					BGdatacount=BGdatacount;
					j=j;
				}
			}
			print("Background identification successful");
			selectWindow(bgcoord);
			saveAs("Tiff",procdir+bgcoord);
			close(bgcoord);
		}
//////////////////////////////////////////////////Setup some storage array/images
		phenom=newArray("Extinction","Scattering");
		sigcoord="PartCoord.tif";
		bgcoord="BGCoord.tif";
		tlabels=tablab(setstrt,setend,phenom,chans,polnames,tau,zeta,eta,aroute,amode,usescale);
		bgsaveti="bgdata";
		sigsaveti="sigdatafull";
///////////////////////////////////////////////////Measure data
		bgpoints=dataGrab(finpath,procdir,signam,bgcoord,bgsaveti,setstrt,setend,chans,polarnum,polnames,polinsert,aroute,amode,Array.concat(driftx,drifty), Array.concat(refshiftx,refshifty),areacal,Nsig,radius,radius2);///////measure background data
		sigpoints=dataGrab(finpath,procdir,signam,sigcoord,sigsaveti,setstrt,setend,chans,polarnum,polnames,polinsert,aroute,amode,Array.concat(driftx,drifty), Array.concat(refshiftx,refshifty),areacal,Nsig,radius,radius2);///////measure signal data
///////////////////////////////////////////////////Save all unfiltered data as .xls
		bgsaveti="bgdata.tif";
		sigsaveti="sigdatafull.tif";
		if(colen>1 && polarization==true)
			residx=mainidx;////////////////residx is an index related to which directory these data should be saved to based on whether or not data was polarization-resolved and whether it was multi-channel
		else
			residx=0;//////////////
		bgfulltab=imtoxls(procdir,bgcoord,bgsaveti,presdir,"Full_BG_Data",colen,polarnum,tlabels,aroute,amode,setstrt,setend); //presdirs[residx]-->presdir in v4.0.7
		sigfulltab=imtoxls(procdir,sigcoord,sigsaveti,presdir,"Unfiltered_Particle_Data",colen,polarnum,tlabels,aroute,amode,setstrt,setend); //presdirs[residx]-->presdir in v4.0.7
//////////////////////////////////////////////////Present histograms for polarization-averaged particle data (data will be gated to remove spurrious cases using the histograms)
		if(usetxt==false || txtexst==false){
			repeat=true;
			histcount=0;
			while(repeat==true){
				if(histcount==0)
					newbins=0;
				else
					newbins=newbins;
				if(findhow==signam[0]){
					sidx=0;
					units="(nm^2)";
					if(usescale==false)
						units="(unscaled)";
					subscript="ext";
				}
				else{
					if((tau==1 && zeta==1 && eta==1) || usescale==false)
						units="(unscaled)";
					else
						units="(nm^2)";
					if(aroute==amode[1])
						sidx=1;
					else if(aroute==amode[2])
						sidx=0;
					subscript="scat";
				}
				lims=imtostat(procdir,sigsaveti,mainidx*polarnum,polarnum,sidx);
				hist0=imtohistbg(procdir,sigsaveti,bgsaveti,mainidx,1,polarnum,colen,true,presdirs[residx],"Unfiltered"+signam[sidx]+"_"+chans[mainidx]+"nm","Unfiltered "+signam[sidx]+" cross-section at "+chans[mainidx]+"nm","Sigma_"+subscript+" "+units,"Occurrence",pcolor[mainidx],true,false,sidx,newbins);
				setBatchMode("exit and display");
				binchoice=newArray("Yes","No");
				Dialog.create("Histogram Display Options");
				Dialog.addMessage("The auto-binning assumes a Normal/Gaussian distribution.\nIf the unfiltered distribution does not resemble a Gaussian,\nthe binning may be too sparse/dense. You may choose to add your\nown binning below and redisplay the histogram. The default value\nis the current bin number.");
				Dialog.addChoice("Redisplay Histogram?",binchoice,binchoice[1]);
				Dialog.addNumber("New bin number to redisplay histogram= ",hist0);
				Dialog.show();
					
				repbin=Dialog.getChoice();
				newbins=Dialog.getNumber();
				histcount++;
				if(repbin==binchoice[0]){
					repeat=true;
					close();
					setBatchMode(true);
				}
				else
					repeat=false;
			}
			waitForUser("Before closing this dialogue, you can use the Plot Window zoom/scroll\nbuttons to examine the data more precisely. Once this dialogue is closed\nyour inputs will be requested and you cannot go back.");
			Dialog.create("Cross-sectional range filter");
			Dialog.addMessage("The histogram presents the distribution of sigma_"+subscript+" for the highest ranking spectral\nrange. Using the histogram, determine lower and upper limits to define a range of\ncross-sections you would like to accept. Once you enter these limits, the data will be\nfiltered first for the limits in this channel, and then by the color ranks you entered earlier.");
			Dialog.addNumber("Lower Limit",lims[0]);
			Dialog.addNumber("Upper Limit",lims[1]);
			Dialog.show();
	
			lowlim=Dialog.getNumber();
			uplim=Dialog.getNumber();
			close();
			strcnt=dualprint("Peak Finding identity:\t"+sidx,parfile,"",0,strcnt);//40
			strcnt=dualprint("Lower limit of acceptance range:\t"+lowlim,parfile,"",0,strcnt);//41
			strcnt=dualprint("Upper limit of acceptance range:\t"+uplim,parfile,"",0,strcnt);//42
		}
		////////////////////////////////////////////////////////////////////////filter the data
			setBatchMode(true);
			filtindx=finalfilter(procdir,sigsaveti,mainidx*polarnum,polarnum,lowlim,uplim,ranks,colen,polarnum,findhow,signam,aroute,amode,sidx);
		///////////////////////////////////////////////////////////////////using filtered data output histograms for polarization-averaged information, and individual histograms for each polarizer angle and spectral range
		setBatchMode(true);
		if(filtindx.length==1 && filtindx[0]==-1){
			selectWindow("Log");
			saveAs("Text", polbase+"Log.txt");
			exit("No particles match the range requirement you set. All user inputs saved to Log in base folder.");
		}
		else if(filtindx.length==1 && filtindx[0]==-2){
			selectWindow("Log");
			saveAs("Text", polbase+"Log.txt");
			exit("No particles match the color filter (rank) requirement you set. All user inputs saved to Log in base folder.");
		}
		else{
			filtsigxy="FilteredCoord";
			xyfilt=imfilter(procdir,sigcoord,filtindx,procdir,filtsigxy);//////////////////create new coordinate array using filter
			filtsigxy="FilteredCoord.tif";
		 	filtsig="SigDataFilt";
		 	if(gaussfit==true){////////////////////////////////////////////////////////////////////Use Gaussian fitting for sub-pixel location determination
			 	filtgauss="GaussCoord";
			 	gimname=signam[sidx]+"_"+chans[mainidx]+polinsert+polnames[0]+".tif";
			 	resrad=mag*chint[mainidx]/(2*NA*1000*pixs);
			 	gxy=useGauss(procdir,filtsigxy,finpath,gimname,procdir,filtgauss,resrad);/////////////////use Gaussian fitting and create new coordinate array from filtered one
			 	filtgauss=filtgauss+".tif";
				sigpoints=dataGrab(finpath,procdir,signam,filtgauss,filtsig,setstrt,setend,chans,polarnum,polnames,polinsert,aroute,amode,Array.concat(driftx,drifty), Array.concat(refshiftx,refshifty),areacal,Nsig,radius,radius2);///////create data images using filter and updated coordinates (these are new main resource)
				whichcoord=filtgauss;
			}
			else{////////////////////////////////////////////////////////////////Or do not use Gaussian fitting and simply filter previous data array
				filtsig=imfilter(procdir,sigsaveti,filtindx,procdir,filtsig); ///////////////////////////////create new data images using filter only with unaltered coordinates
				whichcoord=filtsigxy;
			}
			filtsig="SigDataFilt.tif";
			sigfulltab=imtoxls(procdir,whichcoord,filtsig,presdir,"Filtered_Particle_Data",colen,polarnum,tlabels,aroute,amode,setstrt,setend);//presdirs[residx]-->presdir in v4.0.7
			if(findhow==signam[0])
				labim=mainimage;
			else
				labim=maindimage;
			labelim(labim,finpath,procdir,filtsigxy,refshiftx,refshifty,mainidx);
			for(sets=setstrt;sets<setend;sets++){
				subscript=newArray("ext","scat");
				if(aroute!=amode[2])
					in=sets;
				else
					in=sets-1;
				if(sets==0){
					if (usescale==true)
						units=" (nm^2)";
					else
						units=" (unscaled)";
				}
				else{
					if((tau==1 && zeta==1 && eta==1) || usescale==false)
						units=" (unscaled)";
					else
						units=" (nm^2)";
				}
				for(cs=0;cs<colen;cs++){
					if(colen>1 && polarization==true)
						endirs=presdirs[cs];
					else
						endirs=presdirs[0];
					imtohistbg(procdir,filtsig,bgsaveti,cs,1,polarnum,colen,true,endirs,"PolarizationAveragedStats"+phenom[sets]+"_"+chans[cs]+"nm","Distribution of angle-averaged sigma_"+subscript[sets]+" at lambda="+chans[cs]+"nm","sigma_"+subscript[sets]+" "+units,"Occurrence",pcolor[cs],true,true,in,0);
				}
				if(polarization==true){
					for(cs=0;cs<colen;cs++){
						if(colen>1 && polarization==true)
							endirs=presdirs[cs];
						else
							endirs=presdirs[0];
						for(ps=0;ps<polarnum;ps++){
							imtohistbg(procdir,filtsig,bgsaveti,ps+cs*polarnum,1,polarnum,colen,false,endirs,"PolarizationResolvedStats_"+phenom[sets]+"_"+chans[cs]+polinsert+polnames[ps],"Distribution of sigma_"+subscript[sets]+" at lambda="+chans[cs]+"nm and "+"theta="+polnames[ps],"sigma_"+subscript[sets]+" "+units,"Occurrence",pcolor[cs],true,true,in,0);
						}
					}
				}
			}
			////////////////////////////////////////////////////////////////////Fit polarization data for each particle for each channel
			if(polarization==true){
				fitim(procdir,filtsig,"",setstrt,setend,colen,chans,polnums,signam,pcolor,true,fitdirs,"Rawfits",aroute,amode,fitmodel,1,subscript,usescale);
				fitim(procdir,filtsig,bgsaveti,setstrt,setend,colen,chans,polnums,signam,pcolor,false,arrfill(colen,""),"NoiseFits",aroute,amode,fitmodel,polfits,subscript,usescale);
				ptitles=ptablab(setstrt,setend,phenom,chans,1,polarnum,tau,zeta,eta,aroute,amode,false,usescale);
				noisetitles=ptablab(setstrt,setend,phenom,chans,polfits,polarnum,tau,zeta,eta,aroute,amode,true,usescale);
				rawfits="Rawfits.tif";
				noisefit="NoiseFits.tif";
				fithist(procdir,rawfits,setstrt,setend,chans,tau,zeta,eta,fitdirs,pcolor,subscript,usescale);
				noisehist(procdir,noisefit,setstrt,setend,chans,polfits,tau,zeta,eta,fitstats,pcolor,subscript,usescale);
				histhist(procdir,noisefit,setstrt,setend,chans,polfits,tau,zeta,eta,fitstats,pcolor,subscript,usescale);
				fimtoxls(procdir,rawfits,fitdir,colen,phenom,ptitles,1,aroute,amode,setstrt,setend,false,subscript); //modified in version v4.0.7  with additional argument "phenom" as in updated function and use fitdir instead of fitdir
				fimtoxls(procdir,noisefit,fitstats,colen,phenom,noisetitles,polfits,aroute,amode,setstrt,setend,true,subscript); //modified in version v4.0.7  with additional argument "phenom" as in updated function
			}
			//////////////////////////////////////////////////////////////////// Time series analysis for each particle for each channel, polarization, or experiment type
			if(timereg==true){
				timeGrab(finpath,procdir,signam,filtsigxy,setstrt,setend,chans,polnames,polinsert,aroute,amode,refshiftx,refshifty,areacal,Nsig,radius,radius2);
				flofitim(procdir,setstrt,setend,chans,polinsert,polnames,signam,pcolor,tfitdirs,"tfitpars",aroute,amode,flofitmodel,exptime);
				timtablabs=timtablab(polbase,setstrt,setend,signam,chans,polnames,tau,zeta,eta,aroute,amode,usescale,exptime);
				flofimtoxls(procdir,"tfitpars.tif",tfitdirs,chans,timtablabs,aroute,amode,setstrt,setend,polinsert,polnames);
			}
		}
	}
	////////////End Particle Analysis section
	setBatchMode("exit and display");
	if((usetxt==false || txtexst==false) && partanalysis==true){
		File.close(parfile);
		convertImToText("signal",procdir,sigcoord);
		convertImToText("filter",procdir,filtsigxy);
		convertImToText("BG",procdir,bgcoord);
	}
	//close();
	run("Clear Results");
	logchck=getInfo("log");
	if(logchck!=""){
		selectWindow("Log");
		saveAs("Text", polbase+"Log.txt");
		if(extanalysis==true && partanalysis==false)
		print("\\Clear");	
	}
	beep();
	if(partanalysis==true)
		showMessage("Operation Complete. Analysis log saved to base directory.");
	else
		showMessage("Operation Complete");
}
//////////////////////////////End Extinction Suite

function imageRegSolo(image1, image2, noisetol, shiftol, BGtype1, BGtype2,diam,ovx,ovy,ovwi,ovhe){ ///////////////////registration of single target particle. Registration functions to be merged later.
	width=getWidth();
	height=getHeight();
	if(width>=height)
		gdim=width;
	else
		gdim=height;
	fract=0.04;
	numres1=0;
	selectWindow(image1);
	recwidth=0;
	recheight=0;
	//while(numres1<3 || recwidth<width || recheight<height){
		//makeOval(0.5*gdim-0.5*fract*gdim,0.5*gdim-0.5*fract*gdim,fract*gdim,fract*gdim);
		//makeRectangle(0.5*width-0.5*fract*width,0.5*height-0.5*fract*height,fract*width,fract*height);
		//makeRectangle(0,0,width,height);
		makeOval(ovx,ovy,ovwi,ovhe);
		//run("Find Maxima...", "noise=noisetol output=List exclude"+BGtype1); //deprecated as of v4.1.2 (see next line)
		run("Find Maxima...", "prominence="+noisetol+" strict exclude "+BGtype1+" output=List"); //updated in v4.1.3 on 16/04/2021
		numres1=nResults;
		fract=fract+0.01;
		recwidth=fract*width;
		recheight=fract*height;
	//}
	if(numres1==1){
		x=getResult("X",0);
		y=getResult("Y",0);
		gauss=gaussCoordsimp(x,y,diam,image1);
		x1=newArray(1);
		y1=newArray(1);
		x1[0]=gauss[0];
		y1[0]=gauss[1];
	}
	else{
		x1=newArray(numres1);
		y1=newArray(numres1);
		for (i=0;i<numres1;i++){
			x1[i]=getResult("X",i);
			y1[i]=getResult("Y",i);
			gauss=gaussCoordsimp(x1[i],y1[i],diam,image1);
			x1[i]=gauss[0];
			y1[i]=gauss[1];
		}
	}
	fract=0.04;
	numres2=0;
	selectWindow(image2);
	recwidth=0;
	recheight=0;
	//while(numres2<3 || recwidth<width || recheight<height){
		//makeOval(0.5*gdim-0.5*fract*gdim,0.5*gdim-0.5*fract*gdim,fract*gdim,fract*gdim);
		//makeRectangle(0.5*width-0.5*fract*width,0.5*height-0.5*fract*height,fract*width,fract*height);
		//makeRectangle(0,0,width,height);
		makeOval(ovx,ovy,ovwi,ovhe);
		//run("Find Maxima...", "noise=noisetol output=List exclude"+BGtype2); //deprecated as of v4.1.2 (see next line)
		run("Find Maxima...", "prominence="+noisetol+" strict exclude "+BGtype2+" output=List"); //updated in v4.1.3 on 16/04/2021
		numres2=nResults;
		fract=fract+0.01;
		fract=fract+0.01;
		recwidth=fract*width;
		recheight=fract*height;
	//}
	if(numres2==1){
		x=getResult("X",0);
		y=getResult("Y",0);
		gauss=gaussCoordsimp(x,y,diam,image2);
		x2=newArray(1);
		y2=newArray(1);
		x2[0]=gauss[0];
		y2[0]=gauss[1];
	}
	else{
		x2=newArray(numres2);
		y2=newArray(numres2);
		for (i=0;i<numres2;i++){
			x2[i]=getResult("X",i);
			y2[i]=getResult("Y",i);
			gauss=gaussCoordsimp(x2[i],y2[i],diam,image2);
			x2[i]=gauss[0];
			y2[i]=gauss[1];
		}
	}
	run("Clear Results");
	//////////////Differences
	if(numres1>1 && numres2>1){
		xdif1=newArray(numres1*(numres1-1)/2);
		ydif1=newArray(numres1*(numres1-1)/2);
		xdif2=newArray(numres2*(numres2-1)/2);
		ydif2=newArray(numres2*(numres2-1)/2);
		xsign1=newArray(numres1*(numres1-1)/2);
		ysign1=newArray(numres1*(numres1-1)/2);
		xsign2=newArray(numres2*(numres2-1)/2);
		ysign2=newArray(numres2*(numres2-1)/2);
		
		for(i=0;i<2;i++){
			if(i==0){xs=x1; ys=y1; xdifs=xdif1; ydifs=ydif1; N=numres1; xsigns=xsign1; ysigns=ysign1;}
			if(i==1){xs=x2; ys=y2; xdifs=xdif2; ydifs=ydif2; N=numres2; xsigns=xsign2; ysigns=ysign2;}
			for(n=0;n<N-1;n++){
				for(j=0;j<N-1-n;j++){
					xdifs[j+n*(N-1)-n*n+(n*(n+1)/2)]=xs[n]-xs[n+1+j];
					ydifs[j+n*(N-1)-n*n+(n*(n+1)/2)]=ys[n]-ys[n+1+j];
					xsigns[j+n*(N-1)-n*n+(n*(n+1)/2)]=getSign(xs[n]-xs[n+1+j]);
					ysigns[j+n*(N-1)-n*n+(n*(n+1)/2)]=getSign(ys[n]-ys[n+1+j]);
					xdifs[j+n*(N-1)-n*n+(n*(n+1)/2)]=abs(xs[n]-xs[n+1+j]);
					ydifs[j+n*(N-1)-n*n+(n*(n+1)/2)]=abs(ys[n]-ys[n+1+j]);
				}
			}
		}
	}
	////////////////////////check difference matching
	if(numres1>1 && numres2>1){
		if(numres1<=numres2){
			NR1=numres1;
			NR2=numres2;
			xD1=xdif1;
			yD1=ydif1;
			xD2=xdif2;
			yD2=ydif2;
		}
		else{
			NR1=numres2;
			NR2=numres1;
			xD1=xdif2;
			yD1=ydif2;
			xD2=xdif1;
			yD2=ydif1;
		}
		indocr=newArray(NR1*(NR1-1)/2);
		for(i=0;i<NR1*(NR1-1)/2;i++){
			num=0;
			occur=0;
			while(occur==0 &&  num<(NR2*(NR2-1)/2)){
				if((xD2[num]-2*shiftol<=xD1[i] && xD2[num]+2*shiftol>=xD1[i]) && (yD2[num]-2*shiftol<=yD1[i] && yD2[num]+2*shiftol>=yD1[i]))
					occur++;
				num++;
			}
			if(i==0){
				checker=newArray(1);
				if(occur==0)
					checker[0]=-1;
				else
					checker[0]=num-1;
			}
			else{
				inter=newArray(1);
				if(occur==0)
					inter[0]=-1;
				else
					inter[0]=num-1;
				checker=Array.concat(checker,inter);
			}
			indocr[i]=occur;
		}
		matchindex1=arrayvalueindices(indocr,1);
		matchindex2=arraytrimmer(checker,-1);

		////////////////////create reference array indicating which initial (x,y) point from which to reference using differences.
		ref1=newArray(numres1*(numres1-1)/2);
		ref2=newArray(numres2*(numres2-1)/2);
		for(i=0;i<2;i++){
			if(i==0){N=numres1; refs=ref1;}
			if(i==1){N=numres2; refs=ref2;}
			for(n=0;n<N-1;n++)
				for(j=0;j<N-1-n;j++)
					refs[j+n*(N-1)-n*n+(n*(n+1)/2)]=n;
		}
		////////////////////////find midpoints of matching differences in both sets
		xmid1=newArray(matchindex1.length);
		ymid1=newArray(matchindex1.length);
		xmid2=newArray(matchindex1.length);
		ymid2=newArray(matchindex1.length);
		xreg1=newArray(matchindex1.length);
		yreg1=newArray(matchindex1.length);
		xreg2=newArray(matchindex1.length);
		yreg2=newArray(matchindex1.length);
	
		for(i=0;i<2;i++){
			if(i==0){
				xs=x1; ys=y1; xdifs=xdif1; ydifs=ydif1; xmids=xmid1; ymids=ymid1; refs=ref1; xsigns=xsign1; ysigns=ysign1; xregs=xreg1; yregs=yreg1;
				if(numres1<=numres2)
					match=matchindex1;
				else
					match=matchindex2;
			}
			if(i==1){
				xs=x2; ys=y2; xdifs=xdif2; ydifs=ydif2; xmids=xmid2; ymids=ymid2; refs=ref2; xsigns=xsign2; ysigns=ysign2; xregs=xreg2; yregs=yreg2;
				if(numres1<=numres2)
					match=matchindex2;
				else
					match=matchindex1;
			}
			for(n=0;n<matchindex1.length;n++){
				xmids[n]=xs[refs[match[n]]]-xsigns[match[n]]*xdifs[match[n]]/2;
				ymids[n]=ys[refs[match[n]]]-ysigns[match[n]]*ydifs[match[n]]/2;
				xregs[n]=xs[refs[match[n]]];
				yregs[n]=ys[refs[match[n]]];
			}
		}
		///////////////find shifts from matching midpoints
		xshifts=newArray(matchindex1.length);
		yshifts=newArray(matchindex1.length);
		for(i=0;i<matchindex1.length;i++){
			xshifts[i]=xmid1[i]-xmid2[i];
			yshifts[i]=ymid1[i]-ymid2[i];
		}
		Array.getStatistics(xmid1,min,max,xcom,comstdx);
		Array.getStatistics(ymid1,min,max,ycom,comstdy);
		Array.getStatistics(xshifts,min,max,mshiftx,stdx);
		Array.getStatistics(yshifts,min,max,mshifty,stdy);
	}
	else{
		mshiftx=x1[0]-x2[0];
		stdx=0;
		mshifty=y1[0]-y2[0];
		stdy=0;
		xcom=x1[0];
		ycom=y1[0];
		shiftarr=newArray(mshiftx,stdx,mshifty,stdy,xcom,ycom);
		return shiftarr;
	}
	/////////////determine if there are any abberations in the shift data after first pass
	if(stdx<=sqrt(3)*shiftol/sqrt(xshifts.length) || stdy<=sqrt(3)*shiftol/sqrt(xshifts.length)){
		shiftarr=newArray(mshiftx,stdx,mshifty,stdy,xcom,ycom);
		return shiftarr;
	}
	/////////////////////////////////////if there were abberations in the shift data, run second pass pattern recognition by looking at the relative midpoints
	if(stdx>sqrt(3)*shiftol/sqrt(xshifts.length) || stdy>sqrt(3)*shiftol/sqrt(xshifts.length)){
		repeat=true;
		numb=0;
		while(repeat==true && numb<xmid1.length-1){
			numg=0;
			midcheck=newArray(xmid1.length-numb);
			for(n=0;n<xmid1.length-numb-1;n++){
				xmdif1=xmid1[numb]-xmid1[numb+n+1];
				ymdif1=ymid1[numb]-ymid1[numb+n+1];
				xmdif2=xmid2[numb]-xmid2[numb+n+1];
				ymdif2=ymid2[numb]-ymid2[numb+n+1];
				if((xmdif2-sqrt(6)*shiftol<=xmdif1 && xmdif2+sqrt(6)*shiftol>=xmdif1) && (ymdif2-sqrt(6)*shiftol<=ymdif1 && ymdif2+sqrt(6)*shiftol>=ymdif1)){
					midcheck[n+1]=1;
					numg++;
				}
				else
					midcheck[n+1]=0;
			}
			midcheck[0]=1;
			numg=numg+1;
			nxref=arrayvalueindices(midcheck,1);
			nyref=arrayvalueindices(midcheck,1);
			nshiftx=newArray(nxref.length);
			nshifty=newArray(nyref.length);
			nxreg1=newArray(nxref.length);
			nyreg1=newArray(nyref.length);
			nxreg2=newArray(nxref.length);
			nyreg2=newArray(nyref.length);
			//xcom=0;
			//ycom=0;
			for(n=0;n<nxref.length;n++){
				//nshiftx[n]=xmid1[nxref[n]]-xmid2[nxref[n]];
				//nshifty[n]=ymid1[nyref[n]]-ymid2[nyref[n]];
				//xcom+=xmid1[nxref[n]];
				//ycom+=ymid1[nxref[n]];
				nshiftx[n]=xreg1[0]-xreg2[0];
				nshifty[n]=yreg1[0]-yreg1[0];
			}
			//xcom=xcom/nxref.length;
			//ycom=ycom/nxref.length;
			xcom=xreg1[0];
			ycom=yreg1[0];
			Array.getStatistics(nshiftx,min,max,mshiftx2,stdx2);
			Array.getStatistics(nshifty,min,max,mshifty2,stdy2);
			if(numb==xmid1.length-2){
				print("Low number of values used to register Image:"+image2+".  Consider adjusting shift tolerance.");
				shiftarr=newArray(mshiftx2,stdx2,mshifty2,stdy2,xcom,ycom);
				return shiftarr;
			}
			if(stdx2>sqrt(3)*shiftol/sqrt(nxref.length) || stdy2>sqrt(3)*shiftol/sqrt(nxref.length) || numg<round((0.5*(xmid1.length-numb-1)))){
				numb++;
			}
			else{
				repeat=false;
				shiftarr=newArray(mshiftx2,stdx2,mshifty2,stdy2,xcom,ycom);
				return shiftarr;
			}
		}
	}
}
////////////////////////////////////////////////Gaussian Simplified
function gaussCoordsimp(initialx,initialy,diam,imagename){
	diamint=isInt(diam);
	if(diamint==true){
		//num=rad*2+1;
		num=diam;
	}
	if(diamint==false){
		num=round(diam);
	}
	odd=isOdd(num);
	if(odd==true){
		startx=initialx-(num-1)/2;
		starty=initialy-(num-1)/2;
	}
	else{
		startx=initialx-num/2;
		starty=initialy-num/2;
	}
	//centerR=sqrt(pow(initialx,2)+pow(initialy,2));
	x=newArray(num);
	y=newArray(num);
	xI=newArray(num);
	yI=newArray(num);
	
	selectWindow(imagename);
	
	for(i=0;i<num;i++){
		xo=startx+i;
		yo=starty+i;
		xI[i]=getPixel(xo,initialy);
		yI[i]=getPixel(initialx,yo);
		x[i]=xo+0.5;
		y[i]=yo+0.5;
	}
			
	Fit.doFit("Gaussian",x,xI);
	xoffset=Fit.p(0);
	xamp=Fit.p(1);
	xcent=Fit.p(2);
	xwidth=Fit.p(3);
	rxfit=Fit.rSquared;
	
	Fit.doFit("Gaussian",y,yI);
	yoffset=Fit.p(0);
	yamp=Fit.p(1);
	ycent=Fit.p(2);
	ywidth=Fit.p(3);
	ryfit=Fit.rSquared;

	xy=newArray(xcent,ycent,xwidth,ywidth);
	return xy;
}
//////////////////////////////////////function to register and average time series data
function timeAvg(dir,savedir,radius,aname,tname,noisetol,ovx,ovy,ovwi,ovhe,avsav,ss,se){
	extnames=getFileList(dir);
	extlen=extnames.length;
	
	truncx=newArray(extlen-1);
	truncy=newArray(extlen-1);
	xcrops=newArray(extlen);
	ycrops=newArray(extlen);
	xshift=newArray(extlen);
	yshift=newArray(extlen);
	
	shiftx=newArray(extlen);
	shifty=newArray(extlen);
	dxs=newArray(extlen);
	dys=newArray(extlen);
	xs=newArray(extlen);
	ys=newArray(extlen);
	basexs=newArray(extlen);
	baseys=newArray(extlen);
	finalx=newArray(extlen);
	finaly=newArray(extlen);
	oddevex2=newArray(extlen);
	oddevey2=newArray(extlen);
	xdisttopeak2=newArray(extlen);
	ydisttopeak2=newArray(extlen);
	
	setBatchMode(true);
	
	for(i=0;i<extlen;i++){
		open(dir+extnames[i]);
		if(i==0){
			imwidth=getWidth();
			imheight=getHeight();
		}
		
		shiftregister=imageRegSolo(extnames[0],extnames[i],noisetol,1,"","",round(2*radius),ovx,ovy,ovwi,ovhe);
		shiftx[i]=shiftregister[0];
		shifty[i]=shiftregister[2];
		if(i==0){
			xcom=shiftregister[shiftregister.length-2];
			ycom=shiftregister[shiftregister.length-1];
			dxcom=xcom%1;
			dycom=ycom%1;
			////////////////////
			if(abs(1-dxcom)<abs(dxcom-0.5)){
				//print("true1a");
				pix=floor(radius);
				if(isOdd(pix)) pix=pix-1;
			}
			else if(abs(dxcom-0)<abs(0.5-dxcom)){
				//print("true2a");
				pix=floor(radius);
				if(isOdd(pix)) pix=pix-1;
			}
			else{
				//print("true3a");
				pix=floor(radius);
				if(isOdd(pix)) pix=pix;
				else pix=pix+1;
			}
			if(isOdd(pix)){
				//print("True01a");
				basex1=floor(xcom)-(pix-1)/2;
				topx1=floor(xcom)+(pix-1)/2;
			}
			else{
				//print("True02a");
				if(abs(1-dxcom)<abs(dxcom-0)){
					basex1=floor(xcom)+1-pix/2;
					topx1=floor(xcom)+pix/2;
				}
				else{
					basex1=floor(xcom)-pix/2;
					topx1=floor(xcom)-1+pix/2;
				}
			}
			//////////////////
			if(abs(1-dycom)<abs(dycom-0.5)){
				//print("true1b");
				pix=floor(radius);
				if(isOdd(pix)) pix=pix-1;
			}
			else if(abs(dycom-0)<abs(0.5-dycom)){
				//print("true2b");
				pix=floor(radius);
				if(isOdd(pix)) pix=pix-1;
			}
			else{
				//print("true3b");
				pix=floor(radius);
				if(isOdd(pix)) pix=pix;
				else pix=pix+1;
			}
			if(isOdd(pix)){
				//print("True01b");
				basey1=floor(ycom)-(pix-1)/2;
				topy1=floor(ycom)+(pix-1)/2;
			}
			else{
				//print("True02b");
				if(abs(1-dycom)<abs(dycom-0)){
					basey1=floor(ycom)+1-pix/2;
					topy1=floor(ycom)+pix/2;
				}
				else{
					basey1=floor(ycom)-pix/2;
					topy1=floor(ycom)-1+pix/2;
				}
			}
			
			oddevex1=isOdd(topx1-basex1+1);
			oddevey1=isOdd(topy1-basey1+1);
			xdisttopeak1=floor(xcom)-basex1+1;
			ydisttopeak1=floor(ycom)-basey1+1;
		}
		
		dxs[i]=(xcom-shiftx[i])%1;
		dys[i]=(ycom-shifty[i])%1;
		xs[i]=xcom-shiftx[i];
		ys[i]=ycom-shifty[i];
		
		if(abs(1-dxs[i])<abs(dxs[i]-0.5)){
			pix=floor(radius);
			if(isOdd(pix)) pix=pix-1;
		}
		else if(abs(dxs[i]-0)<abs(0.5-dxs[i])){
			pix=floor(radius);
			if(isOdd(pix)) pix=pix-1;
		}
		else{
			pix=floor(radius);
			if(isOdd(pix)) pix=pix;
			else pix=pix+1;
		}
		if(isOdd(pix)){
			basexs[i]=floor(xs[i])-(pix-1)/2;
			finalx[i]=floor(xs[i])+(pix-1)/2;
		}
		else{
			if(abs(1-dxs[i])<abs(dxs[i]-0)){
				basexs[i]=floor(xs[i])+1-pix/2;
				finalx[i]=floor(xs[i])+pix/2;
			}
			else{
				basexs[i]=floor(xs[i])-pix/2;
				finalx[i]=floor(xs[i])-1+pix/2;
			}
		}
		//////////////////
		if(abs(1-dys[i])<abs(dys[i]-0.5)){
			pix=floor(radius);
			if(isOdd(pix)) pix=pix-1;
		}
		else if(abs(dys[i]-0)<abs(0.5-dys[i])){
			pix=floor(radius);
			if(isOdd(pix)) pix=pix-1;
		}
		else{
			pix=floor(radius);
			if(isOdd(pix)) pix=pix;
			else pix=pix+1;
		}
		if(isOdd(pix)){
			baseys[i]=floor(ys[i])-(pix-1)/2;
			finaly[i]=floor(ys[i])+(pix-1)/2;
		}
		else{
			if(abs(1-dys[i])<abs(dys[i]-0)){
				baseys[i]=floor(ys[i])+1-pix/2;
				finaly[i]=floor(ys[i])+pix/2;
			}
			else{
				baseys[i]=floor(ys[i])-pix/2;
				finaly[i]=floor(ys[i])-1+pix/2;
			}
		}
		
		oddevex2[i]=isOdd(finalx[i]-basexs[i]+1);
		oddevey2[i]=isOdd(finaly[i]-baseys[i]+1);
		xdisttopeak2[i]=floor(xcom-shiftx[i])-basexs[i]+1;
		ydisttopeak2[i]=floor(ycom-shifty[i])-baseys[i]+1;
		
		if((oddevex1==true && oddevex2[i]==true) || (oddevex1==false && oddevex2[i]==false)) xshift[i]=-(basex1-basexs[i]);
		if((oddevex1==true && oddevex2[i]==false) || (oddevex1==false && oddevex2[i]==true)) xshift[i]=-(basex1-(basexs[i]-(xdisttopeak1-xdisttopeak2[i])));
		if((oddevey1==true && oddevey2[i]==true) || (oddevey1==false && oddevey2[i]==false)) yshift[i]=-(basey1-baseys[i]);
		if((oddevey1==true && oddevey2[i]==false) || (oddevey1==false && oddevey2[i]==true)) yshift[i]=-(basey1-(baseys[i]-(ydisttopeak1-ydisttopeak2[i])));
		if(i>0){
			close(extnames[i]);
		}
		showProgress(i,extlen);
	}
	close(extnames[0]);
	print("time x-shifts");
	Array.print(xshift);
	print("time y-shifts");
	Array.print(yshift);
	
	Array.getStatistics(xshift,min, max, mean, stdDev);
	xmin=round(min);
	xmax=round(max);
	Array.getStatistics(yshift,min,max,mean,stdDev);
	ymin=round(min);
	ymax=round(max);
	xcrop=imwidth-(abs(xmin)+abs(xmax));
	ycrop=imheight-(abs(ymin)+abs(ymax));
	if(xmin<0) xstart=-xmin;
	else xstart=0;
	if(ymin<0) ystart=-ymin;
	else ystart=0;
	
	//setBatchMode(false);
	for(i=0;i<extlen;i++){
		open(dir+extnames[i]);
		makeRectangle(xstart+xshift[i],ystart+yshift[i],xcrop,ycrop);
		run("Crop");
	}
	run("Images to Stack");
	//setBatchMode("exit and display");
	saveAs(savedir+tname+".tif");
	run("Z Project...", "start="+ss+" stop="+se+" projection=[Average Intensity]");
	if(avsav==true){
		saveAs(savedir+aname+".tif");
	}
	close();
	close();
	setBatchMode(false);
}

function timeSave(polbase,savedir,signam,setstrt,setend,chans,char1,polnames,polinsert,aroute,amode,radius,avsav){
	pnum=polnames.length;
	cnum=chans.length;
	setnum=setend-setstrt;
	for(sets=setstrt;sets<setend;sets++){
		if(aroute==amode[2])
			in=sets-1;
		else
			in=sets;
		for(cs=0;cs<cnum;cs++){
			for(ps=0;ps<pnum;ps++){
				indir=polbase+signam[in]+"_timepoints_"+chans[cs]+char1+polnames[ps]+"/";
				timename=signam[in]+"_timepoints_"+chans[cs]+char1+polnames[ps];
				avgname=signam[in]+"_"+chans[cs]+char1+polnames[ps];
				run("Image Sequence...", "dir="+indir+" sort");
				imageW=getWidth();
				imageH=getHeight();
				makeOval(0.125*imageW,0.125*imageH,0.75*imageW,0.75*imageH);
				waitForUser("Reformat Selection Oval to choose strong peak.\nUse Find Maxima to determine noise tolerance...");
				getSelectionBounds(rectlocx,rectlocy,ovalwi,ovalhe);
				close();
				Dialog.create("Time registration noise tolerance and averaging");
				Dialog.addNumber("Input noise tolerance for time registration",30);
				Dialog.addNumber("Average slices starting at slice # ",1);
				Dialog.addNumber("Average slices up to slice # ",10);
				Dialog.show();

				noisetol=Dialog.getNumber();
				ss=Dialog.getNumber();
				se=Dialog.getNumber();
				timeAvg(indir,savedir,radius[cs],avgname,timename,noisetol,rectlocx,rectlocy,ovalwi,ovalhe,avsav,ss,se);
			}
		}
	}
	return 0;
}
//////////////////////////////////////////////function to measure all particles at all timepoints
function timeGrab(impath,procdir,signam,coordim,setstrt,setend,chans,polnames,polinsert,aroute,amode,refshiftx,refshifty,areacal,Nsig,r1,r2){
	pnum=polnames.length;
	cnum=chans.length;
	open(procdir+coordim);
	cim=getTitle();
	npart=getHeight();
	if(refshiftx.length==1 && refshiftx[0]==0 && refshifty.length==1 && refshifty[0]==0)
		rshift=false;
	else
		rshift=true;
	xcor=newArray(npart);
	ycor=newArray(npart);
	for(i=0;i<npart;i++){
		xcor[i]=getPixel(0,i);
		ycor[i]=getPixel(1,i);
	}
	close(cim);
	setnum=setend-setstrt;

	for(sets=setstrt;sets<setend;sets++){
		if(aroute==amode[2])
			in=sets-1;
		else
			in=sets;

		for(cs=0;cs<cnum;cs++){
			ROI1=ROIcoord(0,r1[cs]);
			ROI2=ROIcoord(r1[cs],r2[cs]);
			for(ps=0;ps<pnum;ps++){
				//nidx=ps+cs*pnum+in*cnum*pnum;
				//open(impath+signam[in]+"_timepoints_"+chans[cs]+polinsert+polnames[ps]+".tif");
				open(impath+signam[in]+"_timepoints_"+chans[cs]+polinsert+polnames[ps]+".tif");
				sim=getTitle();
				nfram=nSlices();
				newimtitle=signam[in]+"_timedata_"+chans[cs]+polinsert+polnames[ps]+".tif";
				newImage(newimtitle,"32-bit",nfram,npart,1);
				saveim=getTitle();
				for(ns=0;ns<npart;ns++){
					for (ts=0;ts<nfram;ts++){
						selectWindow(sim);
						setSlice(ts+1);
						if(aroute!=amode[2]){
							if((rshift==false && sets==0) || sets==1) {
								crossex=DRext(xcor[ns],ycor[ns],ROI1,ROI2,sim,areacal,Nsig);
								if(sets==1)
									crossex=tau*zeta*crossex/eta;
							}
							if(rshift==true && sets==0)
								crossex=avgsinext(xcor[ns],ycor[ns],refshiftx[cs],refshifty[cs],ROI1,sim,areacal);
						}
						else{
							crossex=DRext(xcor[ns],ycor[ns],ROI1,ROI2,sim,areacal,Nsig);
							crossex=tau*zeta*crossex/eta;
						}
						selectWindow(saveim);
						setPixel(ts,ns,crossex);
					}
				}
				saveAs("Tiff",procdir+newimtitle);
				close(saveim);
				close(sim);
			}
		}
	}
	return 0;
}

///////////////////////////////////////////polarization fit
function flofit(fitmodel,xvals,yvals){
	Fit.doFit(fitmodel, xvals, yvals);
	afit=Fit.p(0);
	bfit=Fit.p(1);
	rfit=Fit.rSquared;
	fitvals=newArray(afit,bfit,rfit);
	return fitvals;
}
/////////////////////////////////////////Polarization fit plots
function floplot(fitmodel,xvals,yvals,lambda,pcolor,polinsert,polnam,signam,CS,pnum,units,savpath,savname,exptime){
	fitvals=flofit(fitmodel,xvals,yvals);
	flen=xvals.length;
	afit=fitvals[0];
	bfit=fitvals[1];
	rfit=fitvals[2];
	xfit=newArray(flen);
	yfit=newArray(flen);
	for(i=0;i<flen;i++){
		xfit[i]=i*exptime;
		yfit[i]=afit*exp(-xfit[i]/bfit);
	}
	Array.getStatistics(yfit, min, max, mean, stdDev);
	RMSEdifsq=newArray(xvals.length);
	for(i=0;i<xvals.length;i++)
		RMSEdifsq[i]=pow(yfit[i]-yvals[i],2);
	RMSEsum=arraySum(RMSEdifsq);
	RMSE=sqrt(RMSEsum/(xvals.length));
	Array.getStatistics(xvals,xmin,xmax,xmean,xstdev);
	Array.getStatistics(yvals,ymin,ymax,ymean,ystdev);
	Plot.create("SignalVsTime_"+lambda+"_"+polinsert+polnam+pnum, "Time (s)", "sigma_"+CS+units);
	Plot.setLimits(0, xmax, ymin, ymax+0.2*ymax);
	Plot.setFontSize(14,"bold");
	Plot.setLineWidth(2);
	Plot.setColor("gray");
	Plot.add("line",xfit, yfit);
	Plot.setColor(pcolor);
	Plot.setLineWidth(2);
	Plot.add("circles",xvals, yvals);
	Plot.setColor("black");
	Plot.setJustification("right");
	Plot.addText("a="+afit,0.99,0.075);
	Plot.addText("b="+bfit,0.99,0.130);
	Plot.addText("R^2="+rfit,0.99,0.185);
	Plot.addText("RMSE="+RMSE,0.99,0.240);
	Plot.show();
	saveAs("PNG",savpath+savname);
	close();
	fitvals=newArray(afit,bfit);
	return fitvals;
}

////////////////////////////////////////////////////function to run time fits and save to additional array resource
function flofitim(procdir,setstrt,setend,chans,polinsert,polnames,signam,pcolor,savdir,datsav,aroute,amode,flofitmodel,exptime){
	//if(plotfit==false)
		//tbg=istdy(procdir,bgdatim,"","",polarnum,colen,true,false);
	setnum=setend-setstrt;
	pnum=polnames.length;
	cnum=chans.length;
	open(procdir+signam[setstrt]+"_timedata_"+chans[0]+polinsert+polnames[0]+".tif");
	npart=getHeight();
	close();
	newImage(datsav,"32-bit",2,npart,pnum*cnum*setnum);
	t2=getTitle();
	subscript=newArray("ext","scat");
	for(sets=setstrt;sets<setend;sets++){
		if(aroute!=amode[2])
			in=sets;
		else
			in=sets-1;
		
		for(cs=0;cs<cnum;cs++){
			for(ps=0;ps<pnum;ps++){
				selectWindow(t2);
				setSlice(1+ps+cs*pnum+in*cnum*pnum);
				intitle=signam[in]+"_timedata_"+chans[cs]+polinsert+polnames[ps]+".tif";
				open(procdir+intitle);
				t1=getTitle();
				//npart=getHeight();
				nfram=getWidth();
				yvals=newArray(nfram);
				//bgs=newArray(nfram);
				
				//if(plotfit==false){
					//selectWindow(tbg);
					//setSlice(in+1);
				//}
				xvals=newArray(nfram);
				for(ns=0;ns<npart;ns++){
					selectWindow(t1);
					//for(ps=0;ps<polarnum;ps++)
					//	yvals[ps]=getPixel(ps+cs*polarnum,ns);
					for(ts=0;ts<nfram;ts++){
						yvals[ts]=getPixel(ts,ns);
						xvals[ts]=ts*exptime;
					}
					//if(plotfit==true){
						pname=buffer(npart,ns+1);
						psavname="Particle_"+pname+"_"+signam[in]+"_TimeDependence_at_"+chans[cs]+"nm"+polinsert+polnames[ps];
						finsav=savdir[ps+cs*pnum+sets*cnum*pnum];
						fitvals=floplot(flofitmodel,xvals,yvals,chans[cs],pcolor[cs],polinsert,polnames[ps],signam[sets],subscript[in],ns+1,units,finsav,psavname,exptime);
						selectWindow(t2);
						for(i=0;i<2;i++)
							setPixel(i,ns,fitvals[i]);
					//}
					//else{
					//	selectWindow(tbg);
					//	for(ps=0;ps<polarnum;p++)
					//		bgs[ps]=getPixel(ps+cs*polarnum,0);
					//	selectWindow(t2);
					//	for(j=0;j<polfits;j++){
					//		newvals=addnoise(polarnum,yvals,bgs);
					//		fitvals=polfit(fitmodel,polnums,newvals);
					//		for(i=0;i<3;i++){
					//			setPixel(j+i*polfits+cs*3*polfits,ns,fitvals[i]);
					//		}
					//	}
					//}
				}
				close(t1);
			}
		}
	}
	//if(plotfit==false)
	//	close(tbg);
	selectWindow(t2);
	saveAs("Tiff",procdir+datsav);
	close();
	return 0;
}
////////////////////////////////////////////////function to create table titles following nomenclature of phenomenon name(lambda,angle)
function timtablab(polbase,setstrt,setend,phenom,chans,polnames,tau,zeta,eta,aroute,amode,usescale,exptime){
	cnum=chans.length;
	pnum=polnames.length;
	if(pnum>1){
		polinsert="_"
		tpinsert=",";
	}
	else{
		polinsert="";
		tpinsert="";
	}
	setnum=setend-setstrt;
	for(i=setstrt;i<setend;i++){
		if(i==1 || (aroute==amode[2] && i==0)){
			if((tau==1 && zeta==1 && eta==1) || usescale==false)
				units=" (unscaled)";
			else
				units=" (nm^2)";
		}
		else if (usescale==false)
			units=" (unscaled)";
		else
			units=" (nm^2)";
		if(aroute==amode[2])
			in=i-1;
		else
			in=i;
		for(cs=0;cs<cnum;cs++){
			for(ps=0;ps<pnum;ps++){
				tlist=getFileList(polbase+signam[in]+"_timepoints_"+chans[cs]+polinsert+polnames[ps]);
				nfram=tlist.length;
				titles=newArray(pnum*cnum*setnum*nfram+2);
				for(ts=0;ts<tlist.length;ts++){
					times=ts*exptime;
					titles[ts+ps*(nfram+2)+cs*(nfram+2)*pnum+in*(nfram+2)*pnum*cnum]=phenom[i]+"("+units+")"+"at T="+times+" s, ";
				}
				titles[nfram+ps*nfram+cs*nfram*pnum+in*nfram*pnum*cnum]="Fit parameter, a";
				titles[nfram+1+ps*nfram+cs*nfram*pnum+in*nfram*pnum*cnum]="Fit parameter, b";
			}
		}
	}
	return titles;
}
//////////////////////function to write time data images to xls
function flofimtoxls(procdir,datim,savpath,chans,timtablabs,aroute,amode,setstrt,setend,polinsert,polnames){
	open(procdir+datim);
	tfit=getTitle();
	cnum=chans.length;
	pnum=polnames.length;
	savname="TimeFitData";
	open(procdir+signam[in]+"_timedata_"+chans[0]+polinsert+polnames[0]+".tif");
	npart=getHeight();
	close();
	Table.create(savname);
  	for(row=0;row<npart;row++){
  		Table.set("Data type",row,"Datapoint",savname);
  		Table.set("Datapoint index", row, row+1,savname);
  	}
  	Table.set("Data type",npart,"Mean",savname);
  	Table.set("Data type",npart+1,"Standard Deviation",savname);
	Table.update;
	
  	savname0=savname;
  	for(sets=setstrt;sets<setend;sets++){
  		if(aroute==amode[2])
  			in=sets-1;
  		else
  			in=sets;
		for(cs=0;cs<cnum;cs++){
			for(ps=0;ps<pnum;ps++){
  				inim=procdir+signam[in]+"_timedata_"+chans[cs]+polinsert+polnames[ps]+".tif";
  				open(inim);
				tdat=getTitle();
				npart=getHeight();
				nfram=getWidth();
				for (row=0; row<npart; row++) {
					selectWindow(tdat);
					for(ts=0;ts<nfram;ts++){
						datum=getPixel(ts,row);
						Table.set(timtablabs[ts+ps*(nfram+2)+cs*(nfram+2)*pnum+in*(nfram+2)*pnum*cnum],row,datum,savname);
						if(ts==nfram-1){
							selectWindow(tfit);
  							setSlice(1+ps+cs*pnum+in*cnum*pnum);
  							afit=getPixel(0,row);
  							bfit=getPixel(1,row);
							Table.set(timtablabs[nfram+ps*nfram+cs*nfram*pnum+in*nfram*pnum*cnum],row,afit,savname);
							Table.set(timtablabs[nfram+1+ps*nfram+cs*nfram*pnum+in*nfram*pnum*cnum],row,bfit,savname);
						}
					}
				}
				Table.update;
				Table.save(savpath[ps+cs*pnum+in*cnum*pnum]+savname+".xls");
				close(savname);
				close(tdat);
			}
		}
	}
	return 0;
}

function convertImToText(type,path,Im){
	setBatchMode(true); //start batchmode avoiding display of image
	open(path+Im); //open the image
	imti=getTitle(); //get its title
	numpts=getHeight(); // get the number of peaks as the height of the image in pixels
	if(type=="signal"){ //check the kind of data to be saved and adjust filename column heading
		txtti="PeakCoordinates.txt";
		indxti="Peak Index";
	}
	else if(type=="filter"){
		txtti="FilteredCoordinates.txt";
		indxti="Peak Index";
	}
	else{
		txtti="BGCoordinates.txt";
		indxti="BG point index";
	}
	txtfile=File.open(path+txtti); //create and open a text file to write the data to
	print(txtfile, indxti+"\t"+"X"+"\t"+"Y\n"); //print the column headers in the first line and start a new line
	selectWindow(imti); //select the image to get the data from
	for(ii=0;ii<numpts;ii++){ //run for loop over the number of point coordinates
		x=getPixel(0,ii); //the x coordinate of the ith point is the 1st column and ith row
		y=getPixel(1,ii); //the y coordinate of the ith point is the 2nd column and ith row
		print(txtfile,ii+1+"\t"+d2s(x,9)+"\t"+d2s(y,9)+"\n"); //print the point index and its coordinates in the appropriate columns in the text file and start a new line
	}
	File.close(txtfile); //close the text file when finished
	close(imti); //close the image
	setBatchMode(false);//end batch mode
}

function convertTextToIm(type,path,txtfil){
	setBatchMode(true); //start batchmode to avoid display of image
	txtstr=File.openAsString(path+txtfil); //open the text file as a single string
	lines=split(txtstr,"\n"); // split the string into an array of lines of text (separated by \n delimiter in text file)
	numpts=lines.length-1; // get the number of points, note that the number of lines is 1 more than points because of the header row
	if(type=="signal") //check the kind of data to be saved and adjust the image title accordingly
		imti="PartCoord";
	else if(type=="filter")
		imti="FilteredCoord";
	else
		imti="BGCoord";
	newImage(imti,"32-bit",2,numpts,1); //create image which will be filled with data from text file
	for(ii=0;ii<numpts;ii++){ //iterate over the point positions
		xstr=substring(lines[ii+1],indexOf(lines[ii+1],"\t")+1,lastIndexOf(lines[ii+1],"\t")); //get the x coordinate from the ith line
		ystr=substring(lines[ii+1],lastIndexOf(lines[ii+1],"\t")+1,lengthOf(lines[ii+1])); //get the y coordinate from the ith line
		setPixel(0,ii,d2s(xstr,9)); //set the pixel in the 0 column and ith row with the x coordinate
		setPixel(1,ii,d2s(ystr,9)); //set the pixel in the 1 column and the ith row with the y coordinate
	}
	saveAs("tiff",path+imti); //save the image
	close(); //close the image
	setBatchMode(false); //exit batchmode
}

function dualprint(value,textfile,fillr,arr,strcnt){ //this function prints to the log directly and to a text file with a specific structure for saving/reloading.
	if(arr){ // check if the variable to print is an array, arr=1 if not arr=0
		print(fillr); // print the pre string
		Array.print(value); // print the array
		if(textfile!=""){ //when the textfile is an empty string, do not print to any file, otherwise proceed
			numstr=value.length; // get the number of array elements
			newstr=d2s(strcnt+1,0)+") "+fillr+"\t"; // add a tab after the pre-string
			for(ii=0;ii<numstr;ii++){ //add each element to the string
				if(ii<numstr-1)
					newstr+=d2s(value[ii],9)+","; //add commas to all but the final values
				else
					newstr+=d2s(value[ii],9); //add the value but no comma in the last case
			}
			print(textfile,newstr+"\n"); //start a new line after completing print of the string
			strcnt=strcnt+1;
		}
	}
	else{ //if variable to print is a string, not an array
		print(value); //prints to log
		if(textfile!=""){ //when textfile is an empty string, do not print to any file, otherwise proceed
			print(textfile,strcnt+1+") "+value+"\n"); //prints to saved textfile
			strcnt=strcnt+1;
		}
	}
	return strcnt;
}